C!F77-INC************************************************************
C
C!DESCRIPTION: This routine contains most COMMON statements used in
C              other routines.
C
c !Input Parameters:  N/A
c !Output Parameters: N/A
c
C!REVISION HISTORY:
C $Log: COMMONS_cirrus.inc,v $
c 10/07/1999 fhliang
c Replaced "DATA" statements throughout with "inline initialization" in type
c declaration statements.  The latter is an ANSI extension permitted by ECS.
c
C Modified by Liqun Ma  Mar   1998
C updated prolog
C
C
C!TEAM-UNIQUE HEADER:
C
C!REFERENCES AND CREDITS
C
C    Written by
C
C    Dr. Allen Chu                          November, 1996
C    SSAI
C    NASA GSFC Code 913
C    Greenbelt, MD 20771
C    email: achu@climate.gsfc.nasa.gov
C    phone: 301-614-6237
C
C!DESIGN NOTES: most data and variables are passed with common
C               statements in this routine.
C
c!END****************************************************************

      INTEGER  Max_Frames_Per_Line, No_Lines_Per_Scan
      PARAMETER (Max_Frames_Per_Line=1500, No_Lines_Per_Scan=10)

      INTEGER  Max_Scan_Per_Granule
      PARAMETER (Max_Scan_Per_Granule = 300)

C---A parameter for whole image

      INTEGER    Max_Lines_Per_Img
      PARAMETER (Max_Lines_Per_Img = 3000)

C
C-----------------------------------------------------------------------
C---A parameter for flagging BAD or MISSING data

      REAL*4     BAD_FLAG
      PARAMETER (BAD_FLAG = -9999.)

C-----------------------------------------------------------------------
C---Arrays for whole images

      REAL*4     REFL_0P67_Img, REFL_1P24_Img, REFL_1P38_Img
      REAL*4     REFL_0P86_Img, REFL_0P94_Img, REFL_1P64_Img
      REAL*4     REFL_2P13_Img

      DIMENSION REFL_0P67_Img(Max_Frames_Per_Line, Max_Lines_Per_Img),
     1          REFL_1P24_Img(Max_Frames_Per_Line, Max_Lines_Per_Img),
     2          REFL_1P38_Img(Max_Frames_Per_Line, Max_Lines_Per_Img),
     3          REFL_0P86_Img(Max_Frames_Per_Line, Max_Lines_Per_Img),
     4          REFL_0P94_Img(Max_Frames_Per_Line, Max_Lines_Per_Img),
     5          REFL_1P64_Img(Max_Frames_Per_Line, Max_Lines_Per_Img),
     6          REFL_2P13_Img(Max_Frames_Per_Line, Max_Lines_Per_Img)

      COMMON /MODIS_REFL_Img/ REFL_0P67_Img, REFL_1P24_Img,
     1                        REFL_1P38_Img, REFL_1P64_Img,
     2                        REFL_0P86_Img, REFL_0P94_Img,
     3                        REFL_2P13_Img

C---Ancillary data from MODIS cube on the whole image basis,
C     including solar zenith angle (degree), view zenith angle, and
C     surface elevation (km).

      REAL*4    ZENITH_SOL_Img, ZENITH_VIEW_Img, AZIMUTH_REL_Img

      DIMENSION ZENITH_SOL_Img (Max_Frames_Per_Line,Max_Lines_Per_Img),
     2          ZENITH_VIEW_Img(Max_Frames_Per_Line,Max_Lines_Per_Img),
     3          AZIMUTH_REL_Img(Max_Frames_Per_Line,Max_Lines_Per_Img)
      COMMON /ANCILLARY_1_Img/ ZENITH_SOL_Img, ZENITH_VIEW_Img,
     1                         AZIMUTH_REL_Img

C---Ancillary data - pixel indicators, land_water (I_LAND_WATER),
C      and cloud (I_CLOUD) on the whole image basis.

      INTEGER   I_LAND_WATER_Img, I_CLOUD_Img

      DIMENSION
     2   I_LAND_WATER_Img (Max_Frames_Per_Line, Max_Lines_Per_Img),
     3   I_CLOUD_Img      (Max_Frames_Per_Line, Max_Lines_Per_Img)
      COMMON /ANCILLARY_2_Img/ I_LAND_WATER_Img, I_CLOUD_Img

C---An index to count the number of lines in an image (= 10 x # of Scans)

      INTEGER L_Index_Img, No_Of_Samples, No_Of_Lines
      COMMON /Line_Index/ L_Index_Img, No_Of_Samples, No_Of_Lines

C-----------------------------------------------------------------------
C Parameters and arrays used in correcting angular dependencies of
C      the 1.38-micron channel water vapor absorption.

C   N_GH2O is the number of geometrical factors used in the
C      calculation of 1.375-micron channel transmittances.

      INTEGER    N_GH2O
      PARAMETER (N_GH2O = 220)

      REAL*4    GH2O, T_1P38_0P50, T_1P38_0P55, T_1P38_0P60,
     @   T_1P38_0P65, T_1P38_0P70, T_1P38_0P75, T_1P38_0P80

      DIMENSION GH2O(N_GH2O), T_1P38_0P50(N_GH2O),
     @ T_1P38_0P55(N_GH2O),T_1P38_0P60(N_GH2O),T_1P38_0P65(N_GH2O),
     @ T_1P38_0P70(N_GH2O),T_1P38_0P75(N_GH2O),T_1P38_0P80(N_GH2O)

      COMMON /T_H2O_1P38/ GH2O, T_1P38_0P50, T_1P38_0P55, T_1P38_0P60,
     @             T_1P38_0P65, T_1P38_0P70, T_1P38_0P75, T_1P38_0P80

C-----------------------------------------------------------------------
C---Output cirrus reflectance image

      REAL*4    cirrus_refl
      DIMENSION cirrus_refl (Max_Frames_Per_Line, Max_Lines_Per_Img)
      COMMON    /OUTPUT_cirrus_refl/ cirrus_refl

C Set a parameter, cirrus_refl_min, such that if the derived cirrus
C       reflectance for a pixel is less than cirrus_refl_min, the pixel
C       is considered to be cirrus free.

      REAL*4     cirrus_refl_min
      PARAMETER (cirrus_refl_min = 0.0002)

C---Output contrail index image

      INTEGER   I_CONTRAIL
      DIMENSION I_CONTRAIL (Max_Frames_Per_Line, Max_Lines_Per_Img)
      COMMON    /OUTPUT_CONTRAIL_index/ I_CONTRAIL

C---Output array for QA flags

      INTEGER*4   I_QA_cirrus
      DIMENSION   I_QA_cirrus (Max_Frames_Per_Line, Max_Lines_Per_Img)
      COMMON      /OUTPUT_QA_cirrus/ I_QA_cirrus

C---Parameters for dividing large imaging arrays into 4 quarters and
C     using quarter images in our contrail detection algorithm. The
C     purpose is to reduce the memory usage for the contrail algorithm.

      INTEGER    Max_Frames_Per_Line_HF,  Max_Lines_Per_Img_HF
      PARAMETER (Max_Frames_Per_Line_HF = Max_Frames_Per_Line/2)
      PARAMETER (Max_Lines_Per_Img_HF   = Max_Lines_Per_Img/2)

C--------0---------0---------0---------0---------0---------0---------072

C---  Parameters for computing histograms and for estimating slopes
C         from scatter plots of sub-images.
C
C       N         = number of sub_image pixels, N = N_SUB_X * N_SUB_Y,
C       N_STEP    = number of steps used in generating the histogram of
C                   1.38-micron channel sub-image,
C       N_AVG_MAX = maximum number of points needed to compute
C                   averaged (X,Y) values and therefore the slope.

       INTEGER N, N_SUB_X, N_SUB_Y, N_STEP, N_TRANSM_TABLE, N_AVG_MAX
       parameter (N = 100000,  N_SUB_X = 300, N_SUB_Y = 300)
       parameter (N_STEP = 20, N_TRANSM_TABLE = 220, N_AVG_MAX = 100)

C---   For input image files

       REAL*4 DATA_FLOAT_138, DATA_FLOAT_124, DATA_FLOAT_164
       REAL*4 DATA_FLOAT_086, DATA_FLOAT_094, DATA_FLOAT_213
       REAL*4 DATA_FLOAT_066, DATA_FLOAT_1380,DATA_FLOAT_1381

       DIMENSION DATA_FLOAT_138(N), DATA_FLOAT_124(N), DATA_FLOAT_164(N)
       DIMENSION DATA_FLOAT_086(N), DATA_FLOAT_094(N), DATA_FLOAT_213(N)
       DIMENSION DATA_FLOAT_066(N), DATA_FLOAT_1380(N)
       DIMENSION DATA_FLOAT_1381(N)
       COMMON /SUB_IMG/ DATA_FLOAT_138, DATA_FLOAT_124, DATA_FLOAT_086,
     $                  DATA_FLOAT_094, DATA_FLOAT_066, DATA_FLOAT_164,
     $                  DATA_FLOAT_213, DATA_FLOAT_1380,DATA_FLOAT_1381

C---   For output image files

       REAL*4    REFL_CIRRUS,    TRANSM_CIRRUS
       dimension REFL_CIRRUS(N), TRANSM_CIRRUS(N)
       COMMON /DATA_REFL_TRANSM/ REFL_CIRRUS, TRANSM_CIRRUS

       REAL*4 EXP_FACTOR /1.0/

C---   For image file buffers

       REAL*4 buf_h_138,buf_h_124,buf_h_066,buf_h_164,buf_h_213
       REAL*4 buf_l_138,buf_l_124,buf_l_066,buf_l_164,buf_l_213

       dimension buf_h_138(n),buf_h_124(n),buf_h_066(n),buf_h_164(n),
     $           buf_h_213(n)
       dimension buf_l_138(n),buf_l_124(n),buf_l_066(n),buf_l_164(n),
     $           buf_l_213(n)
       COMMON /BUFF/buf_h_138,buf_h_124,buf_h_066,buf_h_164,buf_h_213
     $             ,buf_l_138,buf_l_124,buf_l_066,buf_l_164,buf_l_213

C---   Variables related to subroutine GET_SLOPE

       INTEGER   N_SUM,INDEX_OCELAND(5,7,3)
       REAL*4    Y1,X1,SLOPE,X_138
       REAL*4    SLOPE_IMG(5,7,3),SLOPE_MEAN_LAND(3),SLOPE_N_LAND(3)
       REAL*4    SLOPE_MEAN_OCEAN(3),SLOPE_N_OCEAN(3)
       dimension Y1(5),X1(5,3),SLOPE(4,3)
       dimension X_138(N_STEP + 1), N_SUM(N_STEP)
       COMMON /DIVIDE/  Y1, X1

       INTEGER N_SLOPE
       COMMON /SLOPES/ SLOPE, N_SLOPE
       COMMON /SUMM/   N_SUM

       REAL*4    SLOPE_BOX
       DIMENSION SLOPE_BOX(15,30,4)
       COMMON /SLOPE_BX/  SLOPE_BOX

       REAL*4 ZEPS /0.00001/

       REAL*4 R_138_INTERVAL_SLOPE /30.0/

C--- In this program, it was assumed that the input apparent
C          reflectances were  multiplied by 1000.
       REAL*4 REFL_TH_MIN_138 /2.0/
       REAL*4 REFL_TH_MAX_138


       REAL*4 PER_LOWER_124 /0.01/, PER_UPPER_124 /0.10/

       REAL*4 PER_LOWER_066 /0.01/, PER_UPPER_066 /0.10/

       INTEGER        I_HIST_PEAK, N_MIN_138, N_MAX_138
       COMMON /HISTO/ I_HIST_PEAK, N_MIN_138, N_MAX_138

C---   Variables related to water vapor transmittance

       REAL*4    VAPOR,        TRANSM_0p86,   TRANSM_0p905,
     $           TRANSM_0p935, TRANSM_0p94,   TRANSM_1p24,
     $           TRANSM_1p38,
     $           TRANSM_R_0p94, TRANSM_R_0p94_MES,
     $           TRANSM_R_0p94_CLOUDY, TRANSM_1p38_THRESHOLD

       dimension VAPOR(N_TRANSM_TABLE),
     $           TRANSM_0p86(N_TRANSM_TABLE),
     $           TRANSM_0p905(N_TRANSM_TABLE),
     $           TRANSM_0p935(N_TRANSM_TABLE),
     $           TRANSM_0p94(N_TRANSM_TABLE),
     $           TRANSM_1p24(N_TRANSM_TABLE),
     $           TRANSM_1p38(N_TRANSM_TABLE),
     $           TRANSM_R_0p94(N_TRANSM_TABLE),
     $           TRANSM_R_0p94_MES(N)

       COMMON /TRANSMM/ TRANSM_0p86, TRANSM_0p905, TRANSM_0p935,
     $                TRANSM_0p94,   TRANSM_1p24,  TRANSM_1p38,
     $                TRANSM_R_0p94, TRANSM_R_0p94_CLOUDY,
     $                TRANSM_1p38_THRESHOLD

C---   Variables used in subroutine CLEAR_OR_CIRRUS

       REAL*4    COEFF_FINAL
       dimension COEFF_FINAL(4,3)
       COMMON    /COEFFF/ COEFF_FINAL

       INTEGER    I_PROCESS_MODE
       parameter (I_PROCESS_MODE = 2)
