C-----------------------------------------------------------------------
C !F77-INC
C
C !Description: This program is to define parameters for populating
C               'Water_Vapor_Near_Infrared' array
C
c !Input Parameters:  N/A
c !Output Parameters: N/A
c
C !Revision History:
c 02/02/98 achu
c added prolog.
c
C !TEAM-UNIQUE HEADER:
C
C   This software was developed by the MODIS Atmosphere Science Team
C   for the National Aeronautics and Space Administration at
C   Goddard Space Flight Center.
C
C !REFERENCES and CREDITS:
C
C   Written by
C   Dr. Allen Chu                 10/17/97
C   Code 913
C   NASA Goddard Space Flight Center
C   Greenbelt, MD 20771
C   achu@climate.gsfc.nasa.gov
C
C !END
C-----------------------------------------------------------------------


      INTEGER   Lines_Per_Write
      CHARACTER*(*) SDS1_N,DTYPE1
      PARAMETER (SDS1_N='Water_Vapor_Correction_Factors',DTYPE1='INTEGER*2')
      PARAMETER (Lines_Per_Write=10)
      DOUBLE PRECISION SCALE1,OFFSET1
      PARAMETER (SCALE1=1000.0)
      PARAMETER (OFFSET1=0.0)
      REAL ZEPS
      PARAMETER(ZEPS=0.00001)

C
C PCF logical identifiers
C
c rhucek 02/11/98:  Modified PCF LUNs
      INTEGER  LRN_CORR1,LRN_CORR2,LRN_QC,LRN_L1B,LRN_Geo,LRN_CldMsk,
     1         LRN_MOD04,LRN_MOD05,LRN_MCFQC,LRN_QCMET
      PARAMETER (LRN_CORR1=410171,LRN_CORR2=410181,
     1           LRN_L1B=700002,LRN_Geo=600000,
     2           LRN_CldMsk=422500,LRN_MOD04=405000,LRN_MOD05=410000,
     3           LRN_QC=410002,LRN_MCFQC=410502,LRN_QCMET=410602)

C
C Declarations of PARAMETERS to hold Geolocation array Fill Values
C
      INTEGER*2 FV_SolAz,FV_SenAz,FV_SolZen,FV_SenZen,
     &          FV_Height,FV_Range
      REAL FV_Lat, FV_Lon
      PARAMETER ( FV_SolAz = -32767,  FV_SenAz = -32767,
     &            FV_SolZen = -32767, FV_SenZen = -32767,
     &            FV_Height = -32767, FV_Range = 0,
     &            FV_Lat = -999.0,    FV_Lon = -999.0 )

C
C Declaration of PARAMETERS to hold Fill Values of L1B and
C Geolocation parameter data in science code.
C
      REAL FV_L1B, FV_GEO
      PARAMETER ( FV_L1B = -999.0, FV_GEO = -999.0 )
C
      INTEGER N_VAPOR, N_REFL_SUR, N_TAU, N_ZENITH_SOL
      INTEGER N_ZENITH_VIEW, N_AZIMUTH_REL

      PARAMETER ( N_VAPOR         = 13 )
      PARAMETER ( N_REFL_SUR      = 8  )
      PARAMETER ( N_TAU           = 8  )
      PARAMETER ( N_ZENITH_SOL    = 10 )
      PARAMETER ( N_ZENITH_VIEW   = 7  )
      PARAMETER ( N_AZIMUTH_REL   = 19 )


C
C Aerosol correction related lookup tables and parameters
C
        INTEGER     Max_Frames_Per_Line,No_Lines_Per_Scan,
     $              Max_Scans_Per_Granule,
     $              Max_Grids_Per_Line,No_Grids_Per_Scan
        PARAMETER (Max_Frames_Per_Line=1500, No_Lines_Per_Scan=10)
        PARAMETER (Max_Scans_Per_Granule=300)
        PARAMETER (Max_Grids_Per_Line=150, No_Grids_Per_Scan=1)

        REAL REFL_0P86       (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       REFL_0P94       (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       RATIO_0P94_0P86 (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       ZENITH_SOL      (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       ZENITH_VIEW     (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       AZIMUTH_REL     (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       TAU_AER         (Max_Frames_Per_Line, No_Lines_Per_Scan)
C     $       TAU_AER         (Max_Grids_Per_Line, No_Grids_Per_Scan)

        REAL VAPOR_TRANS_ONLY(Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       VAPOR_AER       (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       REFL_SUR_0P86   (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       VAPOR_OLD       (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       VAPOR_CORRECTED_FACTOR (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       FACTOR          (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       FACTOR2         (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $       FACTOR_FINAL    (Max_Frames_Per_Line, No_Lines_Per_Scan)

       REAL Aerosol_OD(Max_Grids_Per_Line,No_Grids_Per_Scan)

        INTEGER   I_CLOUD    (Max_Frames_Per_Line, No_Lines_Per_Scan),
     $            I_LAND_WATER (Max_Frames_Per_Line, No_Lines_Per_Scan)


       COMMON /PARA_INPUT/ REFL_0P86,REFL_0P94,RATIO_0P94_0P86,
     $                     ZENITH_SOL,ZENITH_VIEW,AZIMUTH_REL,
     $                     TAU_AER,I_CLOUD,I_LAND_WATER,Aerosol_OD

       COMMON /PARA_OUT/ VAPOR_TRANS_ONLY,VAPOR_AER,VAPOR_OLD,
     $                   VAPOR_CORRECTED_FACTOR,FACTOR, FACTOR2,FACTOR_FINAL,
     $                   REFL_SUR_0P86

      REAL REFL_0P86_T(N_REFL_SUR, N_VAPOR, N_TAU,
     $       N_ZENITH_SOL, N_AZIMUTH_REL, N_ZENITH_VIEW)
      REAL RATIO_T_AER(N_VAPOR, N_REFL_SUR, N_TAU,
     $       N_ZENITH_SOL, N_AZIMUTH_REL, N_ZENITH_VIEW)

      COMMON /TABLES/ REFL_0P86_T, RATIO_T_AER
      Real ts, tv, tr, tf, tp,  tt,tf2
      integer izs, izr, izv, iref, ivp, itau,iref2

      COMMON/GET_INDEX/izs,izr,izv,iref,ivp,itau,iref2


C
C  Parameters for cloud mask arrays
C
      INTEGER Buf_cldmsk,Buf_cldmsk_QA
      PARAMETER(Buf_cldmsk=6,Buf_cldmsk_QA=10)
