C----------------------------------------------------------------------
C !F77-INC
C
C !DESCRIPTION: cirrus.inc include file containing type declarations and
C               parameters statements for product specific variables
C               and constants.
C
c !Input Parameters:  N/A
c !Output Parameters: N/A
c
C !REVISION HISTORY:  
C Modified by Liqun Ma   Mar 1998
C Updated LUN numbers and SDS names.
C $Log: cirrus.inc,v $
C
C !TEAM-UNIQUE HEADER: 
C
C   This software was developed by the MODIS Science Data Support Team
C   (SDST) for the National Aeronautics and Space Administration, 
C   Goddard Space Flight Center, under contract NAS5-32373.
C
C !REFERENCES and CREDITS:
C
C   Modified according to mod05.inc by
C
C   Dr. Allen Chu                     November, 1996
C   SSAI
C   NASA/GSFC code 913
C   Greenbelt, MD 21718
C   email: achu@dustdevil.gsfc.nasa.gov
C   phone: 301-286-4080
C
C !DESIGN NOTES:
C
C Internals:
C
C   Variables:
C      Glob_d*  Global metadata names 
C      SDS*_N   SDS metadata names
C      DTYPE*   SDS data types
C      Dim*_N   Dimension names
C      Attr*_N  Attribute names
C      LRN_*    File Logical Reference in PCF (Process Control File) 
C
C !END
C----------------------------------------------------------------------

C Global metadata, SDS names and data types
      CHARACTER*(*) Glob_d1,SDS1_N,SDS2_N,
     2               DTYPE1,DTYPE2
      INTEGER   Lines_Per_Write
      PARAMETER (Glob_d1='Number of Instrument Scans')
      PARAMETER (SDS1_N='Cirrus_Reflectance',DTYPE1='INTEGER*2')
      PARAMETER (SDS2_N='Cirrus_Reflectance_Flag',DTYPE2='INTEGER*1')
      PARAMETER (Lines_Per_Write=10)

C Dimension names & metadata
      CHARACTER*(*) Dim1_N,Dim2_N,Attr1_N,Attr2_N,Attr3_N,Attr4_N,
     &              Attr5_N,Attr6_N,Attr7_N
      PARAMETER (Dim1_N='Pixels Per Scan Line')
      PARAMETER (Dim2_N='Number of Scan Lines')
      PARAMETER (Attr1_N='sampling_factor',Attr2_N='scale_factor',
     &           Attr3_N='add_offset',Attr4_N='units',
     &           Attr5_N='valid_range',Attr6_N='long_name',
     &           Attr7_N='_FillValue')

C Scales & offsets
      DOUBLE PRECISION SCALE1,SCALE2,SCALE3,OFFSET1,OFFSET2,OFFSET3
      PARAMETER (SCALE1=1000.0,SCALE2=1.0,SCALE3=1.0)
      PARAMETER (OFFSET1=0.0,OFFSET2=0.0,OFFSET3=0.0)
C
C The followings are subject to changes due to cirrus algorithm
C------------------------------------------------------------------------
C PCF logical identifiers
      INTEGER  LRN_QC,     LRN_L1B,  LRN_Geo,  LRN_CldMsk,
     2         LRN_mod06cd,LRN_TRANS,LRN_MCFQC,LRN_QCMET,
     3         LRN_DIAG, LRN_MCF
      PARAMETER (LRN_L1B=700002,    LRN_Geo=600000,
     2           LRN_CldMsk=422500, LRN_mod06cd=412500,
     3           LRN_TRANS=400002,  LRN_QC=400000, 
     4           LRN_MCFQC=400004,  LRN_QCMET=400005,
     5           LRN_DIAG=10911,    LRN_MCF=412501)
C------------------------------------------------------------------------
C
C Declarations of PARAMETERS to hold Geolocation array Fill Values
      INTEGER*2 FV_SolAz,FV_SenAz,FV_SolZen,FV_SenZen,
     &          FV_Height,FV_Range
      REAL FV_Lat, FV_Lon
      PARAMETER ( FV_SolAz = -32767,  FV_SenAz = -32767,
     &            FV_SolZen = -32767, FV_SenZen = -32767,
     &            FV_Height = -32767, FV_Range = 0,
     &            FV_Lat = -999.9,    FV_Lon = -999.9 )

C Declaration of PARAMETERS to hold Fill Values of L1B and
C Geolocation parameter data in science code.
      REAL FV_L1B, FV_GEO
      PARAMETER ( FV_L1B = -999.0, FV_GEO = -999.0 )

C Declare PARAMETERS identifying first and last detectors for MODIS 
C solar reflectance bands
      INTEGER First_Detector_Band1,   First_Detector_Band2, 
     &        First_Detector_Band3,   First_Detector_Band4,
     &        First_Detector_Band5,   First_Detector_Band6,
     &        First_Detector_Band7,   First_Detector_Band8,
     &        First_Detector_Band9,   First_Detector_Band10,
     &        First_Detector_Band11,  First_Detector_Band12,
     &        First_Detector_Band13L, First_Detector_Band13H,
     &        First_Detector_Band14L, First_Detector_Band14H,
     &        First_Detector_Band15,  First_Detector_Band16,
     &        First_Detector_Band17,  First_Detector_Band18,
     &        First_Detector_Band19,  First_Detector_Band20,
     &        First_Detector_Band21,  First_Detector_Band22,
     &        First_Detector_Band23,  First_Detector_Band24,
     &        First_Detector_Band25,  First_Detector_Band26,
     &        First_Detector_Band27,  First_Detector_Band28,
     &        First_Detector_Band29,  First_Detector_Band30,
     &        First_Detector_Band31,  First_Detector_Band32,
     &        First_Detector_Band33,  First_Detector_Band34,
     &        First_Detector_Band35,  First_Detector_Band36
              
      PARAMETER (First_Detector_Band1=1,     First_Detector_Band2=41,
     &           First_Detector_Band3=81,    First_Detector_Band4=101,
     &           First_Detector_Band5=121,   First_Detector_Band6=141,
     &           First_Detector_Band7=161,   First_Detector_Band8=181,
     &           First_Detector_Band9=191,   First_Detector_Band10=201,
     &           First_Detector_Band11=211,  First_Detector_Band12=221,
     &           First_Detector_Band13L=231, First_Detector_Band13H=241,
     &           First_Detector_Band14L=251, First_Detector_Band14H=261,
     &           First_Detector_Band15=271,  First_Detector_Band16=281,
     &           First_Detector_Band17=291,  First_Detector_Band18=301,
     &           First_Detector_Band19=311,  First_Detector_Band20=321,
     &           First_Detector_Band21=331,  First_Detector_Band22=341,
     &           First_Detector_Band23=351,  First_Detector_Band24=361,
     &           First_Detector_Band25=371,  First_Detector_Band26=381,
     &           First_Detector_Band27=391,  First_Detector_Band28=401,
     &           First_Detector_Band29=411,  First_Detector_Band30=421,
     &           First_Detector_Band31=431,  First_Detector_Band32=441,
     &           First_Detector_Band33=451,  First_Detector_Band34=461,
     &           First_Detector_Band35=471,  First_Detector_Band36=481)

      INTEGER Last_Detector_Band1,   Last_Detector_Band2,
     &        Last_Detector_Band3,   Last_Detector_Band4,
     &        Last_Detector_Band5,   Last_Detector_Band6,
     &        Last_Detector_Band7,   Last_Detector_Band8,
     &        Last_Detector_Band9,   Last_Detector_Band10,
     &        Last_Detector_Band11,  Last_Detector_Band12,
     &        Last_Detector_Band13L, Last_Detector_Band13H,
     &        Last_Detector_Band14L, Last_Detector_Band14H,
     &        Last_Detector_Band15,  Last_Detector_Band16,
     &        Last_Detector_Band17,  Last_Detector_Band18,
     &        Last_Detector_Band19,  Last_Detector_Band20,
     &        Last_Detector_Band21,  Last_Detector_Band22,
     &        Last_Detector_Band23,  Last_Detector_Band24,
     &        Last_Detector_Band25,  Last_Detector_Band26,
     &        Last_Detector_Band27,  Last_Detector_Band28,
     &        Last_Detector_Band29,  Last_Detector_Band30,
     &        Last_Detector_Band31,  Last_Detector_Band32,
     &        Last_Detector_Band33,  Last_Detector_Band34,
     &        Last_Detector_Band35,  Last_Detector_Band36

      PARAMETER (Last_Detector_Band1=40,    Last_Detector_Band2=80,
     &           Last_Detector_Band3=100,   Last_Detector_Band4=120,
     &           Last_Detector_Band5=140,   Last_Detector_Band6=160,
     &           Last_Detector_Band7=180,   Last_Detector_Band8=190,
     &           Last_Detector_Band9=200,   Last_Detector_Band10=210,
     &           Last_Detector_Band11=220,  Last_Detector_Band12=230,
     &           Last_Detector_Band13L=240, Last_Detector_Band13H=250,
     &           Last_Detector_Band14L=260, Last_Detector_Band14H=270,
     &           Last_Detector_Band15=280,  Last_Detector_Band16=290,
     &           Last_Detector_Band17=300,  Last_Detector_Band18=310,
     &           Last_Detector_Band19=320,  Last_Detector_Band20=330,
     &           Last_Detector_Band21=340,  Last_Detector_Band22=350,
     &           Last_Detector_Band23=360,  Last_Detector_Band24=370,
     &           Last_Detector_Band25=380,  Last_Detector_Band26=390,
     &           Last_Detector_Band27=400,  Last_Detector_Band28=410,
     &           Last_Detector_Band29=420,  Last_Detector_Band30=430,
     &           Last_Detector_Band31=440,  Last_Detector_Band32=450,
     &           Last_Detector_Band33=460,  Last_Detector_Band34=470,
     &           Last_Detector_Band35=480,  Last_Detector_Band36=490)
