C----------------------------------------------------------------------
C !F77-INC
C
C !DESCRIPTION:  MOD05 include file containing type declarations and
C               parameters statements for product specific variables
C               and constants.
c
c !Input Parameters:  N/A
c !Output Parameters: N/A
C
C !REVISION HISTORY:  
C $Log: mod05.inc,v $
C Revision 1.3  1996/12/31  16:30:35  jguu
C The LUN for the MCF is given a process-specific value.
C
C Revision 1.2  1996/08/30  14:54:58  jguu
C The original file name for this file is mod05_inc.f. The logical
C unit numbers for the input and output files are updated.
C
c Revision 1.1  1996/07/08  18:32:08  vlin
c Initial revision
c
c Revision 1.1  1996/06/26  06:31:46  rhucek
c Initial revision
c
c Revision 2.1  1996/04/29  18:00:58  vlin
c updated to version 1
c
c Revision 1.4  1996/03/26  19:07:09  rhucek
c Prefixed parameters associated with PCF file logical reference
c numbers with LRN_.
c
c Revision 1.3  1995/11/06  13:23:10  vlin
c Put in PCF logical identifiers
c
c Revision 1.3  1995/11/06  13:20:01  vlin
c Put in PCF logical identifiers
c
c Revision 1.2  1995/11/01  15:24:38  vlin
c Before code walkthrough
c
c Revision 1.1  1995/10/18  17:18:27  vlin
c Initial revision
C
C !TEAM-UNIQUE HEADER: 
C
C   This software was developed by the MODIS Science Data Support Team
C   (SDST) for the National Aeronautics and Space Administration, 
C   Goddard Space Flight Center, under contract NAS5-32373.
C
C !REFERENCES and CREDITS:
C
C   Written by 
C   Vicky Lin                     10/18/95
C   Research and Data Systems Corporation
C   SAIC/GSC MODIS Science Data Support Office
C   7501 Forbes Blvd, Seabrook  MD 20706
C
C   vlin@ltpmail.gsfc.nasa.gov
C
C !DESIGN NOTES:
C
C Internals:
C
C   Variables:
C      Glob_d*  Global metadata names 
C      SDS*_N   SDS metadata names
C      DTYPE*   SDS data types
C      Dim*_N   Dimension names
C      Attr*_N  Attribute names
C      LRN_*    File Logical Reference in PCF (Process Control File) 
C
C !END
C----------------------------------------------------------------------

C Global metadata, SDS names and data types
C
      CHARACTER*(*) Glob_d1,SDS1_N,SDS2_N,SDS3_N,DTYPE1,DTYPE2,DTYPE3
      INTEGER   Lines_Per_Write
      PARAMETER (Glob_d1='Number of Instrument Scans')
C      PARAMETER (SDS1_N='Column_Water_Vapor',DTYPE1='INTEGER*2')
      PARAMETER (SDS1_N='Water_Vapor_Near_Infrared',DTYPE1='INTEGER*2')
      PARAMETER (SDS2_N='Cloud_Mask_QA',DTYPE2='INTEGER*1')
      PARAMETER (SDS3_N='Quality_Assurance_Near_Infrared',
     &           DTYPE3='INTEGER*1')
CDAC
      PARAMETER (Lines_Per_Write=10)

C Dimension names & metadata
      CHARACTER*(*) Dim1_N,Dim2_N,Attr1_N,Attr2_N,Attr3_N,Attr4_N,
     &              Attr5_N,Attr6_N,Attr7_N
      PARAMETER (Dim1_N='Pixels Per Scan Line')
      PARAMETER (Dim2_N='Number of Scan Lines')
      PARAMETER (Attr1_N='sampling_factor',Attr2_N='scale_factor',
     &           Attr3_N='add_offset',Attr4_N='units',
     &           Attr5_N='valid_range',Attr6_N='long_name',
     &           Attr7_N='_FillValue')

C Scales & offsets
      DOUBLE PRECISION SCALE1,SCALE2,SCALE3,OFFSET1,OFFSET2,OFFSET3
      PARAMETER (SCALE1=1000.0,SCALE2=1.0,SCALE3=1.0)
      PARAMETER (OFFSET1=0.0,OFFSET2=0.0,OFFSET3=0.0)

*/  CCR
*/  Modified by JC Guu  12/30/96
*/  The LUN for the MCF is given here.

C PCF logical identifiers
      INTEGER  LRN_TRANS1,LRN_TRANS2,LRN_TRANS3,LRN_TRANS4,LRN_TRANS5,
     2         LRN_TRANS6,LRN_Weight,LRN_QC,LRN_L1B,LRN_Geo,LRN_CldMsk,
     3         LRN_Anc,LRN_MOD05,LRN_MCF,LRN_MCFQC,LRN_QCMET,LRN_MOD07,
     &          LRN_WISC_ANC_met,LRN_WISC_ANC_ozn,
     &          LRN_WISC_ANC_ice,LRN_WISC_ANC_sst,INSCI
      PARAMETER (LRN_TRANS1=410101,LRN_TRANS2=410111,LRN_TRANS3=410121,
     2           LRN_TRANS4=410131,LRN_TRANS5=410141,LRN_TRANS6=410151,
     3           LRN_Weight=410161,LRN_L1B=700002,LRN_Geo=600000,
     4           LRN_CldMsk=422500,LRN_MOD05=410000,LRN_MOD07=420000,
     5           LRN_QC=410001,LRN_MCF=410500,LRN_MCFQC=410501,
     6           LRN_QCMET=410601,INSCI=407640,
     7           LRN_WISC_ANC_met=900000,LRN_WISC_ANC_ozn=900020,
     &           LRN_WISC_ANC_ice=900040,LRN_WISC_ANC_sst=900030)

C Declarations of PARAMETERS to hold Geolocation array Fill Values
      INTEGER*2 FV_SolAz,FV_SenAz,FV_SolZen,FV_SenZen,
     &          FV_Height,FV_Range
      REAL FV_Lat, FV_Lon
      PARAMETER ( FV_SolAz = -32767,  FV_SenAz = -32767,
     &            FV_SolZen = -32767, FV_SenZen = -32767,
     &            FV_Height = -32767, FV_Range = 0,
     &            FV_Lat = -999.0,    FV_Lon = -999.0 )

C Declaration of PARAMETERS to hold Fill Values of L1B and
C Geolocation parameter data in science code.
      REAL FV_L1B, FV_GEO
      PARAMETER ( FV_L1B = -999.0, FV_GEO = -999.0 )
C
C FOR QA array of Water_Vapor_Near_Infrared
C 
      INTEGER QA_BYTE
      PARAMETER (QA_BYTE=1)
c
c  Parameters for cloud mask array
C
      INTEGER Buf_cldmsk,Buf_cldmsk_QA
c
      PARAMETER(Buf_cldmsk=6,Buf_cldmsk_QA=10)

C Declare PARAMETERS identifying first and last detectors for MODIS 
C solar reflectance bands
      INTEGER First_Detector_Band1,   First_Detector_Band2, 
     &        First_Detector_Band3,   First_Detector_Band4,
     &        First_Detector_Band5,   First_Detector_Band6,
     &        First_Detector_Band7,   First_Detector_Band8,
     &        First_Detector_Band9,   First_Detector_Band10,
     &        First_Detector_Band11,  First_Detector_Band12,
     &        First_Detector_Band13L, First_Detector_Band13H,
     &        First_Detector_Band14L, First_Detector_Band14H,
     &        First_Detector_Band15,  First_Detector_Band16,
     &        First_Detector_Band17,  First_Detector_Band18,
     &        First_Detector_Band19,  First_Detector_Band26
              
      PARAMETER (First_Detector_Band1=1,     First_Detector_Band2=41,
     &           First_Detector_Band3=81,    First_Detector_Band4=101,
     &           First_Detector_Band5=121,   First_Detector_Band6=141,
     &           First_Detector_Band7=161,   First_Detector_Band8=181,
     &           First_Detector_Band9=191,   First_Detector_Band10=201,
     &           First_Detector_Band11=211,  First_Detector_Band12=221,
     &           First_Detector_Band13L=231, First_Detector_Band13H=241,
     &           First_Detector_Band14L=251, First_Detector_Band14H=261,
     &           First_Detector_Band15=271,  First_Detector_Band16=281,
     &           First_Detector_Band17=291,  First_Detector_Band18=301,
     &           First_Detector_Band19=311,  First_Detector_Band26=381)

      INTEGER Last_Detector_Band1,   Last_Detector_Band2,
     &        Last_Detector_Band3,   Last_Detector_Band4,
     &        Last_Detector_Band5,   Last_Detector_Band6,
     &        Last_Detector_Band7,   Last_Detector_Band8,
     &        Last_Detector_Band9,   Last_Detector_Band10,
     &        Last_Detector_Band11,  Last_Detector_Band12,
     &        Last_Detector_Band13L, Last_Detector_Band13H,
     &        Last_Detector_Band14L, Last_Detector_Band14H,
     &        Last_Detector_Band15,  Last_Detector_Band16,
     &        Last_Detector_Band17,  Last_Detector_Band18,
     &        Last_Detector_Band19,  Last_Detector_Band26

      PARAMETER (Last_Detector_Band1=40,    Last_Detector_Band2=80,
     &           Last_Detector_Band3=100,   Last_Detector_Band4=120,
     &           Last_Detector_Band5=140,   Last_Detector_Band6=160,
     &           Last_Detector_Band7=180,   Last_Detector_Band8=190,
     &           Last_Detector_Band9=200,   Last_Detector_Band10=210,
     &           Last_Detector_Band11=220,  Last_Detector_Band12=230,
     &           Last_Detector_Band13L=240, Last_Detector_Band13H=250,
     &           Last_Detector_Band14L=260, Last_Detector_Band14H=270,
     &           Last_Detector_Band15=280,  Last_Detector_Band16=290,
     &           Last_Detector_Band17=300,  Last_Detector_Band18=310,
     &           Last_Detector_Band19=320,  Last_Detector_Band26=390)
