      SUBROUTINE PROCESS_Land(HANDLE_LUT486,HANDLE_LUT663,
     &HANDLE_LUTPHS,IMONTH,ISCAN,IDATA,NUMSQ,MTHET0,MTHET,MDPHI,
     &MHGHT,LAT_CENTER,LON_CENTER,START_500,END_500,START_250,END_250,
     &START_1KM,END_1KM,W470_SYN,W213_SYN,W375_SYN,W659_SYN,
     &W865_SYN,CLDMSK_250,SET_COUNTER_LAND,QA_Flag_Land,
     &Success_Ret_Land,Fail_Ret_Land,
     &SDSLAT,SDSLON,SDS_MTHET0,SDS_MTHET,SDS_MPHI,SDS_Tau_Land_Ocean,
     &CLDMSK_500,SDS_Aerosol_Type,
     &SDS_SCAT_ANGLE_land,SDS_mass_conc_land,SDS_angs_coeff_land,
     &SDS_CLDFRC_land,SDS_dust_weighting,SDSTAU_continantal,
     &SDS_est_uncer,SDS_RefF_land,SDS_TranF_land,SDS_NUMPIXELS_land,
     &SDSTAU_corrected,
     &SDS_ref_land,SDS_ref_STD_land,SDS_QCONTROL_land,
     * SDS_Mean_Reflectance_Land_All, SDS_SDev_Reflectance_Land_All,
     * SDS_Path_Radiance_Land, SDS_Critical_Reflectance_Land,
     * SDS_Error_Crit_Reflectance_Land,SDS_Error_Path_Radiance_Land,
     * SDS_QFlag_Critical_Ref_Land,SDS_QFlag_Path_Radiance_Land,
     * SDS_QCONTROL_CritRef_land,G_factor,quality_land,Ret_Quality_cirrus,
     *cloud_num_land)

C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C
C   The program is to derive aerosol optical thickness from MODIS
c   measured radiances over land. The optical thicknesses are derived
C   at 0.47 and 0.66 micron channels using  2.13 or 3.95 micron
C   mid-IR channels to identify dark pixels and estimate their surface
C   reflectances. The optical thickness derived are based on the
C   average of apparent reflectance and surface reflectance within
C   10 x 10 km (at nadir) between the lowest 10th and 40th percentile
C   of the apparent reflectance. It will derive aerosol optical
C   thickness for cloud and water free pixels greater than 40 (10% of
C   the total 400 pixels of 10x10 km area). The derived aerosol optical
C   thickness is from continental model. It is then corrected by
C   the phase function and single scattering albedo according to the
C   dynamic aerosol models from a series of SCAR experiments.
C
c   The program assumes that the input data are in reflectance units
C   of PI*L/(F0*cos(theta0)) for visible and mid-IR channels. It also
C   assumes that the data in 3.75 micron channel are already corrected
C   for the emissive part. The input data are on MODIS spatial
C   resolutions with pixel size of the 0.47 um channel is half of 0.66
C   um and 2.1 um and quarter of 3.95 um channels.
C
C   Note: the program uses input cloud mask file (cloudy=0,clear=1),
C   which is determined by the thresholds of visible reflectance and
C   the visible reflectance ratios.  It requires that for 0.47 micron
C   channel 75% of the subpixels (250 m resolution) to be cloud free
C   and 100% of the subpixels to be water free. For 0.66 micron channel
C   it requires 100% subpixels to be cloud and water free. The selection
C   of 10th to 40th percentile will eliminate the residual clouds.
C
C!INPUT PARAMETERS:
C
C   HANDLE_LUT486        Logical unit number to open lookup table at 0.466 micron
C   HANDLE_LUT663        Logical unit number to open lookup table at 0.670 micron
C   HANDLE_LUTPHS        Logical unit number to open lookup table of phase function
C   IMONTH               Calendar month
C   ISACN                Scan number
C   IDATA                Index of 10x10 km box
C   NUMSQ                Total number of 10x10 km boxes
C   MTHET0               Solar zenith angle
C   MTHET                viewing angle
C   MDPHI                Relative azimuth angle
C   LAT_CENTER           Center latitude of 10x10 km box
C   LON_CENTER           Center longitude of 10x10 km box
C   START_500            Starting index for 500 m resolution array
C   END_500              Ending index for 500 m resolution array
C   START_250            Starting index for 250 m resolution array
C   END_250              Ending index for 250 m resolution array
C   START_1KM            Starting index for 1 km resolution array
C   END_1KM              Ending index for 1 km resolution array
C   W470_SYN             L1B data at 0.47 micron
C   W213_SYN             L1B data at 2.13 micron
C   W375_SYN             L1B data at 2.75 micron
C   W659_SYN             L1B data at 0.66 micron
C   W865_SYN             L1B data at 0.86 micron
C   CLDMSK_250           Cloud mask in 250 m resolution
C   SET_COUNTER_LAND     Counter of land
C
C!OUTPUT PARAMETERS:
C
C   QA_Flag_Land         QA flag araay for aerosol over land
C   Success_Ret_Land     Number of successfully retrieved value
C   Fail_Ret_Land        Number of failed retrieved value
C   SDSLAT               HDF SDS array for latitude
C   SDSLON               HDF SDS array for longitude
C   SDS_MTHET0           HDF SDS array for solar zenith angle
C   SDS_MTHET            HDF SDS array for viewing angle
C   SDS_MPHI             HDF SDS array for relative azimuth
C   SDS_Aerosol_Type     HDF SDS array for aerosol type
C   SDS_SCAT_ANGLE_land  HDF SDS array for scattering angle
C   SDS_mass_conc_lan    HDF SDS array for mass concentration
C   SDS_angs_coeff_land  HDF SDS array for angstrom coefficient
C   SDS_CLDFRC_land      HDF SDS array for number of cloudy pixels
C   SDS_dust_weighting   HDF SDS array for dust weighting factor
C   SDSTAU_continantal   HDF SDS array for continental aerosol optical depth
C   SDS_est_uncer        HDF SDS array for estimated uncertainties
C   SDS_RefF_land        HDF SDS array for reflected flux
C   SDS_TranF_land       HDF SDS array for transmitted flux
C   SDS_NUMPIXELS_land   HDF SDS array for number of pixels used in retrieval
C   SDSTAU_corrected     HDF SDS array for corrected aerosol optical depth
C   SDS_ref_land         HDF SDS array for mean apparent reflectance
C   SDS_ref_STD_land     HDF SDS array for std of apparent reflectance
C   SDS_QCONTROL_land    HDF SDS array for quality control
C
C!REVISION HISTORY:
C $Log: Process_land_V2.f,v $
c 10/08/1999 fhliang
c removed multiple specification of data type for variable weight.
c
c 02/02/1998 achu
c identified input/output arguments in INPUT / OUTPUT PARAMETERS sections
c in prolog.
c
c 01/29/1998 fhliang
c fixed prolog.
c
c Revision 1.5  1995/11/30  00:59:25  rhucek
c defined ISULP and ISMK in subroutine OUTPUT prolog
c
c Revision 1.4  1995/11/30  00:46:49  rhucek
c Added/completed prologs for subroutines INTERP, LOCATE, MOMENT,
c OPTICAL, OUTPUT, READTM
c
c Revision 1.3  1995/11/29  16:06:14  vlin
c Updated prolog (line 19)
c
c Revision 1.2  1995/11/17  12:53:51  vlin
c Before code walkthrough
c
c Revision 1.1  1995/10/17  12:59:52  rhucek
c Initial revision
c
c!TEAM-UNIQUE HEADER:
c
c Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
c GSFC, Greenbelt, MD
c
C
C!DESIGN NOTES:
C
C The following ERROR CODE indicates no retrieval possibly done:
C
C      1     Solar, satellite zenith angle and relative azimuth angles
C            are out of bound of look-up tables
C      2     Threshold for 3.95 um is not met in the entire grid box
C      3     The thresold for number of cloudfree points is not met.
C            (NUMCLDFREE see the parameter) or there is water.
C      4     The computed reflectance for 0.47 or 0.66 channels is out
C            of bounds from look-up table
C      5     Threshold for 2.13 um is not met in option 2 and 3
C      6     Aerosol type can not be determined
C
C The values of aerosol optical thickness corresponding to error code
C Error code       Value of aerosol optical thickness
C
C      1              -3.0
C      2              -4.0
C      3              -5.0
C      4              -6.0
C      5              -7.0
C      6              -8.0
C
C Aerosol type determined:
C
C      0              Continental
C      1              Dust
C      2              Sulfate
C      3              Smoke
C
C Procedure used in retrieving aerosol optical thickness:
C
C      0 = no dark targets possibly met by the following thresholds
C      1 = threshold used for wave 2.13 um 0.01 - 0.05
C      2 = threshold used for wave 3.95 um
C      3 = threshold used for wave 2.13 um 0.01 - 0.10
C      4 = threshold used for wave 2.13 um 0.01 - 0.15
C      5 = threshold used for wave 2.13 um 0.15 - 0.25
C
C INTERNALS:
C
C   Subroutines:
C      RLOOKUP       -  READS THE LOOK-UP TABLES
C      SELECT_LOOKUP -  SELECT LOOK-UP TABLE
C      STORE_DATA    -  INPUTS DATA. NOW IS SET FOR THE SIMULATED DATA SET
C      SUBOPT1       -  Derives aerosol optical thickness for 3.7 micron channel
C      SUBOPT2       -  Derives aerosol optical thickness for 2.1 micron channel
C      OPTION1       -  Finds dark targets using 3.95 micron channel
C      OPTION2       -  Finds dark targets using 2.1 micron channel
C      SORT          -  Sorts an array into ascending order
C      SHLSRT_LAND   -  Sorts an array into ascending order
C      INTERP_LAND   -  Linear interpolation
C      LOCATE        -  Obtains index for a given value
C      PERCENTILE    -  Computes average value/standard deviation of selected
C                       pixels given percentiles
C      OPTICAL       -  Computes the optical thickness from look-up tables
C      LOCATE_OPT    -  Obtain index for a given value (used only in OPTICAL)
C      LININT_OPT    -  Linear interpolation (used only in OPTICAL)
C      ERROR         -  SETS THE ERROR CODES
C      OUTPUT        -  WRITES THE OUTPUT TO HDF FILE
C
C   Variables:
C     W375_SYN(IGRIDX,IGRIDY)        Reflectance for wav=3.95um
C     W470_SYN(2*IGRIDX,2*IGRIDY)    Reflectance for wav=0.47um
C     W213_SYN(2*IGRIDX,2*IGRIDY)    Reflectance for wav=2.13um
C     W659_SYN(4*IGRIDX,4*IGRIDY)    Reflectance for wav=0.66um
C     CLDMSK_250(4*ISWATH,4*ILINE)   Cloud mask of 250 m resolution
C                                    (0=cloudy,1=clear)
C     MTHET0                         solar zenith angle (degree)
C     MTHET                          satellite viewangle angle (degree)
C     MDPHI                          relative azimuth in angle (degree)
C     MHGHT                          Topographic altitude (km)
C
C LAND SDS_ARRAYS.........
C
C    SDS_QCONTROL_land      Quality control SDS array
C    SDS_Aerosol_Type       Index of Aerosol type
C    SDS_SCAT_ANGLE_land    Scattering Angle
C    SDS_angs_coeff_land    Angstrom exponent for 0.47 and 0.67 micron
C    SDS_CLDFRC_land        Cloud fraction (%)
C    SDS_dust_weighting     Dust aerosol weighting factor
C    SDSTAU_continantal     Continental optical thickness at 0.47, and 0.66 micron
C    SDS_est_uncer          Uncertainty of optical thickness at 0.47 and 0.66 micron
C    SDS_RefF_land          Normalized reflected flux at 0.47 and 0.66 microns
C    SDS_TranF_land         Normalized Transmitted flux at 0.47 and 0.66 microns
C    SDS_NUMPIXELS_land     Number of pixels with desired percentile
C    SDSTAU_corrected       Corrected optical thickness at 0.47 0.55 and 0.66 micron
C    SDS_ref_land           Mean reflectance at five bands
C    SDS_ref_STD_land       Standard deviation of reflectance at five bands
C    SDS_mass_conc_land     Mass concentration
C
C!END
C----------------------------------------------------------------------
C  Parameters for the resolution of modis data

      IMPLICIT NONE
      SAVE

      INCLUDE 'mod04.inc'
      INCLUDE 'mapi.inc'
C
C Define SDS array for hdf output
C
      INTEGER *2   SDS_MTHET0(NUMCELLS),SDS_MTHET(NUMCELLS),
     &             SDS_MPHI(NUMCELLS),
     &             SDS_Tau_Land_Ocean(NUMCELLS)
            REAL   SDSLAT(NUMCELLS),SDSLON(NUMCELLS),
     &             SDS_mass_conc_land(NUMCELLS)
C
C  LAND SDS_ARRAYS........
C
      BYTE         SDS_QCONTROL_land(QA_LAND,NUMCELLS)
      BYTE         SDS_QCONTROL_CritRef_land(QA_LAND,NUMCELLS)
      INTEGER *2   SDS_Aerosol_Type(NUMCELLS),
     &             SDS_SCAT_ANGLE_land(NUMCELLS),
     &             SDS_angs_coeff_land(NUMCELLS),
     &             SDS_CLDFRC_land(NUMCELLS),
     &             SDS_dust_weighting(NUMCELLS),
     &             SDSTAU_continantal(NUMCELLS,Land_Sol1),
     &             SDS_est_uncer(NUMCELLS,Land_Sol1),
     &             SDS_RefF_land(NUMCELLS,Land_Sol2),
     &             SDS_TranF_land(NUMCELLS,Land_Sol1),
     &             SDS_NUMPIXELS_land(NUMCELLS,Land_Sol1),
     &             SDSTAU_corrected(NUMCELLS,Land_Sol2),
     &             SDS_ref_land(NUMCELLS,Band_land),
     &             SDS_ref_STD_land(NUMCELLS,Band_land)
C
C EXTRA LAND SDS_ARRAYS..........FOR LAND Statistics ONLY
C
       INTEGER *2  SDS_Mean_Reflectance_Land_All(NUMCELLS,Land_Sol3),
     &SDS_SDev_Reflectance_Land_All(NUMCELLS,Land_Sol3),
     &SDS_Path_Radiance_Land(NUMCELLS,Land_Sol1),
     &  SDS_Critical_Reflectance_Land(NUMCELLS,Land_Sol1),
     &  SDS_Error_Crit_Reflectance_Land(NUMCELLS,Land_Sol1),
     &  SDS_Error_Path_Radiance_Land(NUMCELLS,Land_Sol1),
     &  SDS_QFlag_Critical_Ref_Land(NUMCELLS,Land_Sol1),
     &  SDS_QFlag_Path_Radiance_Land(NUMCELLS,Land_Sol1)
      INTEGER QA_Flag_Land(19),Success_Ret_Land,Fail_Ret_Land
      INTEGER NUMSQDIM,NUMXBOX,NUMYBOX
      PARAMETER (NUMSQDIM=400)
      INTEGER RTN,START_DATA,END_DATA
      CHARACTER*100 fname
      CHARACTER*20 att_n,dtype
      INTEGER NUMMODEL,NUMTAU,NUMSCATT,LINES
      PARAMETER (NUMMODEL=4,NUMTAU=12,NUMSCATT=12,LINES=100)
      INTEGER NUMRED(4*IGRIDX,4*IGRIDY),NUMBLUE(2*IGRIDX,2*IGRIDY),nms
      INTEGER IBLUE,ICLDBLUE,ICLDRED,IDATA,ISCAN,IT
      INTEGER IERROR,IRD,IRED,NUMDATA,NUMSCAN,IWEIGHT
      INTEGER Ret_Quality_cirrus 
      INTEGER IFINISH,IMONTH,IPR,IPROCE,IRPHSOM,ISMK,ISULP,LINE,NUMSQ,ISMK_H,ISMK_M
      INTEGER HANDLE_HDF,IAER,MODFIL_MOD04(MODFILLEN),
     &        MODFIL_Geo(MODFILLEN),MODFIL_L1B(MODFILLEN),
     &        MODFIL_Cldmsk(MODFILLEN),HANDLE_TM(5),
     &        MODFIL_MOD05(MODFILLEN),MODFIL_MOD07(MODFILLEN),
     &        MODFIL_ANC(MODFILLEN)
      INTEGER Sea_landFSQ(NUMSQDIM,IGRIDX,IGRIDY),cloud_num_land
      INTEGER Sea_landF(IGRIDX,IGRIDY),Set_Counter_Land
      INTEGER START_500,END_500,START_250,END_250,START_1KM,END_1KM
      INTEGER CLDMSKD3(NUMSQDIM,4*IGRIDX,4*IGRIDY)
      INTEGER CLDMSK_250(4*ISWATH,4*ILINE)
      INTEGER CLDMSK_500(2*ISWATH,2*ILINE)
      REAL PCONTINTALB,PCONTINTALR,PDUSTSMKB,PDUSTSMKR
      REAL PDUSTSULB,PDUSTSULR,TAUABLUEC,TAUAREDC,WEIGHT,SCAT_ANGLE_LAND
      REAL W375_SYN(ISWATH,ILINE),W470_SYN(2*ISWATH,2*ILINE),
     *     W213_SYN(2*ISWATH,2*ILINE),W865_SYN(4*ISWATH,4*ILINE),
     *     W659_SYN(4*ISWATH,4*ILINE)
      REAL BLUE(2*IGRIDX,2*IGRIDY),RED(4*IGRIDX,4*IGRIDY),
     *     R0P86(4*IGRIDX,4*IGRIDY),L0P86((4*IGRIDX)*(4*IGRIDY)),
     *     RHOBLUE(2*IGRIDX,2*IGRIDY),RHORED(4*IGRIDX,4*IGRIDY),
     *     LBLUE((2*IGRIDX)*(2*IGRIDY)),LRED((4*IGRIDX)*(4*IGRIDY)),
     *     RBLUE((2*IGRIDX)*(2*IGRIDY)),RRED((4*IGRIDX)*(4*IGRIDY))
      REAL LATLON(2,NUMSQDIM),ANGLE(3,NUMSQDIM),MTHET0,MTHET,MDPHI,MHGHT
      REAL BARLBLUE,BARRBLUE,BARRRED,BARLRED,BARL0P86,TAUABLUE,TAUARED
      REAL ARRAY1((4*IGRIDX)*(4*IGRIDY)),ARRAY2((4*IGRIDX)*(4*IGRIDY))
      REAL BARRBLUEN,BARRBLUENN
      REAL BARRREDN,BARRREDNN
      REAL DTAUBLUE,DTAURED,SDBLUE,SDRED,SD0P86,TAUABLUEN
      REAL TAUABLUENN,TAUAREDN,TAUAREDNN
      REAL OMRED(NUMSCATT,NUMTAU,NUMMODEL)
      REAL OMBLUE(NUMSCATT,NUMTAU,NUMMODEL)
      REAL PHSRED(NUMSCATT,NUMTAU,NUMMODEL)
      REAL PHSBLUE(NUMSCATT,NUMTAU,NUMMODEL)
      REAL RATIO(NUMSCATT,NUMTAU,NUMMODEL)
      REAL TAUMODEL(NUMTAU),SCATT(NUMSCATT),SAVERATIO
      REAL LATM,LONM,LAT_CENTER,LON_CENTER
      REAL WATER_VAPOR(NUMSQDIM),OZONE_COL(NUMSQDIM)
      CHARACTER * 5 IOFLAG

C*********************************************************************
C   Parameters used from look-up tables
C ********************************************************************
C   wav=wavelengths(2)(.466,.670 micrometers)                2
C   lthet0=number of theta0(solar zenith angle)              9
C   thet0=(0,12,24,36,48,54,60,66,72   degrees)
C   lthe =number of the(observation zenith angle)           13
C   the=(0,6,.....72 degrees every 6 degrees)
C   lphi =number of phi(observation azimuth angle)          31
C   phi=(0,6,12,18,24,30,36,42,48,54,60,66,72,78,84,90,96,
C        102,108,114,120,126,132,138,144,150,156,162,168,
C        174,180)
C   ltau =number of opth(optical thickness)                  8
C   opth=(0.0,0.25,0.50,0.75,1.0,2.0,3.0,5.0)
C   lhght = number of hght(observation heights)              1
C   hght=(80.0 km)
C**********************************************************************
      INTEGER  NUMWAV,LTHET0,LTHE,LPHI,LTAU,NTABLE
      PARAMETER (NUMWAV=2,LTHET0=9,LTHE=13,LPHI=31,LTAU=7,NTABLE=5)

      REAL SBAR_A(LTAU,NUMWAV,NTABLE),OPTH_A(LTAU,NUMWAV,NTABLE)
      REAL FDOWN_B(LTAU,LTHET0,NUMWAV,NTABLE)
      REAL FDOWN_A(LTHET0,LTAU,NUMWAV,NTABLE)
      REAL FUP_B(LTAU,LTHET0,NUMWAV,NTABLE)
      REAL FUP_A(LTHET0,LTAU,NUMWAV,NTABLE)
      REAL FLUXDN(NUMWAV),FLUXUP(NUMWAV)
      REAL FLUXDN0P47(NTABLE),FLUXDN0P66(NTABLE)
      REAL FLUXUP0P47(NTABLE),FLUXUP0P66(NTABLE)
      REAL FLUXDNBLUE,FLUXDNRED,FLUXUPBLUE,FLUXUPRED
      REAL INT_B(LPHI,LTHE,LTAU,LTHET0,NUMWAV,NTABLE)
      REAL INT_A(LTHET0,LPHI,LTHE,LTAU,NUMWAV,NTABLE)
      REAL PIT_A(LTHE,LTAU,NUMWAV,NTABLE)

      REAL WAV(NUMWAV),THET0(LTHET0),THE(LTHE),PHI(LPHI)
      REAL SBAR(LTAU,NUMWAV),OPTH(LTAU,NUMWAV)
      REAL FDOWN(LTHET0,LTAU,NUMWAV),FUP(LTHET0,LTAU,NUMWAV)
      REAL INT(LTHET0,LPHI,LTHE,LTAU,NUMWAV)
      REAL PIT(LTHE,LTAU,NUMWAV),MASSCON,MASSFAT(NTABLE)
C      REAL FDOWN(LTAU,LTHET0,NUMWAV)
C      REAL INT(LPHI,LTHE,LTAU,LTHET0,NUMWAV)

      REAL NFDOWN(LTAU),NFUP(LTAU),NEWINT(LPHI,LTHE,LTAU),FINT(LTHE)
      REAL NEW(LPHI),NEWINT2(LTHE,LTAU),IMES(LTAU),WAVOPT(LTAU)
      REAL NEWN(LPHI),NEWN2(LPHI),FT(LTAU),Rayleigh_look(NUMWAV)
      integer ij,ik 

C
C The followings scalars are defined in mod04.inc
C
C MINMTHET0,MAXMTHET0,MINMTHET,MAXMTHET,MINMPHI,MAXMPHI,MAXTAU
C
       REAL THR375,THR21300,THR21305,THR21310,THR21315,THR21320,
     *      THR21325,THR21330,THR21335,THR21340,THR213MIN,THR213MAX,
     *      THR213GF,THR213GF1,THR213GF2,G_factor,DEGRAD
Change
       INTEGER NUMCLDFREE,NUMPERBLUE,NUMPERRED,NumHandles
       PARAMETER(THR375 =0.025,THR21300=0.01,THR21305=0.05,
     * THR21310=0.10,THR21315=0.15,THR21320=0.20,THR21325=0.25,
Change 2/20/2001
     * THR21330=0.30,THR21335=0.35,THR21340=0.40,
C     * NUMCLDFREE=40,NUMPERBLUE=10,NUMPERRED=10)
     * NUMCLDFREE=20,NUMPERBLUE=7,NUMPERRED=7)
C
C Variables used for ECS core meta deta
C
      CHARACTER*(PGSd_MET_GROUP_NAME_L-1) groups(PGSd_MET_NUM_OF_GROUPS)
      CHARACTER*(MAX_ECS_NAME_L-1) HDFAttNms(PGSd_MET_NUM_OF_GROUPS)
c 9 lines added by SDST vlin    08/29/96
      INTEGER file_version,pgs_pc_getreference,i

C
C Percentiles used for the radiances averaged 10th and 40th percentile
C

      INTEGER NLOWBLUE,NUPBLUE,NLOWRED,NUPRED
C      PARAMETER (NLOWBLUE=10,NUPBLUE=40,NLOWRED=10,NUPRED=40)
      PARAMETER (NLOWBLUE=20,NUPBLUE=50,NLOWRED=20,NUPRED=50)
C
C Option 1: surface reflectance at 0.47 and 0.66 micron is defined
C           using 3.75 micro channel
C
      REAL SURBLUE,SURRED
      PARAMETER(SURBLUE =0.01,SURRED =0.02)
C
C Option 2: surface reflectance at 0.47 and 0.66 micron is defined
C           as ref(0.47)=ref(2.13)/divblue and ref(0.66)=ref(2.13)/divred
C
      REAL DIVBLUE,DIVRED
      PARAMETER (DIVBLUE = 4.0, DIVRED  = 2.0)

C
C Method 2: variables specifically used by method 2
C

      INTEGER ITABLE,JTABLE,MTABLE,BTABLE,STABLE,IMETHOD,quality_land
      REAL I0P47(NTABLE),I0P66(NTABLE),IRET(NTABLE),
     *    TAUA0P47(NTABLE),TAUA0P66(NTABLE),
     *    TAUA0P47_NN(NTABLE),TAUA0P66_NN(NTABLE),DTAUBLUE_A(NTABLE),
     *    DTAURED_A(NTABLE),
     *   SAVERATIO_A(NTABLE),ANGSTROM,ANGSTROM_A(NTABLE),ANG_FACTOR

C
C Mass concentration factors for 4 aerosol models
C

      DATA MASSFAT/1.23,7.84,8.16,11.88,11.88/

C
C Initialization of variables for method selection
C
C ITABLE=0 continental model
C ITABLE=\0 specific aerosol model of desire (=1,2,3,4)
C

      IMETHOD=2
      ITABLE=0
      JTABLE=1

Change      IF(IMETHOD.EQ.2) MTABLE=4
      IF(IMETHOD.EQ.2) MTABLE=1
      IF(IMETHOD.EQ.1) MTABLE=1

      IF(ITABLE.EQ.0) THEN
        BTABLE=1
        STABLE=1
C        STABLE=4
        MTABLE=1
      ELSE
        BTABLE=ITABLE
        STABLE=ITABLE
        MTABLE=ITABLE
      ENDIF

C
C Execution only for the first time
C

       IF(Set_Counter_Land.EQ.1) THEN

        NUMDATA=NUMSQ
        IRPHSOM = HANDLE_LUTPHS

C
C Subroutine RDPHSOM reads the phase function and single scattering
C albedo for differnt models as function of optical thickness and
C scattering angles
C
        CALL RDPHSOM(OMRED,OMBLUE,PHSRED,PHSBLUE,RATIO,TAUMODEL,
     *       SCATT,NUMMODEL,NUMTAU,NUMSCATT,IRPHSOM)
C
C Subroutine RLOOKUP reads the look-up table for 0.47 and 0.66 micron channels
C

        CALL RLOOKUP(WAV,INT_A,INT_B,PIT_A,FDOWN_A,FDOWN_B,FUP_A,FUP_B,
     *       OPTH_A,SBAR_A,THET0,THE,PHI,NUMWAV,LTHET0,LTHE,LPHI,LTAU,
     *       HANDLE_LUT486,HANDLE_LUT663,NTABLE)

      ENDIF


         
C
C Initialization of variables for retrieval
C

      DO 10 IT=1,NTABLE
        TAUA0P47(IT)=0.0
        TAUA0P66(IT)=0.0
        TAUA0P47_NN(IT)=0.0
        TAUA0P66_NN(IT)=0.0
        FLUXDN0P47(IT)=0.0
        FLUXDN0P66(IT)=0.0
        FLUXUP0P47(IT)=0.0
        FLUXUP0P66(IT)=0.0
10    CONTINUE

      START_DATA=1
      END_DATA=NUMDATA
      IFINISH=0
Change
      THR213MIN=0.0
      THR213MAX=0.0
      TAUARED=0.0
      TAUABLUE=0.0
      ANG_FACTOR=1.0/ALOG(0.466/0.658)

C
C Checking the angle bounds
C

       IF(MTHET0 .GE. MINMTHET0 .AND. MTHET0 .LE. MAXMTHET0.AND.
     *    MTHET  .GE. MINMTHET  .AND. MTHET  .LE. MAXMTHET .AND.
     *    MDPHI  .GE. MINMPHI   .AND. MDPHI  .LE. MAXMPHI) THEN
c
        CALL  COMPUTE_SCATTANGLE_LAND(MTHET0,MTHET,MDPHI,IDATA,
     *  SCAT_ANGLE_LAND)

c  Select the first table......         
         MTABLE=1
         CALL SELECT_LOOKUP(INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,
     *       INT,PIT,FDOWN,FUP,OPTH,SBAR,NUMWAV,LTHET0,LTHE,LPHI,
     *       LTAU,NTABLE,MTABLE)
c  Get rayleigh......
         Call Get_Rayleigh(INT,THET0,THE,PHI,LTAU,
     *  NUMWAV,LTHET0,LTHE,LPHI,MTHET0,MTHET,MDPHI,
     *  Rayleigh_look,NEW, NEWINT,NEWINT2,
     *  NEWN,NEWN2,FINT)
     

C   call subroutine to compute Statistics

          
          Call statistics_land(W470_SYN,W659_SYN,W213_SYN,CLDMSK_500,
     *      SDS_Mean_Reflectance_Land_All,
     *      SDS_SDev_Reflectance_Land_All,
     *      SDS_Path_Radiance_Land,
     *      SDS_Critical_Reflectance_Land,
     *      SDS_Error_Crit_Reflectance_Land,
     *    SDS_Error_Path_Radiance_Land,
     *    SDS_QFlag_Critical_Ref_Land,SDS_QFlag_Path_Radiance_Land,
     *    START_1KM,END_1KM,Idata,iscan,MTHET0,MTHET,
     *    SDS_QCONTROL_CritRef_land,SCAT_ANGLE_LAND,
     *    Rayleigh_look,W865_SYN)
    

   

       LATM=Lat_center
       LONM=Lon_center

C
C --------------------------------------------------------------------------
C Method 2: perform retrieval of aerosol optical thickness using continental
C           dust, sulfate and smoke models and determine non-dust and
C           dust aerosol types based upon path radiance ratio of red and blue
C           channels and smoke and sulfate aerosols based upon geolocation
C --------------------------------------------------------------------------
C

        IF(IMETHOD.EQ.2) THEN

C
C  FIRST criterion for detecting dark targets using 2.1 micron channel
C  (reflectance at 2.1 micron between 0.01 and 0.05) if the second
C  criterion fails (i.e., IFINISH=0)
C

 
         G_factor=1.0
        IPROCE=1
        THR213GF=THR21305*G_factor
        CALL SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,W659_SYN,
     *  W865_SYN,W213_SYN,START_500,END_500,START_250,END_250,
     *  START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     *  NUMBLUE,NUMRED,RHORED,RHOBLUE,THR21300,THR213GF,
     *  IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,SD0P86,
     *  BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *  IERROR,BARLBLUE,BARLRED,BARL0P86,
     *  ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,SAVERATIO,WEIGHT,
     *  NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,TAUABLUENN,
     *  TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     *  LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     *  NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     *  ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *  TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *  FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *  INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)

Change
        THR213MIN=THR21300
        THR213MAX=THR213GF
C
C  SECOND criterion for detecting dark targets using 2.1 micron channel
C  (reflectance at 2.1 micron between 0.01 and 0.10)if the second
C  criterion fails (i.e., IFINISH=0)
C

        IF(IFINISH.EQ.0) THEN
        IPROCE=2
        THR213GF=THR21310*G_factor
        CALL SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,W659_SYN,
     *  W865_SYN,W213_SYN,START_500,END_500,START_250,END_250,
     *  START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     *  NUMBLUE,NUMRED,RHORED,RHOBLUE,THR21300,THR213GF,
     *  IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,SD0P86,
     *  BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *  IERROR,BARLBLUE,BARLRED,BARL0P86,
     *  ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,SAVERATIO,WEIGHT,
     *  NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,TAUABLUENN,
     *  TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     *  LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     *  NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     *  ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *  TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *  FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *  INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)
Change
        THR213MIN=THR21300
        THR213MAX=THR213GF
c Endif for  IPROCE=2
        ENDIF

C
C  THIRD criterion for detecting dark targets using 2.1 micron channel
C  (reflectance at 2.1 micron between 0.01 and 0.15)if the third
C  criterion fails (i.e., IFINISH=0)
C

        IF(IFINISH.EQ.0) THEN
        IPROCE=3
        THR213GF=THR21315*G_factor
        CALL SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,W659_SYN,
     *  W865_SYN,W213_SYN,START_500,END_500,START_250,END_250,
     *  START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     *  NUMBLUE,NUMRED,RHORED,RHOBLUE,THR21300,THR213GF,
     *  IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,SD0P86,
     *  BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *  IERROR,BARLBLUE,BARLRED,BARL0P86,
     *  ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,SAVERATIO,WEIGHT,
     *  NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,TAUABLUENN,
     *  TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     *  LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     *  NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     *  ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *  TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *  FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *  INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)
Change
        THR213MIN=THR21300
        THR213MAX=THR213GF
c Endif for  IPROCE=3
        ENDIF

C
C  FOURTH criterion for detecting dark targets using 2.1 micron channel
C  (reflectance at 2.1 micron between 0.01 and 0.20)if the third
C  criterion fails (i.e., IFINISH=0)
C

        IF(IFINISH.EQ.0) THEN
        IPROCE=4
        THR213GF=THR21325*G_factor
        CALL SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,W659_SYN,
     *  W865_SYN,W213_SYN,START_500,END_500,START_250,END_250,
     *  START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     *  NUMBLUE,NUMRED,RHORED,RHOBLUE,THR21300,THR213GF,
     *  IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,SD0P86,
     *  BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *  IERROR,BARLBLUE,BARLRED,BARL0P86,
     *  ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,SAVERATIO,WEIGHT,
     *  NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,TAUABLUENN,
     *  TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     *  LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     *  NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     *  ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *  TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *  FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *  INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)
Change
        THR213MIN=THR21300
        THR213MAX=THR213GF
c Endif for  IPROCE=4
        ENDIF

C
C Reset IPROCE=0 if dark targets selection criteria are not met
C

        IF(IFINISH.EQ.0) THEN
          IPROCE=0
          IERROR=0
        ELSE IF(IFINISH.EQ.1) THEN

C
C Use weighting to determine DUST and non-DUST aerosols and
C use geolocations to determine smoke and sulfate aerosol
C
C         Weighting > 0 --> smoke or sulfate aerosol
C         Weighting = 0 --> dust
C

          DO JTABLE=BTABLE,STABLE
            TAUABLUE=TAUA0P47(JTABLE)
            TAUARED=TAUA0P66(JTABLE)

            CALL RATIO_METHOD2(TAUARED,TAUABLUE,RATIO,SCATT,
     *      SCAT_ANGLE_LAND,NUMMODEL,NUMTAU,NUMSCATT,MTHET0,
     *      MTHET,MDPHI,SAVERATIO,ANGSTROM,JTABLE)

            ANGSTROM_A(JTABLE)=ANGSTROM
            SAVERATIO_A(JTABLE)=SAVERATIO
          ENDDO

          CALL WEIGHTING_METHOD2(MTABLE,BTABLE,STABLE,IERROR,
     *         SAVERATIO_A,ANGSTROM_A,SCAT_ANGLE_LAND,WEIGHT)

          IWEIGHT=IFIX(WEIGHT)

C
C CALL subroutine GEOSULSMK to determine smoke or sulfate aerosol based upon
C geolocations.
C
C if weighting factor = 1, smoke or sulfate aerosol is detected
C if weighting factor = 0, dust particle criteria will be tested
C

          IF(WEIGHT.GT.0.0)THEN

            CALL GEOSULSMK(IMONTH,LATM,LONM,ISULP,ISMK,ISMK_H,ISMK_M)
c           write(37,*)'out1',IMONTH,LATM,LONM,ISULP,ISMK,ISMK_H,ISMK_M,WEIGHT

            IF(ISULP.EQ.1) THEN

              MTABLE=3
              CALL SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,W659_SYN,
     *        W865_SYN,W213_SYN,START_500,END_500,START_250,END_250,
     *        START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     *        NUMBLUE,NUMRED,RHORED,RHOBLUE,THR213MIN,THR213MAX,
     *        IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,SD0P86,
     *        BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *        IERROR,BARLBLUE,BARLRED,BARL0P86,
     *        ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,SAVERATIO,WEIGHT,
     *        NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,TAUABLUENN,
     *        TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     *        LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     *        NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     *        ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *        TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *        FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *        INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)

              IF(IWEIGHT.NE.1) THEN

              MTABLE=2
              CALL SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,W659_SYN,
     *        W865_SYN,W213_SYN,START_500,END_500,START_250,END_250,
     *        START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     *        NUMBLUE,NUMRED,RHORED,RHOBLUE,THR213MIN,THR213MAX,
     *        IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,SD0P86,
     *        BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *        IERROR,BARLBLUE,BARLRED,BARL0P86,
     *        ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,SAVERATIO,WEIGHT,
     *        NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,TAUABLUENN,
     *        TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     *        LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     *        NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     *        ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *        TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *        FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *        INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)

              IF(TAUA0P66(2).GT.0.0.AND.TAUA0P47(2).GT.0.0) THEN
                ANGSTROM_A(2)=ANG_FACTOR*ALOG(TAUA0P66(2)/TAUA0P47(2))
              ENDIF

              ENDIF

              TAUAREDC=WEIGHT*TAUA0P66(3)+(1.0-WEIGHT)*TAUA0P66(2)
              TAUABLUEC=WEIGHT*TAUA0P47(3)+(1.0-WEIGHT)*TAUA0P47(2)
              MASSCON=WEIGHT*MASSFAT(3)*TAUA0P66(3)+
     *                (1.0-WEIGHT)*MASSFAT(2)*TAUA0P66(2)
              FLUXDNBLUE=WEIGHT*FLUXDN0P47(3)+(1.0-WEIGHT)*FLUXDN0P47(2)
              FLUXDNRED=WEIGHT*FLUXDN0P66(3)+(1.0-WEIGHT)*FLUXDN0P66(2)
              FLUXUPBLUE=WEIGHT*FLUXUP0P47(3)+(1.0-WEIGHT)*FLUXUP0P47(2)
              FLUXUPRED=WEIGHT*FLUXUP0P66(3)+(1.0-WEIGHT)*FLUXUP0P66(2)
              IF(TAUABLUEC.GT.0.0.AND.TAUAREDC.GT.0.0) THEN
                 ANGSTROM=ANG_FACTOR*ALOG(TAUAREDC/TAUABLUEC)
              ELSE
                 ANGSTROM=-9999 
              ENDIF

            ELSE IF(ISMK.EQ.1) THEN

              IF(ISMK_M.EQ.1) MTABLE=4
              IF(ISMK_H.EQ.1) MTABLE=5
              
              CALL SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,W659_SYN,
     *        W865_SYN,W213_SYN,START_500,END_500,START_250,END_250,
     *        START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     *        NUMBLUE,NUMRED,RHORED,RHOBLUE,THR213MIN,THR213MAX,
     *        IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,SD0P86,
     *        BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *        IERROR,BARLBLUE,BARLRED,BARL0P86,
     *        ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,SAVERATIO,WEIGHT,
     *        NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,TAUABLUENN,
     *        TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     *        LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     *        NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     *        ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *        TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *        FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *        INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)

              IF(IWEIGHT.NE.1) THEN

              MTABLE=2
              CALL SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,W659_SYN,
     *        W865_SYN,W213_SYN,START_500,END_500,START_250,END_250,
     *        START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     *        NUMBLUE,NUMRED,RHORED,RHOBLUE,THR213MIN,THR213MAX,
     *        IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,SD0P86,
     *        BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *        IERROR,BARLBLUE,BARLRED,BARL0P86,
     *        ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,SAVERATIO,WEIGHT,
     *        NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,TAUABLUENN,
     *        TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     *        LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     *        NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     *        ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *        TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *        FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *        INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)

              IF(TAUA0P66(2).GT.0.0.AND.TAUA0P47(2).GT.0.0) THEN
                ANGSTROM_A(2)=ANG_FACTOR*ALOG(TAUA0P66(2)/TAUA0P47(2))
              ENDIF

              ENDIF
              IF(ISMK_M.EQ.1) MTABLE=4
              IF(ISMK_H.EQ.1) MTABLE=5
c              if(MTABLE .eq. 4 .or. MTABLE .eq.5 ) write(37,*)'mtable2',MTABLE
              TAUAREDC=WEIGHT*TAUA0P66(MTABLE)+(1.0-WEIGHT)*TAUA0P66(2)
              TAUABLUEC=WEIGHT*TAUA0P47(MTABLE)+(1.0-WEIGHT)*TAUA0P47(2)
              MASSCON=WEIGHT*MASSFAT(MTABLE)*TAUA0P66(MTABLE)+
     *                (1.0-WEIGHT)*MASSFAT(2)*TAUA0P66(2)
              FLUXDNBLUE=WEIGHT*FLUXDN0P47(MTABLE)+(1.0-WEIGHT)*FLUXDN0P47(2)
              FLUXDNRED=WEIGHT*FLUXDN0P66(MTABLE)+(1.0-WEIGHT)*FLUXDN0P66(2)
              FLUXUPBLUE=WEIGHT*FLUXUP0P47(MTABLE)+(1.0-WEIGHT)*FLUXUP0P47(2)
              FLUXUPRED=WEIGHT*FLUXUP0P66(MTABLE)+(1.0-WEIGHT)*FLUXUP0P66(2)
              IF(TAUABLUEC.GT.0.0.AND.TAUAREDC.GT.0.0) THEN
                 ANGSTROM=ANG_FACTOR*ALOG(TAUAREDC/TAUABLUEC)
              ELSE
                 ANGSTROM=-9999 
              ENDIF

            ENDIF

C          ENDIF
          
C 12/15/2001

          ELSE IF(WEIGHT.EQ.0.0) THEN

            THR213GF1=THR21310*G_factor
            THR213GF2=THR21325*G_factor
            WEIGHT=0.0
            MTABLE=1
            IF(LATM.GE.30.0.AND.LATM.LE.60.0) THEN
              IF(LONM.GE.105.0.AND.LONM.LE.150.0) THEN
                MTABLE=2
              ENDIF  
            ENDIF
            CALL SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,W659_SYN,
     *      W865_SYN,W213_SYN,START_500,END_500,START_250,END_250,
     *      START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     *      NUMBLUE,NUMRED,RHORED,RHOBLUE,THR213GF1,THR213GF2,
     *      IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,SD0P86,
     *      BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *      IERROR,BARLBLUE,BARLRED,BARL0P86,
     *      ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,SAVERATIO,WEIGHT,
     *      NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,TAUABLUENN,
     *      TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     *      LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     *      NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     *      ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *      TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *      FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *      INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)

            IF(MTABLE.EQ.2) THEN
              TAUAREDC=TAUA0P66(2)
              TAUABLUEC=TAUA0P47(2)
              IF(TAUAREDC.GT.0.0.AND.TAUABLUEC.GT.0.0) THEN
                ANGSTROM_A(2)=ANG_FACTOR*ALOG(TAUAREDC/TAUABLUEC)
                ANGSTROM=ANG_FACTOR*ALOG(TAUAREDC/TAUABLUEC)
                MASSCON=MASSFAT(2)*TAUAREDC
              ELSE
                ANGSTROM=-9999
              ENDIF
              FLUXDNBLUE=FLUXDN0P47(2)
              FLUXDNRED=FLUXDN0P66(2)
              FLUXUPBLUE=FLUXUP0P47(2)
              FLUXUPRED=FLUXUP0P66(2)
            ENDIF
              TAUAREDC=TAUA0P66(MTABLE)
              TAUABLUEC=TAUA0P47(MTABLE)
              IF(TAUAREDC.GT.0.0.AND.TAUABLUEC.GT.0.0) THEN
                ANGSTROM_A(MTABLE)=ANG_FACTOR*ALOG(TAUAREDC/TAUABLUEC)
                ANGSTROM=ANG_FACTOR*ALOG(TAUAREDC/TAUABLUEC)
                MASSCON=MASSFAT(MTABLE)*TAUAREDC
              ELSE
                ANGSTROM=-9999
              ENDIF
              FLUXDNBLUE=FLUXDN0P47(MTABLE)
              FLUXDNRED=FLUXDN0P66(MTABLE)
              FLUXUPBLUE=FLUXUP0P47(MTABLE)
              FLUXUPRED=FLUXUP0P66(MTABLE)

          ENDIF

C 12/15/2001
C only continental model Procedure =5
            
            IF(IFINISH.EQ.0.OR.IERROR.NE.0) THEN 
             DEGRAD=ACOS(-1.)/180.
                G_factor=1./COS(DEGRAD*MTHET)
     *                +1./SQRT(COS(DEGRAD*MTHET0))
                G_factor=0.5*G_factor
              IPROCE=5
              THR213GF1=THR21300*G_factor
c              THR213GF2=THR21340*G_factor
c Shana   made the change HR21325 should be 0.25  9/25/2003
                THR213GF2=THR21325*G_factor
              MTABLE=1
              IF(THR213GF2.LE.40.0) THEN

              CALL SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,W659_SYN,
     *        W865_SYN,W213_SYN,START_500,END_500,START_250,END_250,
     *        START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     *        NUMBLUE,NUMRED,RHORED,RHOBLUE,THR213GF1,THR213GF2,
     *        IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,SD0P86,
     *        BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *        IERROR,BARLBLUE,BARLRED,BARL0P86,
     *        ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,SAVERATIO,WEIGHT,
     *        NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,TAUABLUENN,
     *        TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     *        LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     *        NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     *        ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *        TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *        FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *        INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)
                   
                   TAUABLUEC= TAUABLUE
                    TAUARED=TAUABLUE*0.64
                    TAUAREDC=TAUARED
                    WEIGHT=-1
 
              ENDIF
C 1c Endif for  IPROCE=5
            ENDIF

C  Endif for Finish of all procedures
        ENDIF


        IF(IFINISH.EQ.0) THEN
          IPROCE=0
          IERROR=0
        ENDIF

C
C********************** END OF METHOD 2 ********************************

       ENDIF

C
C Rayleith correction for 0.47 micron channel only
C

      CALL RAYCORT(TAUABLUE,TAUABLUEC,MHGHT)

C
C Output parameters to HDF file
C
 
      CALL OUTPUT(LATM,LONM,IAER,BARLBLUE,IDATA,
     *TAUABLUE,BARLRED,BARL0P86,TAUARED,SDBLUE,SDRED,SD0P86,TAUABLUEC,
     *TAUAREDC,IBLUE,IRED,IERROR,weight,ISULP,ISMK,IPROCE,SAVERATIO,
     *BARRBLUE,BARRRED,TAUABLUENN,TAUAREDNN,SCAT_ANGLE_LAND,ANGSTROM,
     *MTHET0,MTHET,MDPHI,QA_Flag_Land,Success_Ret_Land,Fail_Ret_Land,
     *FLUXDNBLUE,FLUXDNRED,FLUXUPBLUE,FLUXUPRED,MASSCON,
     *SDSLAT,SDSLON,SDS_MTHET0,SDS_MTHET,SDS_MPHI,SDS_Tau_Land_Ocean,
     *SDS_Aerosol_Type,
     *SDS_SCAT_ANGLE_land,SDS_mass_conc_land,SDS_angs_coeff_land,
     *SDS_CLDFRC_land,SDS_dust_weighting,SDSTAU_continantal,
     *SDS_est_uncer,SDS_RefF_land,SDS_TranF_land,SDS_NUMPIXELS_land,
     *SDSTAU_corrected,
     *SDS_ref_land,SDS_ref_STD_land,SDS_QCONTROL_land,quality_land,
     * Ret_Quality_cirrus,cloud_num_land)
 
C
C                        **Following else for 'ifang'

      ELSE
C                        **Write the output if the angles are out of
C                          bounds from the lookup table for different
C                          options with error indicators.
C
        IERROR=1
        TAUARED=-3.0
        TAUABLUE=-3.0
        TAUABLUENN=-3.0
        TAUAREDNN=-3.0
C
C          CALL ERROR(BARLBLUE,BARRBLUE,TAUABLUE,SAVERATIO,WEIGHT,
C     *      BARLRED,BARRRED,TAUARED,SDBLUE,DTAUBLUE,BARRBLUEN,
C     *      TAUABLUEN,SDRED,DTAURED,BARRREDN,TAUAREDN,TAUABLUENN,
C     *      TAUAREDNN,BARRBLUENN,BARRREDNN,IBLUE,IRED)
C
C        DTAUBLUE=TAUABLUENN -TAUABLUE
C        DTAURED =TAUAREDNN -TAUARED
c
            
        CALL OUTPUT(LATM,LONM,IAER,BARLBLUE,IDATA,TAUABLUE,BARLRED,
     *       BARL0P86,TAUARED,SDBLUE,SDRED,SD0P86,TAUABLUEC,TAUAREDC,
     *       IBLUE,IRED,IERROR,weight,ISULP,ISMK,IPROCE,SAVERATIO,
     *       BARRBLUE,BARRRED,TAUABLUENN,TAUAREDNN,SCAT_ANGLE_LAND,
     *       ANGSTROM,MTHET0,MTHET,MDPHI,QA_Flag_Land,Success_Ret_Land,
     *       Fail_Ret_Land,FLUXDNBLUE,FLUXDNRED,FLUXUPBLUE,FLUXUPRED,
     *       MASSCON,SDSLAT,SDSLON,SDS_MTHET0,SDS_MTHET,SDS_MPHI,
     *       SDS_Tau_Land_Ocean,
     *       SDS_Aerosol_Type,SDS_SCAT_ANGLE_land,SDS_mass_conc_land,
     *       SDS_angs_coeff_land,SDS_CLDFRC_land,SDS_dust_weighting,
     *       SDSTAU_continantal,SDS_est_uncer,SDS_RefF_land,
     *       SDS_TranF_land,SDS_NUMPIXELS_land,SDSTAU_corrected,
     *       SDS_ref_land,SDS_ref_STD_land,SDS_QCONTROL_land,quality_land,
     *       Ret_Quality_cirrus,cloud_num_land)
  
C
C Endif for checking the angle bounds
C
      ENDIF
              
           
   1  FORMAT(132A1)
      RETURN
      END

C***********************************************************************
      SUBROUTINE RAYCORT(TAUABLUE,TAUABLUEC,MHGHT)
C-----------------------------------------------------------------------
C!F77
C
C
C!DESCRIPTION:
C               This subroutine is to correct Rayleigh scattering for
C               topograph
C
C!INPUT PARAMETERS:
C
C       TAUABLUE      Continental aerosol optical thickness at 0.47 micron
C       TAUABLUEC     Corrected aerosol optical thickness at 0.47 micron
C       MHGHT         Topography altitude
C
C!OUTPUT PARAMETERS:
C
C       TAUABLUE      Continental aerosol optical thickness at 0.47 micron
C       TAUABLUEC     Corrected   aerosol optical thickness at 0.47 micron
C
C!REVISION HISTORY:
C  2/1/98 DAC created
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS aerosol team at NASA GSFC, Greenbelt, MD 20771
C
C!END
C-------------------------------------------------------------------------
C

      REAL TAUABLUE,TAUEBLUEC,MHGHT,LAMDA0,LAMDA,LAMDA2,LAMDA4
      REAL COEF(3),RAYTAU
      DATA LAMDA0/0.466/,COEF/0.008569,0.0113,0.00013/

      IF(TAUABLUE.GT.0.0) THEN

        MHGHT=MHGHT+0.4
        LAMDA=LAMDA0*EXP((MHGHT-0.4)/36.0)
        LAMDA2=1./(LAMDA*LAMDA)
        LAMDA4=LAMDA2*LAMDA2

        RAYTAU=COEF(1)*LAMDA4*(1.0+COEF(2)*LAMDA2+COEF(3)*LAMDA4)
        CORTAU=0.186-RAYTAU

        TAUABLUE=TAUABLUE+CORTAU
        TAUABLUEC=TAUABLUEC+CORTAU

      ENDIF

      RETURN
      END

C***********************************************************************
      SUBROUTINE RLOOKUP(WAV,INT_A,INT_B,PIT_A,FDOWN_A,FDOWN_B,FUP_A,FUP_B,
     *OPTH_A,SBAR_A,THET0,THE,PHI,NUMWAV,LTHET0,LTHE,LPHI,LTAU,HANDLE_LUT466,
     *HANDLE_LUT670,NTABLE)
C-----------------------------------------------------------------------
C!F77
C
C
C!DESCRIPTION:
C               This subroutine reads 4 lookup tables
C
C!INPUT PARAMETERS:
C
C          HANDLE_LUT466   Unit number of look-up table at 0.466 micron
C          HANDLE_LUT670   Unit number of look-up table at 0.670 micron
C          NUMWAV          Number of wavelengths in lookup table.
C          LTHET0          Number of solarzenith angle in lookup table.
C          LTHE            Number of view angle in lookup table.
C          LPHI            Number of azimuths in lookup table.
C          LTAU            Number of optical depths in lookup table.
C          NTABLE          Number of tables (=5)
C
C!OUTPUT PARAMETERS:
C
C          WAV             Wavelengths
C          INT_A           Reflectance
C          PIT_A           Transmission factor
C          FDOWN_A         Flux down
C          OPTH_A          Optical depth
C          SBAR_A          Sbar
C          THET0           Solar Zenith Angle
C          THE             Satellite View angle
C          PHI             Azimuth angle difference
C
C!DESIGN NOTES:
C
C  Read from look-up table.
C   wav=wavelengths(2)(.466,.670 micrometers)                2
C   lthet0=number of theta0(solar zenith angle)              9
C   thet0=(0,12,24,36,48,54,60,66,72   degrees)
C   lthe =number of the(observation zenith angle)           13
C   the=(0,6,.....72 degrees every 6 degrees)
C   lphi =number of phi(observation azimuth angle)          31
C   phi=(0,6,12,18,24,30,36,42,48,54,60,66,72,78,84,90,96,
C        102,108,114,120,126,132,138,144,150,156,162,168,
C        174,180)
C   ltau =number of opth(optical thickness)                  7
C   opth=(0.0,0.25,0.50,1.0,2.0,3.0,5.0)
C   lhght = number of hght(observation heights)              1
C   hght=(80.0 km)
C
C!REVISION HISTORY:
c Process_land_V2.f
c 10/08/1999 fhliang
c removed multiple specification of data type for variable weight.
c
C  1/28/98 fhliang
C    fixed prolog
c
C  5/1/98 DAC modified interface and reading structure
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS aerosol team at NASA Goddard Space Flight
C Center, Greenbelt, Maryland
C
C!END
C-------------------------------------------------------------------------
C

      IMPLICIT  NONE
      REAL PI,DTR,RTD
      PARAMETER (PI = 3.1415927, DTR = PI/180.0,RTD=180./PI)
      INTEGER HANDLE_LUT466,HANDLE_LUT670
      INTEGER I,IPHI,ITAU,NTABLE,J
      INTEGER ITHE,ITHET0,IWAV,LPHI,LTAU
      INTEGER LTHE,LTHET0,NFILE,NUMWAV
      REAL WAV(NUMWAV),THE(LTHE),PHI(LPHI),THET0(LTHET0)
      REAL OPTH_A(LTAU,NUMWAV,NTABLE)
      REAL FDOWN_B(LTAU,LTHET0,NUMWAV,NTABLE),INTW(31),PITW(13)
      REAL FDOWN_A(LTHET0,LTAU,NUMWAV,NTABLE)
      REAL FUP_B(LTAU,LTHET0,NUMWAV,NTABLE)
      REAL FUP_A(LTHET0,LTAU,NUMWAV,NTABLE)
      REAL SBAR_A(LTAU,NUMWAV,NTABLE)
      REAL INT_B(LPHI,LTHE,LTAU,LTHET0,NUMWAV,NTABLE)
      REAL INT_A(LTHET0,LPHI,LTHE,LTAU,NUMWAV,NTABLE)
      REAL PIT_A(LTHE,LTAU,NUMWAV,NTABLE)
      CHARACTER*1 LINE(132)
      SAVE

C
C Reads observation zenith angles(the),and observation azimuth angles
C from look-up tables
C

      DO 10 J=1,NTABLE

      DO 20 IWAV = 1,NUMWAV

        IF(IWAV.EQ.1)NFILE=HANDLE_LUT466
        IF(IWAV.EQ.2)NFILE=HANDLE_LUT670

        READ(NFILE,1000)(THE(I),I=1,LTHE)
        READ(NFILE,1001)(PHI(I),I=1,LPHI)

C
C Reads wavelength(wav),optical thickness(opth),solar zenith angle(thet0),
C reflectance ofatmosphere(sbarw), flux incident on surface(fdownw)
C from look-up tables.
C

        DO 100 ITHET0=1,LTHET0
          DO 101 ITAU=1,LTAU
              READ(NFILE,1002)WAV(IWAV),OPTH_A(ITAU,IWAV,J),THET0(ITHET0),
     1                    SBAR_A(ITAU,IWAV,J),FDOWN_B(ITAU,ITHET0,IWAV,J),
     1                    FUP_B(ITAU,ITHET0,IWAV,J)

              READ(NFILE,1003)LINE

C
C Reads atmospheric radiance (intw) as a function of solar zenith angle,
C optical thickness, height, observation zenith angle and azimuth angle
C from look-up table.
C

              DO 103 ITHE=1,LTHE
                 READ(NFILE,*)(INTW(IPHI),IPHI=1,LPHI)

                 DO 104 IPHI = 1,LPHI
                   INT_B(IPHI,ITHE,ITAU,ITHET0,IWAV,J)=INTW(IPHI)
104              CONTINUE
103           CONTINUE

C
C Reads transmission as a function of observation zenith angle,
C optical thickness, and  height.
C

              READ(NFILE,1005)(PITW(ITHE),ITHE=1,LTHE)
              DO 105 ITHE = 1,LTHE
                PIT_A(ITHE,ITAU,IWAV,J)=PITW(ITHE)
105           CONTINUE
101       CONTINUE
100     CONTINUE
20      CONTINUE
10    CONTINUE

C
C Re-ordering of the dimensions of FDOWN and INT arrays
C

      DO 200 J=1,NTABLE
        DO 210 I=1,NUMWAV
          DO 220 ITHET0=1,LTHET0
            DO 230 ITAU=1,LTAU
             FDOWN_A(ITHET0,ITAU,I,J)=FDOWN_B(ITAU,ITHET0,I,J)
             FUP_A(ITHET0,ITAU,I,J)=FUP_B(ITAU,ITHET0,I,J)
              DO 240 ITHE=1,LTHE
                DO 250 IPHI=1,LPHI

                  INT_A(ITHET0,IPHI,ITHE,ITAU,I,J)=INT_B(IPHI,ITHE,ITAU,ITHET0,I,J)

250             CONTINUE
240           CONTINUE
230         CONTINUE
220       CONTINUE
210     CONTINUE
200   CONTINUE

C
1000  FORMAT(6X,13F6.0)
1001  FORMAT(4X,(19F6.0))
1002  FORMAT(11X,F7.3,19X,F5.2,22X,F5.0,/,5X,F7.4,11X,F6.4,9X,F6.4)
1003  FORMAT(132A1)
1004  FORMAT(10E12.4)
1005  FORMAT(3X,13f6.5)
      RETURN
      END

C***********************************************************************
      SUBROUTINE SELECT_LOOKUP(INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,
     *INT,PIT,FDOWN,FUP,OPTH,SBAR,NUMWAV,LTHET0,LTHE,LPHI,LTAU,NTABLE,
     *ITABLE)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              This subroutine select look-up table
C
C!INPUT PARAMETERS:
C
C          INT_A      Reflectance
C          PIT_A      Transmission factor
C          FDOWN_A    Flux down
C          FUP_A      Flux up
C          OPTH_A     Optical depth
C          SBAR_A     Sbar
C          NUMWAV     Number of wavelengths in lookup table.
C          LTHET0     Number of solarzenith angle in lookup table.
C          LTHE       Number of view angle in lookup table.
C          LPHI       Number of azimuths in lookup table.
C          LTAU       Number of optical depths in lookup table.
C
C!OUTPUT PARAMETERS:
C
C          INT        Reflectance
C          PIT        Transmission factor
C          FDOWN      Flux down
C          FUP        Flux up
C          OPTH       Optical depth
C          SBAR       Sbar
C
C!REVISION HISTORY:
C 5/1/98 DAC created
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS aerosol team at Goddard Space Flight Center,
C Greenbelt, Maryland.
C
C!END
C-----------------------------------------------------------------
C
      IMPLICIT NONE
      INTEGER I,J,K,L,M,ITABLE,NTABLE
      INTEGER LPHI,LTAU,LTHE,LTHET0,NUMWAV

      REAL OPTH_A(LTAU,NUMWAV,NTABLE)
C      REAL FDOWN_A(LTAU,LTHET0,NUMWAV,NTABLE)
      REAL FDOWN_A(LTHET0,LTAU,NUMWAV,NTABLE)
      REAL FUP_A(LTHET0,LTAU,NUMWAV,NTABLE)
      REAL SBAR_A(LTAU,NUMWAV,NTABLE)
C      REAL INT_A(LPHI,LTHE,LTAU,LTHET0,NUMWAV,NTABLE)
      REAL INT_A(LTHET0,LPHI,LTHE,LTAU,NUMWAV,NTABLE)
      REAL PIT_A(LTHE,LTAU,NUMWAV,NTABLE)

      REAL OPTH(LTAU,NUMWAV)
C      REAL FDOWN(LTAU,LTHET0,NUMWAV)
      REAL FDOWN(LTHET0,LTAU,NUMWAV)
      REAL FUP(LTHET0,LTAU,NUMWAV)
      REAL SBAR(LTAU,NUMWAV)
C      REAL INT(LPHI,LTHE,LTAU,LTHET0,NUMWAV)
      REAL INT(LTHET0,LPHI,LTHE,LTAU,NUMWAV)
      REAL PIT(LTHE,LTAU,NUMWAV)
      SAVE

C
C Select look-up table variable for given ITABLE
C

      DO 10 I=1,NUMWAV
          DO 30 K=1,LTAU
            DO 20 J=1,LTHET0
              FDOWN(J,K,I)=FDOWN_A(J,K,I,ITABLE)
              FUP(J,K,I)=FUP_A(J,K,I,ITABLE)
20          CONTINUE
30        CONTINUE

          DO 31 K=1,LTAU
            DO 41 L=1,LTHE
              DO 51 M=1,LPHI
                DO 21 J=1,LTHET0
                  INT(J,M,L,K,I)=INT_A(J,M,L,K,I,ITABLE)
21              CONTINUE
51            CONTINUE
41          CONTINUE
31        CONTINUE

        DO 60 J=1,LTAU
          DO 70 K=1,LTHE
            PIT(K,J,I)=PIT_A(K,J,I,ITABLE)
70        CONTINUE
60      CONTINUE

        DO 80 J=1,LTAU
          OPTH(J,I)=OPTH_A(J,I,ITABLE)
          SBAR(J,I)=SBAR_A(J,I,ITABLE)
80      CONTINUE
10    CONTINUE

      RETURN
      END

C**********************************************************************
      SUBROUTINE RATIO_METHOD2(TAUARED,TAUABLUE,RATIO,SCATT,
     *SCATMODIS,NUMMODEL,NUMTAU,NUMSCATT,MTHET0,MTHET,MPHI,SAVERATIO,
     *ANGSTROM,ITABLE)
C----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C                This routine computes the ratio of path radiance
C                between red (at 0.66 micron) and blue (at 0.47 micron)
C                wavelength given a scattering angle. For continental
C                and dust aerosol models, the path radiance are
C                independent of optical thickness and for sulfate
C                and smoke aerosols, the phase function and single
C                scattering albedo are a function of aerosol optical thickness.
C
C!INPUT PARAMETERS:
C
C        SCATMODIS    MODIS scattering angle calculated
C        RATIO        Ratio of single scattering and phase function between
C                     red (0.66 micron) and blue (0.47 micron)
C        TAUMODEL     Aerosol optical thickness of aerosol models
C        SCATT        Scattering angles of aerosol models
C        NUMMODEL     Number of aerosol models
C        NUMTAU       Number of optical thicknesses
C        NUMSCATT     Number of scattering angles
C        MTHET0       Measured Solar zenith angle
C        MTHET        Measured view angle
C        MPHI         Measured azimuth angle
C        ITABLE       Index of aerosol model used (=1,2,3,4)
C
C!OUTPUT PARAMETERS:
C
C        SAVERATIO    Ratio of aerosol path radiance of blue over red
C        ANGSTROM     Angstrom coefficient between red and blue wavelengths
C
C!REVISION HISTORY:
C 2/1/98 created by DAC
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS aerosol team at NASA Goddard Space Flight Center
C at Greenbelt, Maryland.
C
C!END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
       IMPLICIT NONE
       INTEGER IJ,LOPT,NUMMODEL,NUMSCATT,NUMTAU,ITABLE,IND_SCAT
       REAL SAVERATIO,TAUABLUE,TAUARED,ANG_FACTOR
       REAL TAURATIO,ANGSTROM,SCATMODIS
       REAL RATIO(NUMSCATT,NUMTAU,NUMMODEL)
       REAL SCATT(NUMSCATT)
C,TAUMODEL(NUMTAU)
       REAL MTHET0,MTHET,MPHI,EPSLON
       REAL XX(100),YY(100),Y1(1),ARG(1)     ! potential trouble
       SAVE
C
C Initialization of variables
C
       SAVERATIO=0.0
       TAURATIO=0.0
       ANGSTROM=0.0
       EPSLON=0.000001
       ANG_FACTOR=1.0/ALOG(0.466/0.658)
C
C Check bounds of MODIS scattering anlge to be within 60 and 170 degrees
C
      IF(SCATMODIS.LE.60.0) SCATMODIS=60.0
      IF(SCATMODIS.GE.170.0) SCATMODIS=170.0
      ARG(1)=SCATMODIS
C
C To derive the ratio of aerosol path radiance of red and blue; ITABLE indicates
C which aerosol look-up table is used; Note that ,for smoke and sulfate aerosol,
C phase function and single scattering albedo are a function of scattering angle
C
C **using angstrom coefficient to determine non-dust and dust particles for
C aerosol optical thickness > 0.6
C

      IF(TAUARED.GT.0.0.AND.TAUABLUE.GT.0.0) THEN
        TAURATIO=TAUARED/TAUABLUE
      ELSE
        TAURATIO=-9999
      ENDIF

      IF(TAUARED.GT.0.0.AND.TAUABLUE.GT.0.0) THEN
        ANGSTROM=ANG_FACTOR*ALOG(TAURATIO)
      ELSE
        ANGSTROM=-9999
      ENDIF

C
C If TAURATIO > 0, calculate the ratio of aerosol path radiance
C between red and blue, otherwise no calculation is done, the
C aerosol optical thickness is reported to be zero (??? FILL_VALUE)
C

      IF(TAURATIO.GT.0) THEN
        DO 100 IJ=1,NUMSCATT
          XX(IJ)=SCATT(IJ)
          IF(ITABLE.LE.2) THEN
            YY(IJ)=TAURATIO*RATIO(IJ,1,ITABLE)
          ELSE IF(ITABLE.GT.2) THEN
            IF(TAUARED.LT.0.05) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,1,ITABLE)
            ELSE IF(TAUARED.GT.0.05-EPSLON.AND.TAUARED.LT.0.10) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,2,ITABLE)
            ELSE IF(TAUARED.GT.0.10-EPSLON.AND.TAUARED.LT.0.15) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,3,ITABLE)
            ELSE IF(TAUARED.GT.0.15-EPSLON.AND.TAUARED.LT.0.20) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,4,ITABLE)
            ELSE IF(TAUARED.GT.0.20-EPSLON.AND.TAUARED.LT.0.25) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,5,ITABLE)
            ELSE IF(TAUARED.GT.0.25-EPSLON.AND.TAUARED.LT.0.30) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,6,ITABLE)
            ELSE IF(TAUARED.GT.0.30-EPSLON.AND.TAUARED.LT.0.35) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,7,ITABLE)
            ELSE IF(TAUARED.GT.0.35-EPSLON.AND.TAUARED.LT.0.40) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,8,ITABLE)
            ELSE IF(TAUARED.GT.0.40-EPSLON.AND.TAUARED.LT.0.45) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,9,ITABLE)
            ELSE IF(TAUARED.GT.0.45-EPSLON.AND.TAUARED.LT.0.50) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,10,ITABLE)
            ELSE IF(TAUARED.GT.0.50-EPSLON.AND.TAUARED.LT.0.55) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,11,ITABLE)
            ELSE IF(TAUARED.GT.0.55-EPSLON) THEN
              YY(IJ)=TAURATIO*RATIO(IJ,12,ITABLE)
            ENDIF
          ENDIF

100   CONTINUE

C
C To interpolate to MODIS scatting angle to derive path radiance;
C SAVERATIO needs to be saved for 4 different aerosol models
C

      CALL LOCATE_OPT(SCATT,NUMSCATT,SCATMODIS,IND_SCAT,ITABLE,LOPT)
      CALL LININT_OPT(SCATMODIS,Y1(1),SCATT,YY,NUMSCATT,IND_SCAT)

      SAVERATIO=Y1(1)

      ELSE

      SAVERATIO=-1.0

      ENDIF
C
      RETURN
      END

C**********************************************************************
      SUBROUTINE WEIGHTING_METHOD2(MTABLE,BTABLE,STABLE,IERROR,
     *SAVERATIO_A,ANGSTROM_A,SCATMODIS,WEIGHT)
C----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C                This routine computes the weighting factor for the
C                relative percentage of smoke/sulfate aerosol as opposed
C                to dust. Continental aerosol and dust models are
C                independent of optical thickness but sulfate and smoke
C                phase function and single scattering albedo are
C                a function of optical thickness.
C
C!INPUT PARAMETERS:
C
C        SAVERATIO_A    Ratio of aerosol path radiance (continental model)
C                       of blue and red for four different aerosol models
C        ANGSTROM_A     Angstrom coefficients between blue and red channels
C        SCATMODIS      MODIS scattering angle
C        MTABLE         Specific table index
C        BTABLE         Beginning table index
C        STABLE         Ending table index
C
C!OUTPUT PARAMETERS:
C
C        WEIGHT         Weighting factor between dust and non-dust aerosols
C                       (for smoke and sulfate aerosol, it is determined
C                        according to geographic locations)
C
C!REVISION HISTORY:
C 2/1/98 created by DAC
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS aerosol team at NASA Goddard Space Flight Center
C at Greenbelt, Maryland.
C
C!END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C

      IMPLICIT NONE
      INTEGER ITABLE,MTABLE,BTABLE,STABLE,IGT,ILT,IBT,IC,
     1        IERROR
      REAL SAVERATIO_A(4),ANGSTROM_A(4),SAVERATIO,ANGSTROM
      REAL SCATMODIS,THRSHOLD1,THRSHOLD2,WEIGHT,RATIO_MEAN

C
C Initialization of indices for determination of aerosol types
C

      IGT=0
      ILT=0
      IBT=0
      IC=0
      WEIGHT=-9999 
      RATIO_MEAN=0.0

C
C To Compute weighting factor between dust and sulfate/smoke aerosols.
C For DUST, weighting=0, i.e., SAVERATIO > THRSHOLD1
C For SMOKE/SULFATE, weighting=1,i.e., SAVERATIO < THRSHOLD2
C Otherwise, calculate the weighting factor between them.
C
C       SMOKE or SULFATE                             DUST
C
C            =1          WEIGHTING FACTOR         =0
C
C   .....THRSHOLD2...|......................|..THRSHOLD1......--->
C
C                   PATH RADIANCE RATIO
C
C Set the thresholds (THRSHOLD1 and THRSHOLD2) dependent upon the
C scattering angle (see Kaufman et al., 1997). It needs to be
C updated by the new values
C

      IF(SCATMODIS .LT.60.0)SCATMODIS=60.0
      IF(SCATMODIS .GT.170.0)SCATMODIS=170.0
      IF(SCATMODIS.GE.60.0.AND.SCATMODIS.LT.150.0) THEN
C        THRSHOLD1=1.2+0.0011*(SCATMODIS-60.0)
C        THRSHOLD2=1.0+0.0011*(SCATMODIS-60.0)
        THRSHOLD1=1.1+0.0011*(SCATMODIS-60.0)
        THRSHOLD2=0.9+0.0011*(SCATMODIS-60.0)
      ELSE IF(SCATMODIS.GE.150.0.AND.SCATMODIS.LE.180.0) THEN
C        THRSHOLD1=1.3-0.0033*(SCATMODIS-150.0)
C        THRSHOLD2=1.1-0.0033*(SCATMODIS-150.0)
        THRSHOLD1=1.2-0.0033*(SCATMODIS-150.0)
        THRSHOLD2=1.0-0.0033*(SCATMODIS-150.0)
      ENDIF
C      IF(SCATMODIS.GE.60.0.AND.SCATMODIS.LE.150.0) THEN
C        THRSHOLD1=0.90
C        THRSHOLD2=0.72
C      ELSE IF(SCATMODIS.GT.150.0.AND.SCATMODIS.LE.168.0) THEN
C        THRSHOLD1=0.90-0.01*(SCATMODIS-150.0)
C        THRSHOLD2=0.72
C      ELSE IF(SCATMODIS.GT.168.0.AND.SCATMODIS.LE.170.0) THEN
C        THRSHOLD1=0.72
C        THRSHOLD2=0.66
C      ENDIF

C
C Test of path radiance ratios for determining dust/non-dust particles
C by using the results derived from the four aerosol models
C
C (if BTABLE=STABLE, either sulfate or smoke will be determined--disabled).
C

C      IF(BTABLE.NE.STABLE) THEN
C      DO ITABLE=1,MTABLE

      DO ITABLE=BTABLE,STABLE
        SAVERATIO=SAVERATIO_A(ITABLE)
        ANGSTROM=ANGSTROM_A(ITABLE)
        IF(SAVERATIO.GT.0.0) THEN
          IF(SAVERATIO.GE.THRSHOLD1)THEN
            IGT=IGT+1
          ELSE IF(SAVERATIO.LT.THRSHOLD2) THEN
            ILT=ILT+1
          ELSE
            IBT=IBT+1
          ENDIF
        ENDIF
      ENDDO

C
C If 3 out 4 aerosol models agree, then the weighting
C factor is determined to be 0 (i.e., dust model)
C or 1 (i.e., sulfate/smoke model). Otherwise the
C weighting factor is determined by the mean of path radiance ratios of
C all the four aerosol models
C
C *********** the approach remains to be seen *************
C

Change  DO ITABLE=1,MTABLE
C        IF(ITABLE.NE.2) THEN   ! not sure about dust look-up tables

      DO ITABLE=BTABLE,STABLE
         IF(SAVERATIO_A(ITABLE).GT.0.0) THEN
           RATIO_MEAN=RATIO_MEAN+SAVERATIO_A(ITABLE)
           IC=IC+1
         ENDIF
C        ENDIF
      ENDDO
Change      IF(IC.GE.3) RATIO_MEAN=RATIO_MEAN/FLOAT(IC)
      IF(IC.GT.0) RATIO_MEAN=RATIO_MEAN/FLOAT(IC)

Change      IF(IGT.GE.3) THEN
      IF(IGT.GT.0) THEN
        WEIGHT=0.0
Change      ELSE IF(ILT.GE.3) THEN
      ELSE IF(ILT.GT.0) THEN
        WEIGHT=1.0
      ELSE
Change        IF(IC.GE.3) THEN
        IF(IC.GT.0) THEN
          WEIGHT=(RATIO_MEAN-THRSHOLD1)/(THRSHOLD2-THRSHOLD1)
C should never happen
          IF(WEIGHT.LT.0)WEIGHT=0.0
          IF(WEIGHT.GT.1)WEIGHT=1.0
        ENDIF
      ENDIF

C      ELSE

C        WEIGHT=1.0

C      ENDIF

C      WRITE(89,*) WEIGHT,SAVERATIO_A

C
C If WEIGHT = -9999.0 (initialized value) no determination of aerosol can be
C made because optical thicknesses can not be retrieved with all 4 models.
C (only one model is used to determine the path radiance ratio in order
C to reduce computational time)
C

      IF(WEIGHT.LT.0.) IERROR=6

      RETURN
      END

C***********************************************************************
       SUBROUTINE SUBOPT1(ISWATH,ILINE,CLDMSK_250,W375_SYN,W470_SYN,
     * W659_SYN,W865_SYN,START_500,END_500,START_250,END_250,
     * START_1KM,END_1KM,TAUARED,TAUABLUE,BLUE,RED,R0P86,
     * NUMBLUE,NUMRED,IBLUE,IRED,SDBLUE,SDRED,TAUABLUENN,
     * TAUAREDNN,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,IERROR,BARLBLUE,
     * BARLRED,RHORED,RHOBLUE,THR375,IGRIDX,IGRIDY,
     * SAVERATIO,WEIGHT,ICLDRED,ICLDBLUE,NUMCLDFREE,RBLUE,RRED,
     * BARRBLUE,BARRRED,NUMPERBLUE,NUMPERRED,MTHET0,MTHET,MDPHI,
     * SURBLUE,SURRED,INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,NUMWAV,
     * LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,LRED,LBLUE,L0P86,
     * FLUXDN,FLUXUP,NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,ARRAY1,
     * ARRAY2,IFINISH)
C----------------------------------------------------------------------
c!F77
C
c!DESCRIPTION:
C   This subroutine processes 10* 10 pixel box for cloud detection and
C   finds the average reflectance for red and blue channels.  Surface
C   Reflectance from wavelength 3.95 . This surface Reflectance and
C   average Reflectance for red and blue channel are send to lookup table
C   and optical thickness is derived.
C
c!INPUT PARAMETERS:
C    INTEGER CLDMSK_250(4*ISWATH,4*ILINE)      Cloud mask 1=cloudy,o=clear
C      W395(IGRIDX,IGRIDY)          Reflectance for wav=3.95um
C      W470_SYN(2*IGRIDX,2*IGRIDY)      Reflectance for wav=0.47um
C      W213_SYN(2*IGRIDX,2*IGRIDY)      Reflectance for wav=2.13um
C      W659_SYN(4*IGRIDX,4*IGRIDY)      Reflectance for wav=0.66um
C                     MTHET0     Measured solar zenith angle in deg.
C                      MTHET     Measured viewangle from ground in deg.
C                      MDPHI     Measured relative azimuth in deg.
C                    *******Lookup table varaibles
C    WAV(NUMWAV)                      Wavelength
C   OPTH(NUMWAV,LTAU)                 Optical depth
C   THET0(LTHET0)                     Solar Angle
C   FDOWN(NUMWAV,LTHET0,LTAU)         Flux down
C   THE(LTHE)                         View angle
C   PHI(LPHI)                         Azimuth
C   SBAR(NUMWAV,LTAU)                 Sbar
C   INT(NUMWAV,LTHET0,LTAU,LTHE,LPHI) Reflectance
C   PIT(NUMWAV,LTHE,LTAU)             Transmission factor
C
C   THR375                            Threshold for 3.95 wavelength.
C
c!OUTPUT PARAMETERS:
C   TAUARED            Derived optical thickness for red channel.
C   TAUABLUE           Derived optical thickness for blue channel
C   SDBLUE            Standard deviation for Reflectance blue channel.
C   SDRED             Standard deviation for Reflectance red channel.
C   SAVERATIO         ratio of aerosol path radiance of red to blue
C   WEIGHT            relative contribution of smoke/sulfate to dust.
C
c!REVISION HISTORY:
c 01/28/98 fhliang
c fixed prolog.
c
c!TEAM-UNIQUE HEADER:
c
c Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
c GSFC, Greenbelt, MD
c
c!END
C-------------------------------------------------------------------
      IMPLICIT NONE
C     INCLUDE 'mod04L_inc.f'
C     INCLUDE 'mod04.inc'

      INTEGER IFINISH,LPHI,LTAU,LTHE,LTHET0,NUMWAV,IGRIDX,IGRIDY,
     &        ISWATH,ILINE,JTABLE
      INTEGER I395,IBLUE,ICLDBLUE,ICLDRED,ITIME
      INTEGER NUMRED(4*IGRIDX,4*IGRIDY),NUMBLUE(2*IGRIDX,2*IGRIDY)
      INTEGER IERROR,IOPT,IRED,NBLUE,NRED
      INTEGER CLDMSK_250(4*ISWATH,4*ILINE)
      INTEGER START_500,END_500,START_250,END_250,START_1KM,END_1KM
C 2 lines added by vlin
      REAL SAVERATIO,WEIGHT
      REAL W470_SYN(2*ISWATH,2*ILINE)
      REAL w865_SYN(4*ISWATH,4*ILINE)
      REAL W659_SYN(4*ISWATH,4*ILINE)
      REAL W375_SYN(ISWATH,ILINE)
      REAL BLUE(2*IGRIDX,2*IGRIDY),RED(4*IGRIDX,4*IGRIDY)
      REAL RHORED(4*IGRIDX,4*IGRIDY),R0P86(4*IGRIDX,4*IGRIDY)
      REAL RHOBLUE(2*IGRIDX,2*IGRIDY),LRED((4*IGRIDX)*(4*IGRIDY))
      REAL LBLUE((2*IGRIDX)*(2*IGRIDY)),L0P86((4*IGRIDX)*(4*IGRIDY))
      REAL RBLUE((2*IGRIDX)*(2*IGRIDY)),RRED((4*IGRIDX)*(4*IGRIDY))
      REAL RRBLUE,MTHET0,MTHET,MDPHI
      REAL RRRED,BARLBLUE,BARRBLUE,BARRRED,BARLRED,BARL0P86,
     &     TAUABLUE,TAUARED
      REAL ARRAY1((4*IGRIDX)*(4*IGRIDY)),ARRAY2((4*IGRIDX)*(4*IGRIDY))
      REAL BARLBLUEN,BARLREDN,BARRBLUEN,BARRBLUENN
      REAL BARRREDN,BARRREDNN
      REAL DTAUBLUE,DTAURED,SDBLUE,SDRED,SD0P86,TAUABLUEN
      REAL TAUABLUENN,TAUAREDN,TAUAREDNN,FLUXDN(NUMWAV),FLUXUP(NUMWAV)

      REAL OPTH(LTAU,NUMWAV),THET0(LTHET0)
      REAL SBAR(LTAU,NUMWAV)
      REAL FDOWN(LTHET0,LTAU,NUMWAV),THE(LTHE),PHI(LPHI)
      REAL FUP(LTHET0,LTAU,NUMWAV)
      REAL INT(LTHET0,LTHE,LPHI,LTAU,NUMWAV),PIT(LTHE,LTAU,NUMWAV)
      REAL NFDOWN(LTAU),NEWINT(LPHI,LTHE,LTAU),FINT(LTHE)
      REAL NFUP(LTAU)
      REAL NEW(LPHI),NEWINT2(LTHE,LTAU),IMES(LTAU),WAVOPT(LTAU)
      REAL NEWN(LPHI),NEWN2(LPHI),FT(LTAU)
      REAL RHO(3,2),MINT(3,2)

C
C                        **NUMCLDFREE is threshold for number of
C                          cloudfree pixels in 3.959um resolution for
C                          option 1 and 3 and 4* NUMCLDFREE for
C                          option2
C                        **NUMPERBLUE is threshold for number of total
C                          number of pixels before finding the average
C                          of percentiles for blue channel.
C                          NUMPERBLUEis threshold for red
C                         channel.
      REAL      THR375
      INTEGER   NUMCLDFREE,NUMPERBLUE,NUMPERRED
C
C*********************************************************************
C                        **The program averages the radiances between
C                          10th and 40th percentile.
C
C                       **Note !!!!
C                       change the number for the percentile desired.

       INTEGER NLOWBLUE,NUPBLUE,NLOWRED,NUPRED
C*********************************************************************
C                        **For option 1 surface reflectance is defined
C
C                        **Note!!!!!
C                        Numbers under further investigation.....
C
       REAL SURBLUE,SURRED
          SAVE
          IFINISH=1
          IERROR=0



C                        **For option = 1 see the documentation
C                          'IFOPT1'- if option = 1
          RRBLUE = SURBLUE
          RRRED  = SURRED
         RHOBLUE(1,1) = RRBLUE
         RHORED(1,1)  = RRRED
C
C                        **Call subroutine option1 to check for the
C                          dark pixels for blue and red channels
C

        CALL OPTION1(ISWATH,ILINE,CLDMSK_250,W375_SYN,
     *  W470_SYN,
     *  W659_SYN,W865_SYN,BLUE,RED,NUMBLUE,
     *  NUMRED,RHORED,RHOBLUE,THR375 ,IGRIDX,IGRIDY,
     *  I395,ICLDRED,ICLDBLUE,START_1KM,END_1KM)
C
C                        **'IFERROR' if
C                           i395>0  and icld>0 indicate that the
C                           threshold for 3.95 um is met  and the
C                           data set is cloud free as defined.
C
C
         IF(I395 .GT.0 .AND.ICLDBLUE .GE.NUMCLDFREE .AND.
     *       ICLDRED .GE. 4*NUMCLDFREE) THEN


C                        **Call to sort routine, prepares the data and
C                          calls subroutine shlsrt_land to sort the arrays
C                          in ascending order of radiance values.
C
                IOPT = 1
       CALL SORT(NUMRED,NUMBLUE,RED,BLUE,R0P86,RHORED,RHOBLUE,
     *      LRED,LBLUE,L0P86,RRED,RBLUE,NBLUE,NRED,IGRIDX,IGRIDY,IOPT)
C
C                        **Call to subroutine percentile, to pick up
C                          only part of the array and find the average
C                          between given range and if the threshold
C                          for number of points in blue and red
C                          channels are met.
C

        IF(NBLUE .GE.NUMPERBLUE .AND. NRED .GE.NUMPERRED) THEN
        CALL PERCENTILE(NBLUE,LBLUE,RBLUE,NRED,LRED,RRED,BARLBLUE,
     *    BARRBLUE,BARRRED,BARLRED,L0P86,BARL0P86,NLOWBLUE,NUPBLUE,
     *    NLOWRED,NUPRED,IGRIDX,IGRIDY,SDBLUE,SDRED,SD0P86,BARRBLUEN,
     *    BARRREDN,BARRBLUENN,BARRREDNN,ARRAY1,ARRAY2,IBLUE,IRED)

C                        **Radiance for red and blue channel computed
C                          by adding standard deviation.

          BARLREDN= BARLRED+SDRED
          BARLBLUEN=BARLBLUE+SDBLUE
C
C                        **Call to subroutine optical, to compute the
C                          optical thickness for red and blue channels
C                          from the detected radiances.
C
C
C Assign apparent and surface reflectances of 3 different conditions to
C 2-D arrarys to computational time
C

         RHO(1,1)=BARRBLUE
         MINT(1,1)=BARLBLUE
         RHO(1,2)=BARRRED
         MINT(1,2)=BARLRED
         RHO(2,1)=BARRBLUEN
         MINT(2,1)=BARLBLUE
         RHO(2,2)=BARRREDN
         MINT(2,2)=BARLRED
c        RHO(3,1)=BARRBLUENN
         RHO(3,1)=BARRBLUE
         MINT(3,1)=BARLBLUEN
c        RHO(3,2)=BARRREDNN
         RHO(3,2)=BARRRED
         MINT(3,2)=BARLREDN

         CALL OPTICAL(INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,
     *   NUMWAV,LTHET0,LTHE,LPHI,LTAU,MTHET0,MTHET,MDPHI,BARLBLUE,
     *   BARRBLUE,BARRRED,BARLRED,TAUABLUE,TAUARED,NFDOWN,NFUP,FINT,NEW,
     *   NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,BARLREDN,BARLBLUEN,
     *   BARRBLUEN,BARRREDN,TAUAREDN,TAUABLUEN,BARRBLUENN,BARRREDNN,
     *   TAUAREDNN,TAUABLUENN,SAVERATIO,WEIGHT,RHO,MINT,FLUXDN,FLUXUP,
     *   IERROR,JTABLE)
C
      ELSE
C
C                        **Threshold for the number of cloudfree red
c                           or blue channel is not met.Set the error
c                           number as defined
C
         IFINISH=0
              IERROR=3
             TAUARED=5.0
            TAUABLUE=5.0
            TAUABLUENN=5.0
            TAUAREDNN=5.0
C           CALL ERROR(BARLBLUE,BARRBLUE,TAUABLUE,SAVERATIO,WEIGHT,
C     *      BARLRED,BARRRED,TAUARED,SDBLUE,DTAUBLUE,BARRBLUEN,
C     *      TAUABLUEN,SDRED,DTAURED,BARRREDN,TAUAREDN,TAUABLUENN,
C     *      TAUAREDNN,BARRBLUENN,BARRREDNN,IBLUE,IRED)
      ENDIF
C

C                        **Following else is for 'iferror'
C
         ELSE
C
C                        **If the threshold for 3.95 um is not met and
C                          data set is not cloudfree set the values
C                          of optical thicknesses of red and blue
C                          channels to indicate that the computations
C                          are not performed.
C
C
       IF  (I395 .LE.0) THEN
           IFINISH =0
            IERROR=2
             TAUARED=4.0
            TAUABLUE=4.0
            TAUABLUENN=4.0
            TAUAREDNN=4.0
C          CALL ERROR(BARLBLUE,BARRBLUE,TAUABLUE,SAVERATIO,WEIGHT,
C     *      BARLRED,BARRRED,TAUARED,SDBLUE,DTAUBLUE,BARRBLUEN,
C     *      TAUABLUEN,SDRED,DTAURED,BARRREDN,TAUAREDN,TAUABLUENN,
C     *      TAUAREDNN,BARRBLUENN,BARRREDNN,IBLUE,IRED)
       ELSE
         IF (ICLDBLUE.LT.NUMCLDFREE .OR.ICLDRED.LT.4*NUMCLDFREE)THEN
         IFINISH =0
           IERROR=3
             TAUARED=5.0
            TAUABLUE=5.0
            TAUABLUENN=5.0
            TAUAREDNN=5.0
C         CALL ERROR(BARLBLUE,BARRBLUE,TAUABLUE,SAVERATIO,WEIGHT,
C     *      BARLRED,BARRRED,TAUARED,SDBLUE,DTAUBLUE,BARRBLUEN,
C     *      TAUABLUEN,SDRED,DTAURED,BARRREDN,TAUAREDN,TAUABLUENN,
C     *      TAUAREDNN,BARRBLUENN,BARRREDNN,IBLUE,IRED)
         ENDIF
       ENDIF
C
C                        **Following endif  is for 'iferror'
C
          ENDIF
C
C                        **Call subroutine OUTPUT to write the OUTPUT
C
           IF( IFINISH .EQ.0) RETURN
        RETURN
        END


C***********************************************************************
       SUBROUTINE OPTION1(ISWATH,ILINE,CLDMSK_250 ,W375_SYN,
     * W470_SYN,W659_SYN,W865_SYN,
     * BLUE,RED,NUMBLUE,NUMRED,RHORED,RHOBLUE,THR375,IGRIDX,IGRIDY,
     * I395,ICLDRED,ICLDBLUE,START_1KM,END_1KM)
       IMPLICIT NONE
C       INCLUDE   'mod04L_inc.f'
C       INCLUDE   'mod04.inc'

C---------------------------------------------------------------------
c!F77
C
c!DESCRIPTION: The following subroutine finds the dark targets using a
C              threshold in the 3.95 micron channel. it averages the
C              cloud free pixels in red and blue to the 1 km resolution.
C              if too cloudy or too bright at 3.95 it leaves the value
C              as zero.
C
c!INPUT PARAMETERS:
C
C              CLDMSK_250      cloud mask 0=clear 1=cloudy
C                W395      Reflectance for wav = 3.95um
C                W470_SYN      Reflectance for wav = 0.47um
C                W659_SYN      Reflectance for wav = 0.66um
C              THR375      Threshold to detect dark pixels for 3.95um
C              IGRIDX      Number of pixel in 1*1 km resolution
C              IGRIDY      Number of pixels in 1*1 km resolution
C
C
c!OUTPUT PARAMETERS:
C                BLUE      Radiances in blue channel(cloudfree)
C                 RED      Radiances in red channel(cloudfree)
C             NUMBLUE      Number of cloudfree subpixels >12(blue)
C              NUMRED      Number of cloudfree subpixels >12(red)
C              RHORED      Reflectance in Red  channel(cloudfree)
C             RHOBLUE      Reflectance in blue channel(cloudfree)
C                I395      number of pixels indicating cloudfree
C             ICLDRED      Number of pixels in red channel(cloudfree)
C            ICLDBLUE      Number of pixels in blue channel(cloudfree)
C
c!REVISION HISTORY:
c 01/28/98 fhliang
c fixed prolog.
c
c Updated code to comply with most MODIS software standards.
c
c!TEAM-UNIQUE HEADER:
c
c Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
c GSFC, Greenbelt, MD
c
!REFERENCES AND CREDITS
C     WRITTEN BY:Shana Mattoo
C
!DESIGN NOTES:
C
c!END
C----------------------------------------------------------------------

        INTEGER I,I395,IBLUE,ICLDBLUE,ICLDRED,IGRIDX,IGRIDY,ISWATH,ILINE
        INTEGER NUMRED(4*IGRIDX,4*IGRIDY),NUMBLUE(2*IGRIDX,2*IGRIDY)
        INTEGER II,IIJ,IIK,IIP,IJ,IK,IMASK,IRED
        INTEGER J,JBLUE,JJ,JMASK,JRED,KKP,NUMCFR,NOWATER
        INTEGER START_1KM,END_1KM, NUM_indx_x
        INTEGER CLDMSK_250(4*ISWATH,4*ILINE)
        REAL W375_SYN(ISWATH,ILINE)
        REAL W470_SYN(2*ISWATH,2*ILINE)
        REAL w865_syn(4*ISWATH,4*ILINE)
        REAL W659_SYN(4*ISWATH,4*ILINE)
        REAL BLUE(2*IGRIDX,2*IGRIDY),RED(4*IGRIDX,4*IGRIDY)
        REAL RHOBLUE(2*IGRIDX,2*IGRIDY),RHORED(4*IGRIDX,4*IGRIDY)
        REAL THR375

        SAVE
C                        **i395>0 indicates that pixels under the
C                          threshold were found
C                          icldblue>0 indicates that there were pixels
C                          that were cloud free in blue channel
c                          icldred>0 indicates that there were pixels
C                          that were cloud free in red channel
C                          that were cloud free
        I395 =0
        ICLDRED =0
        ICLDBLUE =0
        NUM_indx_x=0
c
c                        ** intialize the arrays
c
          DO I = 1,2*IGRIDX
          DO J = 1,2*IGRIDY
              BLUE(I,J) =0.0
              NUMBLUE(I,J) = 0
           RHOBLUE(I,J)=RHOBLUE(1,1)
              ENDDO
              ENDDO
          DO I = 1,4*IGRIDX
          DO J = 1,4*IGRIDY
          RED(I,J) =0.0
          RHORED(I,J) =RHORED(1,1)
         NUMRED(I,J) = 0
         ENDDO
         ENDDO

C                         **Loop for  data set of 3.95 um

       DO 100 I = START_1KM,END_1KM
          IMASK = 4*I - 3
          IRED  = IMASK
          IBLUE = 2*I - 1


       DO 101 J = 1 , IGRIDY

          JMASK = 4*J - 3
          JBLUE = 2*J - 1
          JRED = JMASK
          NUMCFR = 0
          NOWATER=0

C                          check if 3.75 threshold is met.
C             dead detector line will be elminated and missing
C             data pixels will be ignored
        IF(W375_SYN(I,J).GT.0.0 .AND. W375_SYN(I,J) .LT. THR375) THEN
                         I395=I395+1
C
C                        **Count the cloudfree subpixels for each
C                          1 km pixel at 3.95 um using
C                          the cloudmask (of 250 meter resolution)
C
C                        **The following loops are inside the 1 km pixel
c                          on the 250 m pixels
C
          DO 102 II = IMASK , 4*I
          DO 103 JJ = JMASK , 4*J
              IF( CLDMSK_250 (II,JJ) .EQ. 1) THEN
                NUMCFR = NUMCFR + 1
             ENDIF
103       CONTINUE
102       CONTINUE
c                       **** count the water free area
         DO 202 II  = IMASK , 4*I
          DO 203 JJ = JMASK , 4*J
C  Treats missing pixels or noisy detector as water pixel
          IF(W865_SYN(II,JJ).GT.0.0 .AND.W659_SYN(II,JJ).GT.0.0)THEN
           IF((W865_SYN(II,JJ) - W659_SYN(II,JJ)).GT.0) THEN
                NOWATER= NOWATER + 1
           ENDIF
             ENDIF
203       CONTINUE
202       CONTINUE
C
C                        **If 12 out of 16 of the subpixels are
C                          cloudfree, than passed the test.

          IF( NUMCFR .GE. 12.AND.NOWATER.EQ.16) THEN

C
C                        **Count the cloudfree subpixels for 500 meter
C                           pixels
C
            DO 104 IJ = IBLUE, 2*I
              IIJ = 2*IJ - 1
               NUM_indx_x= NUM_indx_x+1
            DO 105 IK = JBLUE , 2*J
              IIK = 2*IK - 1
              NUMCFR = 0
              DO 106 IIP = IIJ , 2*IJ
              DO 107 KKP = IIK , 2*IK
                 IF (CLDMSK_250 (IIP,KKP) .EQ. 1) THEN
                    NUMCFR = NUMCFR+1
                ENDIF
107           CONTINUE
106           CONTINUE
C                        **If 3 out of 4 pixels are cloudfree add
C                          the blue channel reflectance
C
            IF(NUMCFR .GE. 3) THEN

C                        **save blue channel reflectance for all
c                          cloudfree pixels of 250 meters
C
C  IGNORE  missing pixels or noisy detector
                IF(W470_SYN(IJ,IK).GT.0.0) THEN
                  BLUE(NUM_indx_x,IK) =  W470_SYN(IJ,IK)
            NUMBLUE( NUM_indx_x,IK) = NUMBLUE( NUM_indx_x,IK) + 1
                   ICLDBLUE=ICLDBLUE+1
                ENDIF
            ENDIF
105         CONTINUE
104         CONTINUE
C
C                        **save red channel reflectance for all
c                          cloudfree pixels of 250 meters
C
            DO 108 IJ = IRED , 4*I
            DO 109 IK = JRED , 4*J
C
C                       **IF((W865_SYN(IJ,IK) - W659_SYN(IJ,IK)).GT.0)
C                          checks if the pixel is not water
C  Treats missing pixels or noisy detector as water pixes
          IF(W865_SYN(IJ,IK).GT.0.0 .AND.W659_SYN(IJ,IK).GT.0.0)THEN
            IF((W865_SYN(IJ,IK) - W659_SYN(IJ,IK)).GT.0) THEN
               IF (CLDMSK_250 (IJ,IK) .EQ. 1) THEN
                    RED( NUM_indx_x,IK) = W659_SYN(IJ,IK)
                    NUMRED(NUM_indx_x,IK) = NUMRED(NUM_indx_x,IK) + 1
                    ICLDRED=ICLDRED+1
                ENDIF
            ENDIF
          ENDIF
109         CONTINUE
108         CONTINUE
c                        **Finish the if ranges for clouds and 3.95
C                          threshold

          ENDIF
          ENDIF
101    CONTINUE
100    CONTINUE
        RETURN
        END


C********************************************************************
       SUBROUTINE SUBOPT2(ISWATH,ILINE,CLDMSK_250,W470_SYN,
     *  W659_SYN,W865_SYN,W213_SYN,START_500,END_500,
     *  START_250,END_250,START_1KM,END_1KM,TAUARED,tauablue,
     *  BLUE,RED,R0P86,NUMBLUE,NUMRED,RHORED,RHOBLUE,
     *  THR213MIN,THR213MAX,IGRIDX,IGRIDY,IBLUE,IRED,SDBLUE,SDRED,
     *  SD0P86,BARRBLUE,BARRRED,NLOWBLUE,NUPBLUE,NLOWRED,NUPRED,
     *  IERROR,BARLBLUE,BARLRED,BARL0P86,ICLDRED,ICLDBLUE,NUMCLDFREE,
     *  RBLUE,RRED,SAVERATIO,WEIGHT,NUMPERBLUE,NUMPERRED,MTHET0,
     *  MTHET,MDPHI,TAUABLUENN,TAUAREDNN,INT,PIT,FDOWN,FUP,OPTH,THET0,
     *  SBAR,THE,PHI,NUMWAV,LTHET0,LTHE,LPHI,LTAU,NFDOWN,NFUP,FINT,NEW,
     *  LRED,LBLUE,L0P86,NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,
     *  ARRAY1,ARRAY2,DIVRED,DIVBLUE,IFINISH,TAUA0P47,TAUA0P66,
     *  TAUA0P47_NN,TAUA0P66_NN,FLUXDN,FLUXUP,
     *  FLUXDN0P47,FLUXDN0P66,FLUXUP0P47,FLUXUP0P66,
     *  INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,NTABLE,MTABLE,IPROCE)
C----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C
C   This subroutine processes 10*10 pixel box for cloud detection and
C   finds the average reflectance for red and blue channels. Surface
C   Reflectance from wavelength 3.95 . This surface Reflectance and
C   average Reflectance for red and blue channel are send to lookup
C   table and optical thickness is derived.
C
C!INPUT PARAMETERS:
C
C      IGRIDX        Number of pixels in x-direction
C      IGRIDY        Number of pixels in y-direction
C      W470_SYN      Reflectance for wav=0.47um
C      W213_SYN      Reflectance for wav=2.13um
C      W659_SYN      Reflectance for wav=0.66um
C      CLDMSK_250    Cloud mask at 250 m resolution (1=cloudy,o=clear)
C      MTHET0        Measured solar zenith angle in deg.
C      MTHET         Measured viewangle from ground in deg.
C      MDPHI         Measured relative azimuth angle deg.
C      WAV           Wavelength
C      OPTH          Optical depth
C      THET0         Solar Angle
C      FDOWN         Flux down
C      FUP           Flux up
C      THE           View angle
C      PHI           Azimuth
C      SBAR          Sbar
C      INT           Reflectance
C      PIT           Transmission factor
C      THR213MIN     Minumum threshold for 2.13 micro wavelength.
C      THR213MAX     Maximum threshold for 2.13 micro wavelength.
C
C!OUTPUT PARAMETERS:
C
C     TAUARED        Derived optical thickness for red channel.
C     TAUABLUE       Derived optical thickness for blue channel
C     SDBLUE         Standard deviation for Reflectance blue channel.
C     SDRED          Standard deviation for Reflectance red channel.
C     SAVERATIO      Ratio of aerosol path radiance of red to blue
C     WEIGHT         Relative contribution of smoke/sulfate to dust.
C
C!REVISION HISTORY:
C $Log: modaerlandv3.f,v $
C 01/28/98 fhliang
C fixed prolog.
C
C Revision 1.5  1995/11/30  00:59:25  rhucek
C defined ISULP and ISMK in subroutine OUTPUT prolog
C
C Revision 1.4  1995/11/30  00:46:49  rhucek
C Added/completed prologs for subroutines INTERP, LOCATE, MOMENT,
C OPTICAL, OUTPUT, READTM
C
C Revision 1.3  1995/11/29  16:06:14  vlin
C Updated prolog (line 19)
C
C Revision 1.2  1995/11/17  12:53:51  vlin
C Before code walkthrough
C
C Revision 1.1  1995/10/17  12:59:52  rhucek
C Initial revision
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol Retrieval Team at NASA GSFC, Greenbelt, MD
C
C!REFERENCES AND CREDITS
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C 2 lines added by vlin
C
      IMPLICIT NONE
      INTEGER IFINISH,IGRIDX,IGRIDY,LPHI,LTAU,LTHE,LTHET0,NUMWAV,
     &        ISWATH,ILINE,JTABLE,NTABLE,MTABLE,KTABLE,LTABLE,IPROCE,
     &        IITABLE,TTABLE
      REAL DIVBLUE,DIVRED
      INTEGER NUMRED(4*IGRIDX,4*IGRIDY),NUMBLUE(2*IGRIDX,2*IGRIDY)
      INTEGER I213,IBLUE,ICLDBLUE,ICLDRED,ITIME
      INTEGER IERROR,IOPT,IRED,NBLUE,NRED
      INTEGER START_500,END_500,START_250,END_250,START_1KM,END_1KM
      REAL W470_SYN(2*ISWATH,2*ILINE)
      REAL W213_SYN(2*ISWATH,2*ILINE)
      REAL w865_SYN(4*ISWATH,4*ILINE)
      REAL W659_SYN(4*ISWATH,4*ILINE)
      INTEGER CLDMSK_250(4*ISWATH,4*ILINE),I
      REAL BLUE(2*IGRIDX,2*IGRIDY),RED(4*IGRIDX,4*IGRIDY)
      REAL RHORED(4*IGRIDX,4*IGRIDY),R0P86(4*IGRIDX,4*IGRIDY)
      REAL RHOBLUE(2*IGRIDX,2*IGRIDY),LRED((4*IGRIDX)*(4*IGRIDY))
      REAL LBLUE((2*IGRIDX)*(2*IGRIDY)),L0P86((4*IGRIDX)*(4*IGRIDY))
      REAL RBLUE((2*IGRIDX)*(2*IGRIDY)),RRED((4*IGRIDX)*(4*IGRIDY))
      REAL MTHET0,MTHET,MDPHI
      REAL BARLBLUE,BARRBLUE,BARRRED,BARLRED,BARL0P86,
     &     TAUABLUE,TAUARED
      REAL ARRAY1((4*IGRIDX)*(4*IGRIDY)),ARRAY2((4*IGRIDX)*(4*IGRIDY))
      REAL BARLBLUEN,BARLREDN,BARRBLUEN,BARRBLUENN
      REAL BARRREDN,BARRREDNN,SAVERATIO,WEIGHT
      REAL DTAUBLUE,DTAURED,SDBLUE,SDRED,SD0P86,TAUABLUEN
      REAL TAUABLUENN,TAUAREDN,TAUAREDNN

      REAL THET0(LTHET0),THE(LTHE),PHI(LPHI)
      REAL OPTH(LTAU,NUMWAV),SBAR(LTAU,NUMWAV)
C      REAL FDOWN(LTAU,LTHET0,NUMWAV)
      REAL FDOWN(LTHET0,LTAU,NUMWAV)
      REAL FUP(LTHET0,LTAU,NUMWAV)
      REAL FLUXDN(NUMWAV),FLUXUP(NUMWAV)
      REAL FLUXDN0P47(NTABLE),FLUXUP0P47(NTABLE)
      REAL FLUXDN0P66(NTABLE),FLUXUP0P66(NTABLE)
C      REAL INT(LPHI,LTHE,LTAU,LTHET0,NUMWAV),PIT(LTHE,LTAU,NUMWAV)
      REAL INT(LTHET0,LPHI,LTHE,LTAU,NUMWAV),PIT(LTHE,LTAU,NUMWAV)
      REAL NFDOWN(LTAU),NFUP(LTAU),NEWINT(LPHI,LTHE,LTAU),FINT(LTHE)
      REAL NEW(LPHI),NEWINT2(LTHE,LTAU),IMES(LTAU),WAVOPT(LTAU)
      REAL NEWN(LPHI),NEWN2(LPHI),FT(LTAU)
C
C added by DAC 5/1/98
C
      REAL TAUA0P47(NTABLE),TAUA0P66(NTABLE)
      REAL TAUA0P47_NN(NTABLE),TAUA0P66_NN(NTABLE)
      REAL OPTH_A(LTAU,NUMWAV,NTABLE)
C      REAL FDOWN_A(LTAU,LTHET0,NUMWAV,NTABLE),INTW(31),PITW(13)
      REAL FDOWN_A(LTHET0,LTAU,NUMWAV,NTABLE),INTW(31),PITW(13)
      REAL FUP_A(LTHET0,LTAU,NUMWAV,NTABLE)
      REAL SBAR_A(LTAU,NUMWAV,NTABLE)
C      REAL INT_A(LPHI,LTHE,LTAU,LTHET0,NUMWAV,NTABLE)
      REAL INT_A(LTHET0,LPHI,LTHE,LTAU,NUMWAV,NTABLE)
      REAL PIT_A(LTHE,LTAU,NUMWAV,NTABLE)
      REAL RHO(3,2),MINT(3,2)
C
C end
C

      REAL THR213MIN,THR213MAX
      INTEGER NUMCLDFREE,NUMPERBLUE,NUMPERRED
      INTEGER NLOWBLUE,NUPBLUE,NLOWRED,NUPRED

C
C Initialization
C

      IFINISH=1
      IERROR=0

      CALL OPTION2(ISWATH,ILINE,CLDMSK_250,W213_SYN,W470_SYN,
     *     W659_SYN,W865_SYN,BLUE,RED,NUMBLUE,START_500,END_500,
     *     NUMRED,RHORED,RHOBLUE,R0P86,THR213MIN,THR213MAX,
     *     DIVRED,DIVBLUE,IGRIDX,IGRIDY,I213,ICLDBLUE,ICLDRED)

C
C I213>0 and Icld>0 indicate that the threshold for 2.13 micron
C is met and the data set is cloud free as defined.
C
C

      IF(I213 .GT. 0 .AND. ICLDBLUE .GE. NUMCLDFREE .AND.
     *  ICLDRED .GE. 4*NUMCLDFREE) THEN

C
C Call to subroutine SORT to sort the arrays into ascending order
C

        IOPT = 2
        CALL SORT(NUMRED,NUMBLUE,RED,BLUE,R0P86,RHORED,RHOBLUE,
     *    LRED,LBLUE,L0P86,RRED,RBLUE,NBLUE,NRED,IGRIDX,IGRIDY,IOPT)

C
C Call to subroutine PERCENTILE to select only part of the array
C and find the average between given range of percentiles only if
C the thresholds for number of points in 0.47 and 0.66 micron
C channels are met.
C

        IF(NBLUE .GE. NUMPERBLUE .AND. NRED .GE. NUMPERRED) THEN

          CALL PERCENTILE(NBLUE,LBLUE,RBLUE,NRED,LRED,RRED,BARLBLUE,
     *    BARRBLUE,BARRRED,BARLRED,L0P86,BARL0P86,NLOWBLUE,NUPBLUE,
     *    NLOWRED,NUPRED,IGRIDX,IGRIDY,SDBLUE,SDRED,SD0P86,BARRBLUEN,
     *    BARRREDN,BARRBLUENN,BARRREDNN,ARRAY1,ARRAY2,IBLUE,IRED)

C
C  Adding standard deviation of reflectance at 0.47 and 0.66 micron channels
C  (this is used to estimate the uncertainties of retrieval of optical thickness)
C

          BARLREDN=BARLRED+SDRED
          BARLBLUEN=BARLBLUE+SDBLUE

C
C Assign apparent and surface reflectances (for red and blue channels) under 3 different
C conditions to 2-D arrarys to save computational time
C

          RHO(1,1)=BARRBLUE
          MINT(1,1)=BARLBLUE
          RHO(1,2)=BARRRED
          MINT(1,2)=BARLRED
          RHO(2,1)=BARRBLUEN
          MINT(2,1)=BARLBLUE
          RHO(2,2)=BARRREDN
          MINT(2,2)=BARLRED
c         RHO(3,1)=BARRBLUENN
          RHO(3,1)=BARRBLUE
          MINT(3,1)=BARLBLUEN
c         RHO(3,2)=BARRREDNN
          RHO(3,2)=BARRRED
          MINT(3,2)=BARLREDN

C
C Call to subroutine optical to derive optical thicknesses
C at 0.47 and 0.66 um channels using 4 different aerosol
C models
C

Change          KTABLE=1
Change          LTABLE=MTABLE

          KTABLE=MTABLE
          LTABLE=MTABLE

          IF(IPROCE.EQ.5) THEN
c             KTABLE=4
c             LTABLE=4
              KTABLE=1
              LTABLE=1
          ENDIF

          IITABLE=1
c          write(37,*)'KTABLE,LTABLE',KTABLE,LTABLE
          DO JTABLE=KTABLE,LTABLE
            TTABLE=JTABLE

            CALL SELECT_LOOKUP(INT_A,PIT_A,FDOWN_A,FUP_A,OPTH_A,SBAR_A,
     *        INT,PIT,FDOWN,FUP,OPTH,SBAR,NUMWAV,LTHET0,LTHE,LPHI,
     *        LTAU,NTABLE,JTABLE)

            CALL OPTICAL(INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,
     *        NUMWAV,LTHET0,LTHE,LPHI,LTAU,MTHET0,MTHET,MDPHI,BARLBLUE,
     *        BARRBLUE,BARRRED,BARLRED,TAUABLUE,TAUARED,NFDOWN,NFUP,FINT,NEW,
     *        NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,BARLREDN,BARLBLUEN,
     *        BARRBLUEN,BARRREDN,TAUAREDN,TAUABLUEN,BARRBLUENN,BARRREDNN,
     *        TAUAREDNN,TAUABLUENN,SAVERATIO,WEIGHT,RHO,MINT,FLUXDN,FLUXUP,
     *        IERROR,IITABLE)

            IF(TAUABLUE.GT.0.0.AND.TAUARED.GT.0.0) THEN  
              TAUA0P47(TTABLE)=TAUABLUE
              TAUA0P66(TTABLE)=TAUARED
              TAUA0P47_NN(TTABLE)=TAUABLUENN
              TAUA0P66_NN(TTABLE)=TAUAREDNN
              FLUXDN0P47(TTABLE)=FLUXDN(1)
              FLUXDN0P66(TTABLE)=FLUXDN(2)
              FLUXUP0P47(TTABLE)=FLUXUP(1)
              FLUXUP0P66(TTABLE)=FLUXUP(2)
            ENDIF

          ENDDO

C
C Thresholds for the number of cloudfree of 0.47 and 0.66 micron channels if
C not met set the error code
C

        ELSE

          IFINISH =0
C          IERROR=1
          IERROR=3
          TAUARED=-3.0
          TAUABLUE=-3.0
          TAUABLUENN=-3.0
          TAUAREDNN=-3.0

C         CALL ERROR(BARLBLUE,BARRBLUE,TAUABLUE,SAVERATIO,WEIGHT,
C     *        BARLRED,BARRRED,TAUARED,SDBLUE,DTAUBLUE,BARRBLUEN,
C     *        TAUABLUEN,SDRED,DTAURED,BARRREDN,TAUAREDN,TAUABLUENN,
C     *        TAUAREDNN,BARRBLUENN,BARRREDNN,IBLUE,IRED)

        ENDIF

      ELSE
C
C Thresholds for 2.1 micron reflectance and number of cloud-free
C pixels is not met. Set the error index
C

        IF(I213 .LE.0) THEN
          IFINISH =0
C          IERROR=5
          IERROR=2
          TAUARED=-7.0
          TAUABLUE=-7.0
          TAUABLUENN=-7.0
          TAUAREDNN=-7.0

C        CALL ERROR(BARLBLUE,BARRBLUE,TAUABLUE,SAVERATIO,WEIGHT,
C     *       BARLRED,BARRRED,TAUARED,SDBLUE,DTAUBLUE,BARRBLUEN,
C     *       TAUABLUEN,SDRED,DTAURED,BARRREDN,TAUAREDN,TAUABLUENN,
C     *       TAUAREDNN,BARRBLUENN,BARRREDNN,IBLUE,IRED)
        ELSE

        IF(ICLDBLUE.LT.NUMCLDFREE .OR.ICLDRED.LT.4*NUMCLDFREE)THEN
          IFINISH =0
          IERROR=3
          TAUARED=-5.0
          TAUABLUE=-5.0
          TAUABLUENN=-5.0
          TAUAREDNN=-5.0

C       CALL ERROR(BARLBLUE,BARRBLUE,TAUABLUE,SAVERATIO,WEIGHT,
C     *      BARLRED,BARRRED,TAUARED,SDBLUE,DTAUBLUE,BARRBLUEN,
C     *      TAUABLUEN,SDRED,DTAURED,BARRREDN,TAUAREDN,TAUABLUENN,
C     *      TAUAREDNN,BARRBLUENN,BARRREDNN,IBLUE,IRED)
        ENDIF

        ENDIF
C
C                        **Following endif  is for 'iferror'
C
      ENDIF

      IF(IFINISH.EQ.0) RETURN
      RETURN
      END


C***********************************************************************
       SUBROUTINE OPTION2(ISWATH,ILINE,CLDMSK_250,W213_SYN,
     *W470_SYN,W659_SYN,W865_SYN,BLUE,RED,NUMBLUE,START_500,END_500,
     *NUMRED,RHORED,RHOBLUE,R0P86,THR213MIN,THR213MAX,DIVRED,DIVBLUE,
     *IGRIDX,IGRIDY,I213,ICLDBLUE,ICLDRED)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              The following subroutine finds the dark targets using a
C              threshold in the 2.13 micron channel.it averages the
C              cloud free pixels in red channel to the 0.5 km resolution.
C              If too cloudy or too bright at 2.13 it leaves the value
C              as zero.
C
C!INPUT PARAMETERS:
C
C           CLDMSK_250      Cloud mask at 250 m resolution (0=cloudy,1=clear)
C             W213_SYN      Reflectance for at 2.13 um
C             W470_SYN      Reflectance for at 0.47 um
C             W659_SYN      Reflectance for at 0.66 um
C               THR213      Threshold to detect dark pixels for 2.13 um
C               DIVRED      Ratio to compute surface Reflectance at 0.66 um
C              DIVBLUE      Ratio to compute surface Reflectance at 0.47 um
C               IGRIDX      Number of pixels in 1x1 km resolution
C               IGRIDY      Number of pixels in 1x1 km resolution
C
C!OUTPUT PARAMETERS:
C
C                 BLUE      Radiances in blue channel(cloudfree)
C                  RED      Radiances in red channel(cloudfree)
C              NUMBLUE      Number of cloudfree subpixels >12(blue)
C               NUMRED      Number of cloudfree subpixels >12(red)
C               RHORED      Surface Reflectance in Red  ch(cloudfree)
C              RHOBLUE      Surface Reflectance in blue ch(cloudfree)
C                 I213      number of pixels indicating cloudfree
C              ICLDRED      Number of pixels in red channel(cloudfree)
C             ICLDBLUE      Number of pixels in blue channel(cloudfree)
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
C GSFC, Greenbelt, MD
C
C!REFERENCES AND CREDITS
C     WRITTEN BY:Shana Mattoo
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
       IMPLICIT NONE
       INTEGER I,I213,ICLDBLUE,ICLDRED,IGRIDX,IGRIDY,ISWATH,ILINE
       INTEGER NUMRED(4*IGRIDX,4*IGRIDY),NUMBLUE(2*IGRIDX,2*IGRIDY)
       INTEGER II,IJ,IK,IMASK,IRED
       INTEGER J,JJ,JMASK,JRED,NUMCFR,NOWATER
       INTEGER START_500,END_500
       REAL DIVBLUE,DIVRED
       REAL W470_SYN(2*ISWATH,2*ILINE)
       REAL W213_SYN(2*ISWATH,2*ILINE)
       REAL W865_SYN(4*ISWATH,4*ILINE)
       REAL W659_SYN(4*ISWATH,4*ILINE)
       INTEGER CLDMSK_250(4*ISWATH,4*ILINE)
       INTEGER NUM_indx_x
       REAL BLUE(2*IGRIDX,2*IGRIDY),RED(4*IGRIDX,4*IGRIDY),
     &      R0P86(4*IGRIDX,4*IGRIDY)
       REAL RHORED(4*IGRIDX,4*IGRIDY)
       REAL RHOBLUE(2*IGRIDX,2*IGRIDY)
       REAL THR213MIN,THR213MAX
       SAVE

C
C  Initialization of variables and arrays
C

       I213 =0
       ICLDBLUE =0
       ICLDRED  =0
       NUM_indx_x=0

       DO J = 1,2*IGRIDY
         DO I = 1,2*IGRIDX
           BLUE(I,J)=0.0
           RHOBLUE(I,J)=0.0
           NUMBLUE(I,J)=0
         ENDDO
       ENDDO

       DO J = 1,4*IGRIDY
         DO I = 1,4*IGRIDX
           R0P86(I,J)=0.0
           RED(I,J)=0.0
           RHORED(I,J)=0.0
           NUMRED(I,J)=0
         ENDDO
       ENDDO

C
C Loop for 500x500 meter resolution based upon 2.13 um channel
C

       DO 200 I = START_500,END_500
           IMASK=2*I-1
           IRED=IMASK
           NUM_indx_x=NUM_indx_x+1

         DO 201 J = 1,2*IGRIDY
           JMASK=2*J-1
           JRED=JMASK

           NUMCFR=0
           NOWATER=0
C
C Check if therhold for 2.13 um is met
C (missing pixels or noisy detectors are ignored)
C

          IF(W213_SYN(I,J) .LE. THR213MAX .AND.
     *       W213_SYN(I,J) .GT. THR213MIN) THEN

             I213 = I213+ 1

C
C Count the cloud-free 250x250 m subpixels in 500x500 m range
C

          DO 202 II = IMASK , 2*I
            DO 203 JJ = JMASK , 2*J

              IF(CLDMSK_250(II,JJ) .EQ. 1) THEN

               NUMCFR = NUMCFR + 1

              ENDIF

203         CONTINUE
202       CONTINUE

C
C Treats missing or noisy detector as water pixels to be ignored
C

C          DO 302 II = IMASK , 2*I
C            DO 303 JJ = JMASK , 2*J
C
C              IF(W865_SYN(II,JJ) .GT. 0.0 .AND.
C     *           W659_SYN(II,JJ) .GT. 0.0) THEN
C
C              IF(W865_SYN(II,JJ)-W659_SYN(II,JJ) .GT. 0.0) THEN
C
C              NOWATER=NOWATER+1
C
C            ENDIF
C
C          ENDIF
C
C303       CONTINUE
C302       CONTINUE

C
C Compute apparant Reflectance and surface reflectance for 0.66 um channel
C for cloud and water free pixels in 250 m resolution and treats missing
C or noisy pixels as water pixels
C

          DO 204 IJ = IRED , 2*I
            DO 205 IK = JRED , 2*J

              IF(W865_SYN(IJ,IK) .GT. 0.0 .AND.
     *           W659_SYN(IJ,IK) .GT. 0.0) THEN

C              IF((W865_SYN(IJ,IK)-W659_SYN(IJ,IK)).GT.0) THEN
C
C according to Eric Vermote
C
               IF((W865_SYN(IJ,IK)-W659_SYN(IJ,IK))/
     *            (W865_SYN(IJ,IK)+W659_SYN(IJ,IK)).GT.0.1) THEN

                NOWATER=NOWATER+1

                IF(CLDMSK_250(IJ,IK) .EQ. 1) THEN

                  R0P86(NUM_indx_x,J)=R0P86(NUM_indx_x,J)+W865_SYN(IJ,IK)
                  RED(NUM_indx_x,J)=RED(NUM_indx_x,J)+ W659_SYN(IJ,IK)
                  RHORED(NUM_indx_x,J)=W213_SYN(I,J)/DIVRED
                  NUMRED(NUM_indx_x,J)=NUMRED(NUM_indx_x,J)+1
                  ICLDRED=ICLDRED+1

                ENDIF

              ENDIF

            ENDIF

205         CONTINUE
204       CONTINUE

C
C Check if at least 3 out of 4 pixels are cloud free and all pixels
C are no water pixels in 250 m resolution
C Derive surface reflectance for 0.47 um channel (ignore missing or noisy pixels)
C

C          IF(NUMCFR .GE. 3 .AND .NOWATER .EQ. 4) THEN
          IF(NUMCFR .GE. 4 .AND .NOWATER .EQ. 4) THEN

            IF(W470_SYN(I,J ) .GT. 0.0) THEN

              RHOBLUE(NUM_indx_x,J)=W213_SYN(I,J)/DIVBLUE
              BLUE(NUM_indx_x,J)=W470_SYN(I,J )
              NUMBLUE(NUM_indx_x,J)=NUMBLUE(NUM_indx_x,J)+1
              ICLDBLUE = ICLDBLUE + 1

            ENDIF

          ENDIF

C
C Derive average apparent reflectance for 0.66 and 0.86 um channels
C

          IF(NUMRED(NUM_indx_x,J) .GT. 0) THEN

            RED(NUM_indx_x,J)=RED(NUM_indx_x,J)/NUMRED(NUM_indx_x,J)
            R0P86(NUM_indx_x,J)=R0P86(NUM_indx_x,J)/NUMRED(NUM_indx_x,J)

          ENDIF

         ENDIF

201      CONTINUE
200    CONTINUE

       RETURN
       END

C**********************************************************************
      SUBROUTINE SORT(NUMRED,NUMBLUE,RED,BLUE,R0P86,RHORED,RHOBLUE,
     *LRED,LBLUE,L0P86,RRED,RBLUE,NBLUE,NRED,IGRIDX,IGRIDY,IOPT)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              This subroutine calls the subroutine SHLSRT_LAND to sort
C              arrays into ascending order.
C
C!INPUT PARAMETERS:
C
C               NUMRED      Number of cloudfree subpixels >12 (0.66 um)
C              NUMBLUE      Number of cloudfree subpixels >12 (0.47 um)
C                  RED      Apparent Reflectance at 0.66 um (cloudfree)
C                 BLUE      Apparent Reflectance at 0.47 um (cloudfree)
C               RHORED      Surface Reflectance at 0.66 um (cloudfree)
C              RHOBLUE      Surface Reflectance at 0.47 um (cloudfree)
C               IGRIDX      Number of pixel in 1x1 km resolution
C               IGRIDY      Number of pixels in 1x1 km resolution
C                 IOPT      Option for computing optical thicknesses
C
C!OUTPUT PARAMETERS:
C
C                LRED      Sorted array for apparent Reflectance at 0.66 um
C                LBLUE     Sorted array for apparent Reflectance at 0.47 um
C                RRED      Sorted array for surface Reflectance at 0.66 um
C                RBLUE     Sorted array for surface Reflectance at 0.47 um
C                NRED      Sorted array for number of pixels in 0.66 um
C                NBLUE     Sorted array for number of pixels at 0.47 um
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
C GSFC, Greenbelt, MD
C
C!REFERENCES AND CREDITS
C     WRITTEN BY:Shana Mattoo
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
       IMPLICIT NONE
       INTEGER I,J,IJJ,IGRIDX,IGRIDY,IOPT,NBLUE,NRED
       INTEGER NUMBLUE(2*IGRIDX,2*IGRIDY),NUMRED(4*IGRIDX,4*IGRIDY)
       REAL  RED(4*IGRIDX,4*IGRIDY),LRED((4*IGRIDX)*(4*IGRIDY))
       REAL  R0P86(4*IGRIDX,4*IGRIDY),L0P86((4*IGRIDX)*(4*IGRIDY))
       REAL  RHORED(4*IGRIDX,4*IGRIDY)
       REAL  RRED((4*IGRIDX)*(4*IGRIDY)),RBLUE((2*IGRIDX)*(2*IGRIDY))
       REAL  BLUE(2*IGRIDX,2*IGRIDY),LBLUE((2*IGRIDX)*(2*IGRIDY))
       REAL  RHOBLUE(2*IGRIDX,2*IGRIDY)
       SAVE

C
C Initialization
C

       NRED=0
       NBLUE=0
       IJJ =2
       IF(IOPT .EQ.1) IJJ=4

C
C Convert 2-D arrays to 1-D arrays
C

       DO J = 1,IJJ*IGRIDY
         DO I = 1,IJJ*IGRIDX
           IF(NUMRED(I,J) .GT. 0)THEN
             NRED=NRED+1
             LRED(NRED)=RED(I,J)
             L0P86(NRED)=R0P86(I,J)
             RRED(NRED)=RHORED(I,J)
           ENDIF
         ENDDO
       ENDDO

       DO J = 1,2*IGRIDY
         DO I = 1,2*IGRIDX
           IF(NUMBLUE(I,J) .GT. 0)THEN
             NBLUE=NBLUE+1
             LBLUE(NBLUE)=BLUE(I,J)
             RBLUE(NBLUE)=RHOBLUE(I,J)
           ENDIF
         ENDDO
       ENDDO

C
C Sort into ascending order
C

       CALL  SHLSRT_LAND(NRED,LRED,RRED)
       CALL  SHLSRT_LAND(NBLUE,LBLUE,RBLUE)

       RETURN
       END


C**********************************************************************
       SUBROUTINE SHLSRT_LAND(N,ARR1,ARR2)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C               Adopted from numerical recepies Sorts arr1ay 'arr1' of
C               length 'n' into ascending order.  ('arr1' is replaced
C               on OUTPUT by its sorted rearr1angement)
C
C!INPUT PARAMETERS:
C
C                 N    Length of ARR1
C              ARR1    Unsorted array ARR1
C              ARR2    Unsorted array ARR2
C
C!OUTPUT PARAMETERS:
C
C              ARR1    Sorted array ARR1
C              ARR2    Sorted array ARR2
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
C GSFC, Greenbelt, MD
C
C!REFERENCES AND CREDITS
C     WRITTEN BY:Numerical Recepies
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
      IMPLICIT NONE
      INTEGER I,J,K,L,M,N,NN,LOGNB2
      REAL  ALN2I,TINY
      PARAMETER (ALN2I = 1.4426950, TINY = 1.E-5)
      REAL  ARR1(N),ARR2(N)
      REAL Q,T
      SAVE

      LOGNB2 = INT(ALOG(FLOAT(N))*ALN2I+TINY)
      M = N

      DO 12 NN = 1,LOGNB2

        M = M/2
        K = N-M

        DO 11 J = 1,K

          I = J
3         CONTINUE
          L = I+M
          IF(ARR1(L).LT.ARR1(I)) THEN
            T = ARR1(I)
            ARR1(I) = ARR1(L)
            ARR1(L) = T
            Q = ARR2(I)
            ARR2(I) = ARR2(L)
            ARR2(L) = Q
            I = I-M
            IF(I.GE.1) GO TO 3
          ENDIF

11      CONTINUE

12    CONTINUE

      RETURN
      END

C********************************************************************
      SUBROUTINE PERCENTILE(NBLUE,LBLUE,RBLUE,NRED,LRED,RRED,BARLBLUE,
     *   BARRBLUE,BARRRED,BARLRED,L0P86,BARL0P86,NLOWBLUE,NUPBLUE,
     *   NLOWRED,NUPRED,IGRIDX,IGRIDY,SDBLUE,SDRED,SD0P86,BARRBLUEN,
     *   BARRREDN,BARRBLUENN,BARRREDNN,ARRAY1,ARRAY2,IBLUE,IRED)
      IMPLICIT NONE
C-----------------------------------------------------------------------
c!F77
C
c!DESCRIPTION:  This subroutine averages the arrays between the
C               percentiles desired. the percentile are input given by
C               parameter values of NLOWBLUE,NUPBLUE,NLOWRED,NUPRED
C
c!INPUT PARAMETERS:
C      NBLUE  Total number of pixels before averaging(blue)
C      LBLUE  Apparent reflectance before averaging(blue)
C      RBLUE  surface reflanctanve before averaging(blue)
C      NRED   Total number of pixels before averaging(red)
C      LRED   Apparent reflectance before averaging(red)
C      RRED   surface reflanctanve before averaging(red)
C      L0P86  Apparent reflectance before averagig (0.86 micron)
C      NLOWBLUE  Sets value for lower  percentile(blue)
C      NUPBLUE   Sets value for upper  percentile(blue)
C      NLOWBLUE  Sets value for lower  percentile(red)
C      NUPBLUE   Sets value for upper  percentile(red)
C      IGRIDX    pixel (1*1 km resolution)
C      IGRIDY    pixel( 1*1 km resolution)
C      ARRAY1    general array
C      ARRAY2    general array
C
c!OUTPUT PARAMETERS:
C      BARLBLUE  Averaged Reflectance in blue channel.
C      BARRBLUE  Averaged sur. Reflectance in blue channel.
C      BARLRED   Averaged Reflectance in red channel.
C      BARRRED   Averaged sur. Reflectance in red channel.
C      BARL0P86  Averaged Reflectance in 0.86 micron channel.
C      SDBLUE    Standard deviation blue
C      SDRED     Standard deviation red
C      BARRBLUEN Averaged value of blue Reflectance at6 loc.
C      BARRREDN  Averaged value of red Reflectance at6 loc.
C      BARRBLUENN Averaged value of blue Reflectance+ sd at6 loc.
C      BARRBLUENN Averaged value of red Reflectance+ sd at6 loc.
C      IBLUE      Number of pixels used from low to high percentile (blue).
C      IRED       Number of pixels used from low to high percentile (red).
C
c!REVISION HISTORY:
c 01/28/98 fhliang
c fixed prolog.
c
c Updated code to comply with most MODIS software standards.
C
c!TEAM-UNIQUE HEADER:
c
c Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
c GSFC, Greenbelt, MD
c
!REFERENCES AND CREDITS
C     WRITTEN BY:Shana Mattoo
C
!DESIGN NOTES:
C
c!END
C----------------------------------------------------------------------
       INTEGER IBLUE,IDUMMY,IGRIDX,IGRIDY,II,IK,IRED,I0P86
       REAL LBLUE((2*IGRIDX)*(2*IGRIDY)),RBLUE((2*IGRIDX)*(2*IGRIDY))
       REAL LRED((4*IGRIDX)*(4*IGRIDY)),RRED((4*IGRIDX)*(4*IGRIDY))
       REAL L0P86((4*IGRIDX)*(4*IGRIDY))
       REAL ARRAY1((4*IGRIDX)*(4*IGRIDY)),ARRAY2((4*IGRIDX)*(4*IGRIDY))
       REAL BARRREDN,SDRED,SDBLUE,BARRREDNN
       INTEGER N10BLUE,N10RED,N40BLUE,N40RED,NBLUE,NRED,NN
       INTEGER NLOWBLUE,NLOWRED,NUMAVE,NUMAVE1,NUMBER
       INTEGER NUPBLUE,NUPRED,nnn
       REAL BARLBLUE,BARLRED,BARRBLUE,BARRBLUEN,BARRBLUENN
       REAL BARRRED,BB,DUMMY,BARL0P86,SD0P86

C                        ** compute the percentile from
c                            input parameters
         SAVE
         nnn=0
        N10BLUE=0
        N40BLUE=0
        N10RED=0
        N40RED=0
       N10BLUE=(NBLUE*NLOWBLUE/100)
        IF(N10BLUE .LE.0) N10BLUE=1
       N40BLUE=(NBLUE*NUPBLUE/100)
       N10RED=(NRED*NLOWRED/100)
        IF(N10RED .LE.0) N10RED=1
       N40RED=(NRED*NUPRED/100)
       BARLBLUE=0.0
       BARRBLUE=0.0
       IBLUE=0
       BARLRED=0.0
       BARRRED=0.0
       BARL0P86=0.0
       IRED=0
       I0P86=0
       BARRBLUEN=0.0
       BARRREDN=0.0
       BARRBLUENN=0.0
       BARRREDNN=0.0
       SDRED=0.0
       SDBLUE=0
       NUMBER=0
       IF(N10BLUE .EQ.1 .AND. N40BLUE .EQ. 1)RETURN
       IF(N10RED .EQ.1 .AND. N40RED .EQ. 1) RETURN


C
C                        **For blue channel
C
       DO 100 IK = N10BLUE , N40BLUE
        IBLUE=IBLUE+1
        ARRAY1(IBLUE)=LBLUE(IK)
        ARRAY2(IBLUE)=RBLUE(IK)
100    CONTINUE
C
C                        **Call to subroutine moment returns
C                         average,standard deviation and numave
C                          and locates the position of average ave
C                          for array1 ie reflectance in blue channel.
C                          numave1 finds the location of the
C
        CALL MOMENT(ARRAY1,IBLUE,BARLBLUE,SDBLUE,NUMAVE,NUMAVE1)

C
C                        **Call to subroutine moment returns average
C                          and standard deviation  and numave
C                          are  dummy values for the surface
C                         reflectance
C
        CALL MOMENT(ARRAY2,IBLUE,BARRBLUE,DUMMY,IDUMMY,IDUMMY)
C
C                        **Surface reflectance values are averaged
C                          between 6 positions if
C                          standard deviation is greater than zero
C
       DO II = 1,2
       nn=0
       IF( II .EQ. 1) NN=NUMAVE
       IF( II .EQ. 2) NN=NUMAVE1
         NUMBER=0
         IF( SDBLUE .GT. 0.0 .AND.NN .GT.3.AND.NN.LT.IBLUE-3)THEN
           DO 101 IK = NN-3 , NN+3
              NUMBER=NUMBER+1
              ARRAY1(NUMBER)=ARRAY2(IK)
101        CONTINUE
C
C                        **Call to subroutine moment returns
C                         average for surface reflectance
C
           CALL MOMENT(ARRAY1,NUMBER,BB,DUMMY,IDUMMY,IDUMMY)
            IF( II .EQ.1) BARRBLUEN=BB
            IF( II .EQ.2) BARRBLUENN=BB
C
         ELSE
C
C                        **Surface reflectance values is set to
C                          first computed value if the standard
C                           is zero.
C
           BARRBLUEN = BARRBLUE
           BARRBLUENN = BARRBLUE
         ENDIF
       ENDDO

C
C  For 0.86 micron channel
C

       DO 199 IK = N10RED , N40RED
         I0P86=I0P86+1
         ARRAY1(I0P86)=L0P86(IK)
199    CONTINUE

       CALL MOMENT(ARRAY1,I0P86,BARL0P86,SD0P86,NUMAVE,NUMAVE1)

C
C  For red channel
C

       DO 200 IK = N10RED , N40RED
         IRED=IRED+1
         ARRAY1(IRED)=LRED(IK)
         ARRAY2(IRED)=RRED(IK)
200    CONTINUE
            nnn=nnn+1
C
C                        **Call to subroutine moment returns
C                          average,standard deviation and numave
C                          and locates the position of average ave
C                          for array1 ie reflectance in red channel.
C
        CALL MOMENT(ARRAY1,IRED,BARLRED,SDRED,NUMAVE,NUMAVE1)
C
C                        **Call to subroutine moment returns average
C                          and standard deviation  and numave
C                          are  dummy values for the surface
C                          reflectance
C
        CALL MOMENT(ARRAY2,IRED,BARRRED,DUMMY,IDUMMY,IDUMMY)
C
C                        **Surface reflectance values are averaged
C                          between 6 positions if
C                          standard deviation is greater than zero
C
C
       DO II = 1,2
          IF( II .EQ. 1) NN=NUMAVE
          IF( II .EQ. 2) NN=NUMAVE1
          NUMBER =0
         IF( SDRED .GT. 0.0 .AND. NN.GT.3 .AND.NN.LT.IRED-3)THEN
          DO 202 IK = NN-3 , NN+3
              NUMBER=NUMBER+1
              ARRAY1(NUMBER)=ARRAY2(IK)
202       CONTINUE
C
C
C                        **Call to subroutine moment returns
C                         average for surface reflectance
C
        CALL MOMENT(ARRAY1,NUMBER,BB,DUMMY,IDUMMY,IDUMMY)
             IF( II .EQ.1) BARRREDN=BB
             IF( II .EQ.2) BARRREDNN=BB
C
           ELSE
C
C                        **Surface reflectance values is set to
C                          first computed value if the standard
C                           is zero.
C
           BARRREDN  = BARRRED
           BARRREDNN = BARRRED
         ENDIF
        ENDDO
        RETURN
        END

C**********************************************************************
        SUBROUTINE OPTICAL(INT,PIT,FDOWN,FUP,OPTH,THET0,SBAR,THE,PHI,
     *     NUMWAV,LTHET0,LTHE,LPHI,LTAU,MTHET0,MTHET,MDPHI,BARLBLUE,
     *     BARRBLUE,BARRRED,BARLRED,TAUABLUE,TAUARED,NFDOWN,NFUP,FINT,NEW,
     *     NEWINT2,NEWN,NEWN2,NEWINT,IMES,WAVOPT,FT,BARLREDN,BARLBLUEN,
     *     BARRBLUEN,BARRREDN,TAUAREDN,TAUABLUEN,BARRBLUENN,BARRREDNN,
     *     TAUAREDNN,TAUABLUENN,SAVERATIO,WEIGHT,RHO,MINT,FLUXDN,FLUXUP,
     *     IERROR,JTABLE)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C
C          This subroutine derives the optical thicknesses for
C          both red (0.66 micron) and blue (0.47 micron) channels.
C
C!INPUT PARAMETERS:
C
C          NUMWAV   Number of wavelengths in lookup table.
C          LTHET0   Number of solarzenith angle in lookup table.
C          LTHE     Number of view angle in lookup table.
C          LPHI     Number of azimuths in lookup table.
C          LTAU     Number of optical depths in lookup table.
C          WAV      Wavelength
C          OPTH     Optical depth
C          THET0    Solar Angle
C          FDOWN    Flux down
C          FUP      Flux up
C          THE      Satellite View angle
C          PHI      Azimuth
C          SBAR     Sbar
C          INT      Reflectance
C          PIT      Transmission factor
C          NEW      Temporary arrays
C          NEWN     Temporary arrays
C          NEWINT   Temporary arrays
C          NFDOWN   Temporary arrays
C          NFUP     Temporary arrays
C          NEWINT2  Temporary arrays
C          MTHET0   MODIS input solar zenith angle
C          MTHET    MODIS input satellite view angle
C          MDPHI    MODIS input relative azimuth angle
C
C!OUTPUT PARAMETERS:
C
C          WAVOPT      wavelengths
C          IMES        reflectances interpolted.
C          BARLBLUE    Averaged Reflectance in blue channel.
C          BARRBLUE    Averaged surface Reflectance in blue channel.
C          BARRRED     Averaged Reflectance in red channel.
C          BARLRED     Averaged surface Reflectance in red channel.
C          SDBLUE      Standard deviation blue
C          SDRED       Standard deviation red
C          BARRBLUEN   Averaged value of blue Reflectance at6 loc.
C          BARRREDN    Averaged value of red Reflectance at6 loc.
C          BARRBLUENN  Averaged value of blue Reflectance + sd
C                      at6 loc.
C          BARRBLUENN  Averaged value of red Reflectance + sd
C                      at6 loc.
C          TAUABLUE    Optical thickness computed for BARLBLUE
C          TAUABLUEN   Optical thickness computed for BARRBLUEN
C          TAUABLUENN  Optical thickness computed for BARRBLUENN
C          TAUARED     Optical thickness computed for BARLRED
C          TAUAREDN    Optical thickness computed for BARLREDN
C          TAUAREDNN   Optical thickness computed for BARLREDNN
C          IERROR      Error index for no retrieval
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C 2/1/98 DAC change ordering dimensions of all variables
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
C GSFC, Greenbelt, MD
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
      IMPLICIT NONE
      REAL PI,DTR,RTD
      PARAMETER (PI = 3.1415927, DTR = PI/180.0,RTD=180./PI)
      INTEGER I,IBLUE,ICOM,IERROR,IJK,IPHI,IRED,ITAU,JTABLE,INDEX,ITABLE
      INTEGER ITHE,ITHET0,IWAV,LOPT,LPHI,LTAU,LTHE,LTHET0,NUMWAV
      INTEGER IND_THET0_1,IND_THET0_2,IND_THET_1,IND_THET_2,IND_PHI

      REAL WAVOPT(LTAU)
      REAL THET0(LTHET0),THE(LTHE),PHI(LPHI)
      REAL SBAR(LTAU,NUMWAV),OPTH(LTAU,NUMWAV)
C      REAL FDOWN(LTAU,LTHET0,NUMWAV)
      REAL FDOWN(LTHET0,LTAU,NUMWAV)
      REAL FUP(LTHET0,LTAU,NUMWAV)
C      REAL INT(LPHI,LTHE,LTAU,LTHET0,NUMWAV)
      REAL INT(LTHET0,LPHI,LTHE,LTAU,NUMWAV)
      REAL PIT(LTHE,LTAU,NUMWAV)

      REAL NFDOWN(LTAU),NFUP(LTAU),NEW(LPHI)
      REAL NEWINT(LPHI,LTHE,LTAU),NEWINT2(LTHE,LTAU),FT(LTAU)
      REAL NEWN(LPHI),NEWN2(LPHI),FINT(LTHE)
      REAL MTHET0,MTHET,MDPHI,IMES(LTAU)
      REAL ARG(1),YY(1),TAUA(3,2),RHO(3,2),MINT(3,2)

      REAL BARRBLUE,BARLBLUE,BARRRED,BARLRED
      REAL BARLBLUEN,BARLREDN,BARRBLUEN,BARRBLUENN
      REAL BARRREDN,BARRREDNN,DTAUBLUE,DTAURED,SAVERATIO
      REAL SDBLUE,SDRED,TAUABLUE,TAUABLUEN,TAUABLUENN
      REAL TAUARED,TAUAREDN,TAUAREDNN,WEIGHT
      REAL FLUXDN(NUMWAV),FLUXUP(NUMWAV)

      SAVE

      TAUARED=0.
      TAUABLUE=0.
      TAUAREDN=0.
      TAUABLUEN=0.0
      TAUAREDNN=0.0
      TAUABLUENN=0.0

C
C Interpolation of radiance and flux incident upon solar
C zenith angle
C

      DO 9995 IWAV=1,NUMWAV

      DO 240 IPHI=1,LPHI
        NEW(IPHI)=0.0
        NEWN(IPHI)=0.0
        NEWN2(IPHI)=0.0
 240  CONTINUE

      DO 104 ITAU=1,LTAU
        DO 207 ITHET0=1,LTHET0
C          NEWN(ITHET0)=FDOWN(ITAU,ITHET0,IWAV)
          NEWN(ITHET0)=FDOWN(ITHET0,ITAU,IWAV)
          NEWN2(ITHET0)=FUP(ITHET0,ITAU,IWAV)
 207    CONTINUE
        DO 105 ITHE=1,LTHE
          DO 106 IPHI=1,LPHI
            DO 107 ITHET0=1,LTHET0
C              NEW(ITHET0)=INT(IPHI,ITHE,ITAU,ITHET0,IWAV)
              NEW(ITHET0)=INT(ITHET0,IPHI,ITHE,ITAU,IWAV)
 107        CONTINUE

            IJK=1

C
C Call to subroutine INTERP_LAND to interpolate INT.
C If mthet0 is out of range of values of thet0 subroutine
C INTERP_LAND returns the value of LOPT=0
C

            ARG(1)=MTHET0
C            CALL INTERP_OPT(1,LTHET0,ARG,THET0,NEW,YY,
C     *                       LOPT,JTABLE,IND_THET0_1)
            CALL LOCATE_OPT(THET0,LTHET0,MTHET0,IND_THET0_1,JTABLE,LOPT)
            CALL LININT_OPT(MTHET0,YY(1),THET0,NEW,LTHET0,IND_THET0_1)
            NEWINT(IPHI,ITHE,ITAU)=YY(1)

C
C Call to subroutine INTERP_LAND to interpolate FDOWN.
C If mthet0 is out of range of values of thet0 subroutine
C INTERP_LAND returns the value of LOPT=0
C

            ARG(1)=MTHET0
C            CALL INTERP_OPT(1,LTHET0,ARG,THET0,NEWN,YY,
C     *                       LOPT,JTABLE,IND_THET0_2)
C            CALL LOCATE_OPT(THET0,LTHET0,MTHET0,IND_THET0_1,JTABLE,LOPT)
C            IF(LOPT.EQ.0) GO TO 999
            CALL LININT_OPT(MTHET0,YY(1),THET0,NEWN,LTHET0,IND_THET0_1)
            NFDOWN(ITAU)=YY(1)
            CALL LININT_OPT(MTHET0,YY(1),THET0,NEWN2,LTHET0,IND_THET0_1)
            NFUP(ITAU)=YY(1)

 106      CONTINUE
 105    CONTINUE
 104  CONTINUE

C
C Interpolation of radiance upon measured azimuth angle.
C NEWINT is interpolated for measured mphi to derive varaiable
C NEWINT2 (formly NEWNEW).
C
      DO 241 I=1,LPHI
        NEW(I)=0.0
        NEWN(I)=0.0
 241  CONTINUE

      DO 108 ITAU=1,LTAU
        DO 109 ITHE=1,LTHE

          DO 110 IPHI=1,LPHI
            NEW(IPHI)=NEWINT(IPHI,ITHE,ITAU)
 110      CONTINUE

          IJK=3

C
C Call to subroutine INTERP_LAND to interpolate on azimuth angle.
C If mphi is out of range subroutine INTERP_LAND returns
C the value of LOPT=0
C

          ARG(1)=MDPHI
C          CALL INTERP_OPT(1,LPHI,ARG,PHI,NEW,YY,
C     *                     LOPT,JTABLE,IND_PHI)
          CALL LOCATE_OPT(PHI,LPHI,MDPHI,IND_PHI,JTABLE,LOPT)
          CALL LININT_OPT(MDPHI,YY(1),PHI,NEW,LPHI,IND_PHI)
C          IF(LOPT.EQ.0) GO TO 999
          NEWINT2(ITHE,ITAU)=YY(1)

 109    CONTINUE
 108  CONTINUE

C
C Interpolation of radiance and transmission/pi at surface
C satellite zenith angle NEWINT2 is interpolated to get new
C varaiable FINT. Transmission factor PIT is interpolated to
C derive FT.
C

      DO 242 I=1,LPHI
        NEW(I)=0.0
        NEWN(I)=0.0
 242  CONTINUE
      DO 111 ITAU=1,LTAU
        DO 112 ITHE=1,LTHE
          NEW(ITHE)=NEWINT2(ITHE,ITAU)
          NEWN(ITHE)=PIT(ITHE,ITAU,IWAV)
 112    CONTINUE

C
C Call to subroutine INTERP_LAND to interpolate radiance on
C satellite zenith angle. If it is out of range, subroutine
C INTERP_LAND returns the value of lopt=0
C

        ARG(1)=MTHET
C        CALL INTERP_OPT(1,LTHE,ARG,THE,NEW,YY,
C     *                   LOPT,JTABLE,IND_THET_1)
         CALL LOCATE_OPT(THE,LTHE,MTHET,IND_THET_1,JTABLE,LOPT)
         CALL LININT_OPT(MTHET,YY(1),THE,NEW,LTHE,IND_THET_1)
C        IF(LOPT.EQ.0) GO TO 999
        FINT(ITAU)=YY(1)

C
C Call to subroutine INTERP_LAND to interpolate transmission on
C satellite zenith angle. If it is out of range, subroutine
C INTERP_LAND returns the value of lopt=0
C

        ARG(1)=MTHET
C        CALL INTERP_OPT(1,LTHE,ARG,THE,NEWN,YY,
C     *                   LOPT,JTABLE,IND_THET_2)
        CALL LININT_OPT(MTHET,YY(1),THE,NEWN,LTHE,IND_THET_1)
C        IF(LOPT.EQ.0) GO TO 999
        FT(ITAU)=YY(1)

 111  CONTINUE

C
C  Loop over 3 computations for
C    1. mean reflectance and mean surface reflectance
C    2. mean reflectance+sd and mean surface reflectance
C    3. mean reflectanc and mean surface reflectance+sd
C

       DO 9958 ICOM = 1,3

C
C Compute radiance as function of optical thickness given surface reflectance
C

         DO 9997 ITAU=1,LTAU

           IMES(ITAU)=FINT(ITAU)+(RHO(ICOM,IWAV))*NFDOWN(ITAU)*FT(ITAU)
     *    /(1.-(SBAR(ITAU,IWAV)*RHO(ICOM,IWAV)))
           WAVOPT(ITAU)=OPTH(ITAU,IWAV)

 9997    CONTINUE

         IJK = 2

C
C Call to subroutine INTERP_LAND to interpolate IMES on optical
C thickness. If it is out of range, subroutine INTERP_LAND returns
C LOPT=0
C

         ITABLE=1
         ARG(1)=MINT(ICOM,IWAV)
C         CALL INTERP_OPT(1,LTAU,ARG,IMES,WAVOPT,YY,
C     *                    LOPT,ITABLE,INDEX)
         CALL LOCATE_OPT(IMES,LTAU,ARG(1),INDEX,ITABLE,LOPT)
         IF(LOPT.EQ.0) GO TO 999
         CALL LININT_OPT(ARG(1),YY(1),IMES,WAVOPT,LTAU,INDEX)
         TAUA(ICOM,IWAV)=YY(1)
Change 8/16/99
         IF(ICOM.EQ.1) THEN
           CALL LININT_OPT(TAUA(1,IWAV),YY(1),WAVOPT,NFDOWN,LTAU,INDEX)
           FLUXDN(IWAV)=YY(1)
           CALL LININT_OPT(TAUA(1,IWAV),YY(1),WAVOPT,NFUP,LTAU,INDEX)
c             FLUXUP(IWAV)= YY(1)
c subtract rayleigh and * theta0 to be same units as ocean
              FLUXUP(IWAV)=(YY(1)-NFUP(1))* (COS(DTR*MTHET0))
c               FLUXUP(IWAV)= YY(1)* (COS(DTR*MTHET0))
         ENDIF
            
9958     CONTINUE
9995   CONTINUE 
C
C Save optical thickness in terms of the three conditions give above
C

         DO 9994 IWAV=1,2
           DO 9993 ICOM=1,3
             IF(IWAV.EQ.1) THEN
               IF(ICOM.EQ.1) THEN
                 TAUABLUE=TAUA(ICOM,IWAV)
               ELSE IF(ICOM.EQ.2) THEN
                 TAUABLUEN=TAUA(ICOM,IWAV)
               ELSE IF(ICOM.EQ.3) THEN
                 TAUABLUENN=TAUA(ICOM,IWAV)
               ENDIF
             ENDIF

             IF(IWAV.EQ.2) THEN
               IF(ICOM.EQ.1) THEN
                 TAUARED=TAUA(ICOM,IWAV)
               ELSE IF(ICOM.EQ.2) THEN
                 TAUAREDN=TAUA(ICOM,IWAV)
               ELSE IF(ICOM.EQ.3) THEN
                 TAUAREDNN=TAUA(ICOM,IWAV)
               ENDIF
             ENDIF
9993       CONTINUE
9994     CONTINUE

       GO TO 9996
999    CONTINUE
C
C  If the angles are out of boundsfrom look-up tables set the error code
C
       IF(IJK.EQ.1.AND.LOPT.EQ.0)THEN
         IERROR=1
         TAUARED=-3.0
         TAUABLUE=-3.0
         TAUABLUENN=-3.0
         TAUAREDNN=-3.0

C      CALL ERROR(BARLBLUE,BARRBLUE,TAUABLUE,SAVERATIO,WEIGHT,
C     *      BARLRED,BARRRED,TAUARED,SDBLUE,DTAUBLUE,BARRBLUEN,
C     *      TAUABLUEN,SDRED,DTAURED,BARRREDN,TAUAREDN,TAUABLUENN,
C     *      TAUAREDNN,BARRBLUENN,BARRREDNN,IBLUE,IRED)

       ENDIF
C
C If the reflectance is out of bounds from look-up tables set the error code
C

       IF(IJK.EQ.2.AND.LOPT.EQ.0)THEN
         IERROR=4
         TAUARED=-6.0
         TAUABLUE=-6.0
         TAUABLUENN=-6.0
         TAUAREDNN=-6.0

C        CALL ERROR(BARLBLUE,BARRBLUE,TAUABLUE,SAVERATIO,WEIGHT,
C     *      BARLRED,BARRRED,TAUARED,SDBLUE,DTAUBLUE,BARRBLUEN,
C     *      TAUABLUEN,SDRED,DTAURED,BARRREDN,TAUAREDN,TAUABLUENN,
C     *      TAUAREDNN,BARRBLUENN,BARRREDNN,IBLUE,IRED)
       ENDIF

9996   CONTINUE
       RETURN
       END

C*****************************************************************
      SUBROUTINE MOMENT(DATA,N,AVE,SDEV,NUM1,NUM2)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C               This subroutine is borrowed from "numerical Recipes"
C               page 458.  This subroutine returns the values of mean
C               average and standard deviation
C
C!INPUT PARAMETERS:
C
C     INTEGER   N           Number of elements in input array, DATA
C     REAL      DATA(N)     Array of N elements
C
C!OUTPUT PARAMETERS:
C
C     INTEGER   NUM1        Number of array elements (1 to N) with
C                           values less than the mean of array DATA
C     INTEGER   NUM2        Number of array elements (1 to N) with
C                           values less than the sum of the mean and
C                           std of the array DATA
C     REAL      AVE         Arithmetic average of the N values of the
C                           array DATA
C     REAL      SDEV        Standard Deviation of the N values of the
C                           array DATA.
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol Team at NASA GSFC, Greenbelt ,MD
C
C!REFERENCES AND CREDITS: Shana Mattoo
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
      IMPLICIT NONE
      INTEGER J,N,NUM1,NUM2
      REAL DATA(N),S,P,AVE,VAR,SDEV,AVVE
C
C This line was commented out by SDST vlin
C      IF(N.LE.1)PAUSE 'N must be at least 2'
C
      S=0.

      DO 11 J=1,N
        S=S+DATA(J)
11    CONTINUE

      AVE=S/N
      VAR=0.0

      DO 12 J=1,N
        S=DATA(J)-AVE
        P=S*S
        VAR=VAR+P
12    CONTINUE

      IF(VAR.GT.0)THEN
        VAR=VAR/(N-1)
        SDEV=SQRT(VAR)
C
C Returns index of location of average value in array
C
        CALL LOCATE(DATA,N,AVE,NUM1)
        AVVE=AVE+SDEV
        CALL LOCATE(DATA,N,AVVE,NUM2)
      ELSE
        SDEV=0.0
        NUM1=0
        NUM2=0
      ENDIF

      RETURN
      END

C**********************************************************************
      SUBROUTINE LOCATE_OPT(XX,N,X,J,JTABLE,LOPT)
C----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C               This subroutine is borrowed from numerical recepies on
C               page 90.  This subroutine locates position of x in
C               array xx , xx should be ordered in ascending order
C                      Numerical Recipes ver. 2.06
C
C!INPUT PARAMETERS:
C
C     INTEGER   N           Number of elements in input array, XX
C     REAL      XX(N)       Array of N elements
C     REAL      X           Scalar to be compared in value with the
C                           elements of array XX
C     INTEGER   JTABLE      Index of N look-up tables (N=4)
C
C!OUTPUT PARAMETERS:
C
C     INTEGER   J           Number of elements (1 to N) of array XX
C                           with values less than X.
C     INTEGER   LOPT        Index resulting from interpolation
C                           (LOPT=0, no interpolation;
C                            LOPT=1, interpolation is done)
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C 5/1/98 DAC
C modified to replace INTERP_LAND subroutine
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
C GSFC, Greenbelt, MD
C
C!REFERENCES AND CREDITS: Numerical Recepies
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C

      INTEGER   J, N ,JTABLE,LOPT
      INTEGER   JL, JM, JU
      REAL      X
      REAL      XX(N)
      LOPT=0

C      IF(JTABLE.EQ.1) THEN

      JL  = 0
      JU  = N + 1
   10 CONTINUE
      IF( JU - JL .GT. 1 ) THEN
         JM  = ( JU + JL ) / 2
         IF( ( XX(N).GE.XX(1) ) .EQV. ( X.GE.XX(JM) ) ) THEN
            JL  = JM
         ELSE
            JU  = JM
         END IF
         GO TO 10
      END IF
Change 2/20/2001
CTo deal with extrapolation
      IF(X.GT.XX(N)) JL=-N
      IF(X.LT.XX(1)) JL=-1
Change 2/20/2001
      J=JL
      IF(JL.NE.0) LOPT=1

C      ENDIF

      RETURN
      END

C**********************************************************************
       SUBROUTINE LININT_OPT(XI,YO,X,Y,M,INDEX)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C                This subroutine is a general purpose routine and
C                interpolates linearly.  Value of YO is interpolated
C                for XI.  No extrapolation is performed.
C
C!INPUT PARAMETERS:
C
C    REAL      XI       Array of input for parameter X
C    INTEGER   M        Number of values in LUT array for associated
C                       parameters Y and X
C    REAL      X(M)     LUT array of M values of parameter X
C    REAL      Y(M)     LUT array of M values of parameter Y
C    INTEGER   INDEX    INDEX for interpolation
C
C!OUTPUT PARAMETERS:
C
C    REAL      YO       Array of output for parameter Y
C
C!REVISION HISTORY:
c 01/28/98 fhliang
c fixed prolog.
c
c Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
c
c Developed by MODIS Aerosol Team at NASA GSFC, Greenbelt, MD
c
C!REFERENCES AND CREDITS: Allen Chu
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
       IMPLICIT NONE
       INTEGER IL,M,JTABLE,INDEX
       REAL  XI,YO,YN,X(M),Y(M),NUMER,DEMER
       REAL PINTEN,PPHI,SINTEN,SPHI

       YO=0.0

Change 2/20/2001
       IF(INDEX.GT.0) THEN
Change 2/20/2001
         IL=INDEX

C       PPHI=X(IL)
C       SPHI=X(IL+1)
C       PINTEN=Y(IL)
C       SINTEN=Y(IL+1)
C       NUMER=(SINTEN-PINTEN)*(XI-PPHI)
C       DEMER=SPHI-PPHI
C       YO=PINTEN+NUMER/DEMER

         IF(IL.NE.0) THEN
           IF(IL.LT.M) THEN
             NUMER=(Y(IL+1)-Y(IL))*(XI-X(IL))
             DEMER=(X(IL+1)-X(IL))
             YO=Y(IL)+NUMER/DEMER
           ELSE
             YO=Y(IL)
           ENDIF
         ENDIF
Change 2/20/2001
       ELSE IF(INDEX.LT.0) THEN
         YO=Y(ABS(INDEX))
Change 1/6/2002
         IF(ABS(INDEX).EQ.1) THEN
           DEMER=(X(2)-X(1))
           NUMER=(Y(2)-Y(1))*(XI-X(1))
           YN=Y(1)+NUMER/DEMER
           IF(ABS(YN).LT.0.05) YO=YN 
           IF(ABS(YN).GE.0.05) YO=-0.05 
         ENDIF
Change 1/6/2002
       ENDIF
Change 2/20/2001
       RETURN
       END

C**********************************************************************
      SUBROUTINE LOCATE(XX,N,X,J)
C----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C               This subroutine is borrowed from numerical recepies on
C               page 90.  This subroutine locates position of x in
C               array xx , xx should be ordered in ascending order
C               Numerical Recipes ver. 2.06
C
C!INPUT PARAMETERS:
C
C     INTEGER   N           Number of elements in input array, XX
C     REAL      XX(N)       Array of N elements
C     REAL      X           Scalar to be compared in value with the
C                           elements of array XX
C
C!OUTPUT PARAMETERS:
C
C     INTEGER   J           Number of elements (1 to N) of array XX
C                           with values less than X.
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol Team at NASA GSFC, Greenbelt, MD
C
C!REFERENCES AND CREDITS Numerical Recepies
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
      INTEGER   J, N
      INTEGER   JL, JM, JU
      REAL      X
      REAL      XX( N )

      JL  = 0
      JU  = N + 1
   10 CONTINUE
      IF( JU - JL .GT. 1 ) THEN
         JM  = ( JU + JL ) / 2
         IF( ( XX(N).GE.XX(1) ) .EQV. ( X.GE.XX(JM) ) ) THEN
            JL  = JM
         ELSE
            JU  = JM
         END IF
         GO TO 10
      END IF
      J=JL

      RETURN
      END

C*********************************************************************
      SUBROUTINE ERROR(BARLBLUE,BARRBLUE,TAUABLUE,SAVERATIO,WEIGHT,
     *      BARLRED,BARRRED,TAUARED,SDBLUE,DTAUBLUE,BARRBLUEN,
     *      TAUABLUEN,SDRED,DTAURED,BARRREDN,TAUAREDN,TAUABLUENN,
     *      TAUAREDNN,BARRBLUENN,BARRREDNN,IBLUE,IRED)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              This subroutine intializes the output arrays to zero
C              whenever a problem area is detected (not used).
C
C!INPUT PARAMETERS:
C
c     BARLBLUE         Reflectance at 0.47 micron channel
C     BARRBLUE         Surface Reflectance at 0.47 micron channel.
C     TAUABLUE         Aerosol optical thickness at 0.47 micron channel.
C     SAVERATIO        ration of aerosol path radiance red to blue.
C     WEIGHT           Relative contribution of smoke/sulphate to dust.
c     BARLRED          Reflectance for red channel.
C     BARRRED          Surface Reflectance for red channel.
C     TAUARED          Optical thickness for red channel.
C     SDBLUE           standard devation for blue.
c     SDRED            standard devation for   red.
C     IBLUE            Number of blue pixels in 10*10 that passed
C                      all criteria.
C     IRED             Number of blue pixels in 10*10 that passed
C                      all criteria.
C
C!OUTPUT PARAMETERS:
C
C     BARLBLUE         Reflectance for blue channel.
C     BARRBLUE         Surface Reflectance for blue channel.
C     TAUABLUE         Optical thickness for blue channel.
C     SAVERATIO        ration of aerosol path radiance red to blue.
C     WEIGHT           Relative contribution of smoke/sulphate to dust.
c     BARLRED          Reflectance for red channel.
C     BARRRED          Surface Reflectance for red channel.
C     TAUARED          Optical thickness for red channel.
C     SDBLUE           standard devation for blue.
c     SDRED            standard devation for   red.
C     IBLUE            Number of blue pixels in 10*10 that passed
C                      all criteria.
C     IRED             Number of blue pixels in 10*10 that passed
C                      all criteria.
C
C!REVISION HISTORY:
c 01/28/98 fhliang
c fixed prolog.
c
c Updated code to comply with most MODIS software standards.
C
c!TEAM-UNIQUE HEADER:
c
c Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
c GSFC, Greenbelt, MD
c
!REFERENCES AND CREDITS
C     WRITTEN BY:Shana Mattoo
C
!DESIGN NOTES:
C
c!END
C----------------------------------------------------------------------
C
       IMPLICIT NONE
      REAL BARLBLUE,BARRBLUE,BARRRED,BARLRED,TAUABLUE,TAUARED
      REAL  BARRBLUEN,BARRREDN,TAUAREDN,TAUABLUEN
      REAL BARRBLUENN,BARRREDNN,TAUAREDNN,TAUABLUENN
      INTEGER IBLUE,IERROR,IRED
      REAL DTAUBLUE,DTAURED,SAVERATIO,SDBLUE,SDRED,WEIGHT

      BARLBLUE=9.
      BARRBLUE=9.
c     TAUABLUE=0.0
      BARLRED=9.
      BARRRED=9.
c     TAUARED=0.0
      SDBLUE=9.0
      DTAUBLUE=0
      BARRBLUEN=9.0
      TAUABLUEN=0
      SDRED=9.0
      DTAURED=0
      BARRREDN=9.0
      TAUAREDN=0.0
C     TAUABLUENN=0.0
C     TAUAREDNN=0.0
      BARRBLUENN=9.0
      BARRREDNN=9.0
      IBLUE=-1
      IRED=-1
      SAVERATIO=9.0
      WEIGHT=9.0

      RETURN
      END

C**********************************************************************
       SUBROUTINE INTERP_OPT(I,M,X1,X,Y,Y1,LOPT,JTABLE,INDEX)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C                This subroutine is a general purpose routine and
C                interpolates linearly.  Value of y1 is interpolated
C                for x1.  No extrapolation is performed.
C
C!INPUT PARAMETERS:
C
C    INTEGER   I        Number of separate values of parameter Y to
C                       interpolate
C    INTEGER   M        Number of values in LUT array for associated
C                       parameters Y and X
C    REAL      X(M)     LUT array of M values of parameter X
C    REAL      Y(M)     LUT array of M values of parameter Y
C    REAL      X1(I)    Array of I "observations" of parameter X
C
C!OUTPUT PARAMETERS:
C
C    INTEGER   LOPT     Flag indicating whether interpolation was
C                       performed (1) or not (0) for all values of
C                       the observation vector X1(I).  Observations
C                       outside the range of coverage of the X LUT are
C                       not able to be interpolated.
C    REAL      Y1(I)    Array of I interpolated values of Y based
C                       on the values of I observations of X
C                       (i.e., the array X1)
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol Team at NASA GSFC, Greenbelt ,MD
C
C!REFERENCES AND CREDITS Shana Mattoo
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
       IMPLICIT NONE
       INTEGER I,IL,LK,LL,LOPT,M,JTABLE,INDEX
       REAL  X1(I),X(M),Y(M),Y1(I),NUMER,DEMER
       REAL PINTEN,PPHI,SINTEN,SPHI,XBAR

       DO 290 LK = 1,I
       Y1(LK)=0.0
       XBAR=X1(LK)
       LL=M-1

       IF(JTABLE.EQ.1) THEN

C
C No extrapolation allowed
C

C         DO 230 IL=1,LL

           IF(XBAR .LT. X(1) .OR. XBAR .GT. X(LL+1))THEN
             LOPT=0
             RETURN
           ENDIF

C230      CONTINUE

C
C Interpolation between two closest points
C

         DO 240 IL=1,LL

           IF (XBAR .GE. X(IL) .AND. XBAR .LE. X(IL+1)) THEN
             PPHI=X(IL)
             SPHI=X(IL+1)
             PINTEN=Y(IL)
             SINTEN=Y(IL+1)
             NUMER=(SINTEN-PINTEN)*(XBAR-PPHI)
             DEMER=SPHI-PPHI
             Y1(LK)=PINTEN+NUMER/DEMER
             LOPT=1
             INDEX=IL
             RETURN
           ENDIF
240      CONTINUE

       ELSE

C
C No extrapolation allowed
C

C         DO 250 IL=1,LL

           IF(XBAR .LT. X(1) .OR. XBAR .GT. X(LL+1))THEN
             LOPT=0
             RETURN
           ENDIF

C250      CONTINUE

C
C Interpolation between two closest points
C

C         DO 260 IL=1,LL

C           IF (XBAR .GE. X(IL) .AND. XBAR .LE. X(IL+1)) THEN
             IL=INDEX
             PPHI=X(IL)
             SPHI=X(IL+1)
             PINTEN=Y(IL)
             SINTEN=Y(IL+1)
             NUMER=(SINTEN-PINTEN)*(XBAR-PPHI)
             DEMER=SPHI-PPHI
             Y1(LK)=PINTEN+NUMER/DEMER
C             Y1(LK)=PINTEN+((SINTEN-PINTEN)*((XBAR-PPHI)/(SPHI-PPHI)))
             LOPT=1
C             INDEX=IL
             RETURN
C           ENDIF

C260      CONTINUE

       ENDIF

290    CONTINUE

       RETURN
       END
C**********************************************************************
       SUBROUTINE INTERP_LAND(I,M,X1,X,Y,Y1,LOPT)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C
C                This subroutine is a general purpose routine and
C                interpolates linearly.  Value of y1 is interpolated
C                for x1.  No extrapolation is performed.
C
C!INPUT PARAMETERS:
C
C    INTEGER   I        Number of separate values of parameter Y to
C                       interpolate
C    INTEGER   M        Number of values in LUT array for associated
C                       parameters Y and X
C       REAL   X(M)     LUT array of M values of parameter X
C       REAL   Y(M)     LUT array of M values of parameter Y
C       REAL   X1(I)    Array of I "observations" of parameter X
C
C!OUTPUT PARAMETERS:
C
C    INTEGER   LOPT     Flag indicating whether interpolation was
C                       performed (1) or not (0) for all values of
C                       the observation vector X1(I).  Observations
C                       outside the range of coverage of the X LUT are
C                       not able to be interpolated.
C    REAL      Y1(I)    Array of I interpolated values of Y based
C                       on the values of I observations of X
C                       (i.e., the array X1)
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol Team at NASA GSFC, Greenbelt, MD
C
C!REFERENCES AND CREDITS Shana Mattoo
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
       IMPLICIT NONE
       INTEGER I,IL,LK,LL,LOPT,M
       REAL  X1(I),X(M),Y(M),Y1(I)
       REAL PINTEN,PPHI,SINTEN,SPHI,XBAR

       DO 290 LK = 1,I
       Y1(LK)=0.0
       XBAR=X1(LK)
       LL=M-1

         DO 230 IL=1,LL
C
C No extrapolation allowed
C
           IF(XBAR .LT.X(1) .OR. XBAR .GT.X(LL+1))THEN
             LOPT=0
             RETURN
C
C Interpolation between two closest points
C
           ELSE IF (XBAR .GE.X(IL) .AND.XBAR.LE. X(IL+1)) THEN
             PPHI=X(IL)
             SPHI=X(IL+1)
             PINTEN=Y(IL)
             SINTEN=Y(IL+1)
             Y1(LK)=PINTEN+((SINTEN-PINTEN)*((XBAR-PPHI)/(SPHI-PPHI)))
             LOPT=1
             RETURN
           ENDIF

230      CONTINUE

290    CONTINUE

       RETURN
       END

C**********************************************************************
      SUBROUTINE RDPHSOM(OMRED,OMBLUE,PHSRED,PHSBLUE,RATIO,TAUMODEL,
     *      SCATT,NUMMODEL,NUMTAU,NUMSCATT,IRPHSOM)
C----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              Subroutine RDPHSOM reads phase functions and single
C              scattering albedos of 4 different aerosol models.
C
C
C!INPUT PARAMETERS:
C
C          IRPHSOM      Unit number of reading input file.
C
C!OUTPUT PARAMETERS:
C
C          OMRED        Single scattering albedo of 0.66 micron channel
C          OMBLUE       Single scattering albedo of 0.47 micron channel
C          PHSRED       Phase function for 0.66 micron channel.
C          PHSBLUE      Phase function for 0.47 micron channel.
C          RATIO        Ratio of red(omega*phase)/blue(omega*phase)
C          TAUMODEL     Optical thickness dependence of phase function
C                       and single scattering albedo of 4 aerosol models
C          SCATT        Scattering angles of 4 aerosol models in 10 degree
C                       increment
C          NUMMODEL     number of models.
C          NUMTAU       number of optical thicknesses.
C          NUMSCATT     number of scattering angles
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C  1/1/98  DAC change the ordering dimension of five 3-D arrays
C          (OMRED,OMBLUE,PHSRED,PHSBLUE,RATIO)
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol Team at NASA GSFC, Greenbelt, MD
C
C!REFERENCES AND CREDITS
C     WRITTEN BY:Shana Mattoo
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C 1 line added by vlin
C
      IMPLICIT NONE
      INTEGER IMODEL,IRPHSOM,ISCATT,ITAU,NNTAU,NUMMODEL,NUMSCATT,NUMTAU
      REAL OMRED(NUMSCATT,NUMTAU,NUMMODEL)
      REAL OMBLUE(NUMSCATT,NUMTAU,NUMMODEL)
      REAL PHSRED(NUMSCATT,NUMTAU,NUMMODEL)
      REAL PHSBLUE(NUMSCATT,NUMTAU,NUMMODEL)
      REAL RATIO(NUMSCATT,NUMTAU,NUMMODEL)
      REAL TAUMODEL(NUMTAU),SCATT(NUMSCATT)
      REAL TAUASUL(100),TAUASMK(100)
      SAVE

      READ(IRPHSOM,*) (TAUASUL(ITAU),ITAU=1,NUMTAU)
      READ(IRPHSOM,*) (TAUASMK(ITAU),ITAU=1,NUMTAU)
      READ(IRPHSOM,*) (TAUMODEL(ITAU),ITAU=1,NUMTAU)

      DO 100 IMODEL= 1,NUMMODEL

        IF(IMODEL.LT.3)NNTAU=1
        IF(IMODEL.GE.3)NNTAU=NUMTAU

        DO 101 ITAU=1,NNTAU

          DO 102 ISCATT= 1,NUMSCATT

            READ(IRPHSOM,*)SCATT(ISCATT),OMBLUE(ISCATT,ITAU,IMODEL),
     *        OMRED(ISCATT,ITAU,IMODEL),PHSBLUE(ISCATT,ITAU,IMODEL),
     *        PHSRED(ISCATT,ITAU,IMODEL)

            RATIO(ISCATT,ITAU,IMODEL)=
     *       (OMRED(ISCATT,ITAU,IMODEL) *PHSRED(ISCATT,ITAU,IMODEL))/
     *       (OMBLUE(ISCATT,ITAU,IMODEL)*PHSBLUE(ISCATT,ITAU,IMODEL))

102      CONTINUE

101     CONTINUE

100   CONTINUE

      RETURN
      END

C*********************************************************************
       SUBROUTINE WEIGHTING_METHOD1(TAUARED,TAUABLUE,RATIO,WEIGHT,SCATT,
     * OMRED,OMBLUE,PHSRED,PHSBLUE,TAUMODEL, PDUSTSULB,PDUSTSMKB,
     * PCONTINTALB,PDUSTSULR,PDUSTSMKR,PCONTINTALR,
     * NUMMODEL,NUMTAU,NUMSCATT,MTHET0,MTHET,MDPHI,SAVERATIO,
     * SCAT_ANGLE_LAND)
C----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION: subroutine WEIGHTING computes THE WHEIGHTING FACTOR for
C              the relative contribution of smoke/sulfate to dust.
C
C!INPUT PARAMETERS:
C
C          OMRED        single scattering albedo for red channel.
C          OMBLUE       single scattering albedo for blue channel.
C          PHSRED       phase function for red channel.
C          PHSBLUE      phase function for red channel.
C          RATIO        ratio  of red(omega*phase)/blue(omega*phase)
C          TAUMODEL     models for the phase and omega.
C          SCATT        scattering angle.
C          NUMMODEL     number of models.
C          NUMTAU       number of optical thicknesses.
C          NUMSCATT     number of scattering angles.
C          MTHET0       Measured Solar zenith angle.
C          MTHET        Measured view   angle.
C          MDPHI        Measured relative azimuth  angle.
C
C!OUTPUT PARAMETERS:
C
C        WEIGHT         Weighting function FOR smoke/sufate and dust
C        PDUSTSULB      phase function dust and sulfate blue channel
C        PDUSTSMKB      phase function dust and smoke blue channel
C        PCONTINTALB    phase function continental  blue channel
C        PDUSTSULR      phase function dust and sulfate red channel
C        PDUSTSMKR      phase function dust and smoke  red channel
C        PCONTINTALR    phase function continental  red channel
C        SAVERATIO    Ratio of aerosol path radiance(continental)
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol Team at NASA GSFC, Greenbelt, MD
C
C!REFERENCES AND CREDITS Shana Mattoo
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
       IMPLICIT NONE
       INTEGER IANG,ICH,IJ,ITAU,LOPT,NUMMODEL,NUMSCATT,NUMTAU
       REAL DEGRAD,RADDEG,SAVERATIO,TAUABLUE,TAUARED,THRSHOLD1
       REAL THRSHOLD2,WEIGHT
       REAL OMRED(NUMSCATT,NUMTAU,NUMMODEL)
       REAL OMBLUE(NUMSCATT,NUMTAU,NUMMODEL)
       REAL PHSRED(NUMSCATT,NUMTAU,NUMMODEL)
       REAL PHSBLUE(NUMSCATT,NUMTAU,NUMMODEL)
       REAL RATIO(NUMSCATT,NUMTAU,NUMMODEL)
       REAL SCATT(NUMSCATT),TAUMODEL(NUMTAU)
       REAL PDUSTSULB,PDUSTSMKB,PCONTINTALB
       REAL PDUSTSULR,PDUSTSMKR,PCONTINTALR
       REAL XX(100),YY(100),YY1(100),Y1(1),ARG(1),SCATMODIS
       REAL XXX(100,100),YYY(100,100),XX2(100),YY2(100),YY3(100)
       REAL ZZZ(100,100),ZZ1(100),ZZ3(100)
       REAL PP,QQ,RR,SS,FUNVALUE
       REAL MTHET0,MTHET,MDPHI,SCAT_ANGLE_LAND
       SAVE

C
C Initialization
C
C       DEGRAD=ACOS(-1.)/180.
C       RADDEG=180./ACOS(-1.)
C
       WEIGHT=0
       SCATMODIS=0.0
       FUNVALUE=0.0
       PDUSTSULB=0.0
       PDUSTSMKB=0.0
       PCONTINTALB =0.0
       PDUSTSULR=0.0
       PDUSTSMKR=0.0
       PCONTINTALR=0.0

       SCATMODIS=SCAT_ANGLE_LAND
C
C                 *** Change modis geometry to scattering angle.
C
C            SCATMODIS=-(COS(MTHET0*DEGRAD))*(COS(MTHET*DEGRAD))
C     *               +((SIN(MTHET0*DEGRAD))*(SIN(MTHET*DEGRAD))
C     *                   *( COS(MDPHI*DEGRAD)))
C             SCATMODIS= (ACOS(SCATMODIS))*RADDEG

C
C Check the valid boundaries of scattering angle
C

       ARG(1)=SCATMODIS

C
C Thresholds to separate dust and non-dust (smoke or sulfate aerosols)
C

      IF(SCATMODIS.LT.60.0)SCATMODIS=60.0
      IF(SCATMODIS.GT.170.0)SCATMODIS=170.0
      IF(SCATMODIS.GE.40.0.AND.SCATMODIS.LE.150.0) THEN
        THRSHOLD1=0.90
        THRSHOLD2=0.72
      ELSE IF(SCATMODIS.GT.150.0.AND.SCATMODIS.LE.168.0) THEN
        THRSHOLD1=0.90-0.01*(SCATMODIS-150.0)
        THRSHOLD2=0.72
      ELSE IF(SCATMODIS.GT.168.0.AND.SCATMODIS.LE.170.0) THEN
        THRSHOLD1=0.72
        THRSHOLD2=0.66
      ENDIF

C       IF(SCATMODIS.LT.60.0)SCATMODIS=60.0
C       IF(SCATMODIS.GT.170.0)SCATMODIS=170.0
C       IF( SCATMODIS.LE.150.0)THRSHOLD1=0.90
C       IF( SCATMODIS.GT.150.0 .AND.SCATMODIS.LE.160.0)THRSHOLD1=0.85
C       IF( SCATMODIS.GT.160.0 .AND.SCATMODIS.LE.170.0)THRSHOLD1=0.75
C       THRSHOLD2=0.75

C
C Ratio of aerosol path radiance of continental aerosol between red
C (0.66 micron) and blue (0.47 micron) channels
C

       DO 100 IJ = 1,NUMSCATT
         XX(IJ)=SCATT(IJ)
         YY(IJ)=(TAUARED/TAUABLUE)*RATIO(1,1,IJ)
100    CONTINUE

C
C Interpolation of MODIS scattering angle
C

       CALL INTERP_LAND(1,NUMSCATT,ARG,XX,YY,Y1,LOPT)
       IF( LOPT .EQ.0)RETURN
       FUNVALUE=Y1(1)
       SAVERATIO=FUNVALUE

C
C Compute the weighting between dust and non-dust aerosols (smoke/sulfate
C aerosols)
C

      IF(FUNVALUE .GT. THRSHOLD1)THEN
        WEIGHT=0.0
      ELSE
          IF(FUNVALUE .LT. THRSHOLD2)THEN
           WEIGHT=1.0
          ELSE
            WEIGHT=((FUNVALUE-THRSHOLD1)/(THRSHOLD2-THRSHOLD1))
            IF(WEIGHT .LT. 0)WEIGHT=0.0
            IF(WEIGHT .GT. 1)WEIGHT=1.0
          ENDIF
      ENDIF

C
C Compute the quantities of phase function x single scattering albedo for
C 4 different aerosol types
C

      DO 999 ICH = 1,2

        DO 101 ITAU = 1,NUMTAU

          DO 102 IANG= 1,NUMSCATT

            IF( ICH .EQ.1) THEN
              PP=PHSBLUE(IANG,1,2)*OMBLUE(IANG,1,2)
              QQ=PHSBLUE(IANG,ITAU,3)*OMBLUE(IANG,ITAU,3)
              RR=PHSBLUE(IANG,ITAU,4)*OMBLUE(IANG,ITAU,4)
              SS=PHSBLUE(IANG,1,1)*OMBLUE(IANG,1,1)
            ENDIF
            IF( ICH .EQ.2) THEN
              PP=(PHSRED(IANG,1,2)*OMRED(IANG,1,2))
              QQ=(PHSRED(IANG,ITAU,3)*OMRED(IANG,ITAU,3))
              RR=(PHSRED(IANG,ITAU,4)*OMRED(IANG,ITAU,4))
              SS=PHSRED(IANG,1,1)*OMRED(IANG,1,1)
            ENDIF
            XXX(ITAU,IANG)=(PP*(1.0-WEIGHT))+QQ*WEIGHT
            YYY(ITAU,IANG)=(PP*(1.0-WEIGHT))+RR*WEIGHT
            ZZZ(ITAU,IANG)=SS

102       CONTINUE

101     CONTINUE

        DO 103 ITAU=1,NUMTAU

          DO 104 IANG = 1,NUMSCATT

            XX(IANG)=SCATT(IANG)
            YY(IANG)=XXX(ITAU,IANG)
            YY1(IANG)=YYY(ITAU,IANG)
            ZZ1(IANG)=ZZZ(ITAU,IANG)

104       CONTINUE

          ARG(1)=SCATMODIS

C
C Interpolation on MODIS scattering angle
C

          CALL interp_land(1,NUMSCATT,ARG,XX,YY,Y1,LOPT)
          IF( LOPT .EQ.0)RETURN
          XX2(ITAU)=TAUMODEL(ITAU)
          YY2(ITAU)=Y1(1)
          CALL interp_land(1,NUMSCATT,ARG,XX,YY1,Y1,LOPT)
          YY3(ITAU)=Y1(1)
          CALL interp_land(1,NUMSCATT,ARG,XX,ZZ1,Y1,LOPT)
          ZZ3(ITAU)=Y1(1)

103     CONTINUE

        ARG(1)=TAUARED
        IF(ARG(1) .GT. TAUMODEL(NUMTAU))ARG(1)=TAUMODEL(NUMTAU)
        CALL INTERP_LAND(1,NUMTAU,ARG,XX2,YY2,Y1,LOPT)
        IF(LOPT.EQ.0)RETURN
        IF(ICH.EQ.1)PDUSTSULB=Y1(1)
        IF(ICH.EQ.2)PDUSTSULR=Y1(1)
        CALL INTERP_LAND(1,NUMTAU,ARG,XX2,YY3,Y1,LOPT)
        IF(LOPT.EQ.0)RETURN
        IF(ICH.EQ.1)PDUSTSMKB=Y1(1)
        IF(ICH.EQ.2)PDUSTSMKR=Y1(1)
        CALL INTERP_LAND(1,NUMTAU,ARG,XX2,ZZ3,Y1,LOPT)
        IF(LOPT.EQ.0)RETURN
        IF(ICH.EQ.1)PCONTINTALB=Y1(1)
        IF(ICH.EQ.2)PCONTINTALR=Y1(1)

999   CONTINUE

      RETURN
      END

C***********************************************************************
      SUBROUTINE GEOSULSMK(IMONTH,LATM,LONM,ISULP,ISMK,ISMK_H,ISMK_M)
C----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              Subroutine GEOSULSMK determines aerosol type between
C              sulfate and smoke based on geolaocation and season
C
C!INPUT PARAMETERS:
C
C             IMONTH     Month of the data collected.
C             LATM       Latitude of the pixel.
C             LONM       Londitude of the pixel.
C
C!OUTPUT PARAMETERS:
C
C             ISULP      sulfate
C             ISMK       smoke
C
c!REVISION HISTORY:
c 01/28/98 fhliang
c fixed prolog.
c
c Updated code to comply with most MODIS software standards.
C
c!TEAM-UNIQUE HEADER:
c
c Developed by MODIS Aerosol Team at NASA GSFC, Greenbelt, MD
c
C!REFERENCES AND CREDITS Shana Mattoo
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
      IMPLICIT NONE
      INTEGER IMONTH,ISULP,ISMK,ISMK_H,ISMK_M
       REALLATM,LONM,
     *LATMIN1,LATMAX1,LONMIN1,LONMAX1, LATMIN2,LATMAX2, LONMIN2,LONMAX2,
     *LATMIN3,LATMAX3,LONMIN3,LONMAX3, LATMIN4,LATMAX4, LONMIN4,LONMAX4,
     *LATMIN5,LATMAX5,LONMIN5,LONMAX5, LATMIN6,LATMAX6, LONMIN6,LONMAX6,
     *LATMIN7,LATMAX7,LONMIN7,LONMAX7, LATMIN8,LATMAX8, LONMIN8,LONMAX8,
     *LATMIN9,LATMAX9,LONMIN9,LONMAX9, 
     *LATMIN10,LATMAX10, LONMIN10,LONMAX10,
     *LATMIN11,LATMAX11, LONMIN11,LONMAX11,
     *LATMIN12,LATMAX12, LONMIN12,LONMAX12 
C
C urban/industrial pollution in E. US/Canada & W. Europe ( SULPHATE ALL Year)
C
      PARAMETER(LATMIN1=30.0,LATMAX1=70.0,LONMIN1=-100.0,LONMAX1=25.0)
C
C moderate absorption in W. US/Canada including Alaska  and all Pacific islands(  MODERATE SMOKE ALL Year)
C
      PARAMETER(LATMIN2=-90.0,LATMAX2=90.0, LONMIN2=-180.0,LONMAX2=-100.0)
C
C moderate absorption in South America (MODERATE SMOKE ALL Year)
C
      PARAMETER(LATMIN3=-90.0,LATMAX3=30.0,LONMIN3=-100.0,LONMAX3=-25.0)
C
C MOderate Smokeand Heavy smoke North  Africa 
C                                       ( months 1,2 ,3 ,4,12      =      MOderate Smoke)
C                                       ( months 5,6,7,8,9,10,11 =      HEAVY       Smoke)

      PARAMETER(LATMIN4=0.0,LATMAX4=30.0,LONMIN4=-25.0,LONMAX4=50.0)
C
C MOderate Smokeand Heavy smoke Sourth  Africa 
C                                       ( months 1,2 ,3 ,4,11,12 =     Pollution)
C                                        ( months 6,7,8,9,10      =    HEAVY Smoke)

      PARAMETER(LATMIN5=-90.0,LATMAX5=0.0,LONMIN5=-25.0,LONMAX5=50.0)
C
C MOderate Smoke Northern polar (  MODERATE SMOKE ALL Year) 

      PARAMETER(LATMIN6=70.0,LATMAX6=90.0,LONMIN6=-100.0,LONMAX6=180.0)
 
C
C Heavy Smoke China(   Heavy Smoke ALL Year) 

      PARAMETER(LATMIN7=20.0,LATMAX7=39.0,LONMIN7=95.0,LONMAX7=125.0)
 
C
C urban/industrial pollution in S. Korea and Japan (ALL Year)
C
      PARAMETER(LATMIN8=20.0,LATMAX8=39.0,LONMIN8=125.0,LONMAX8=180.0)
c
C moderate absorption in Central  Asia 
C
      PARAMETER(LATMIN9=30.0,LATMAX9=70.0,LONMIN9=25.0,LONMAX9=95.0)
c
C moderate absorption in North China 
C
      PARAMETER(LATMIN10=39.0,LATMAX10=70.0,LONMIN10=95.0,LONMAX10=180.0)
c
C moderate absorption in India
C
      PARAMETER(LATMIN11=-90.0,LATMAX11=30.0,LONMIN11=50.0,LONMAX11=95.0)
c
C moderate absorption in  SE Asia & Australia
C
      PARAMETER(LATMIN12=-90.0,LATMAX12=20.0,LONMIN12=95.0,LONMAX12=180.0)
C
      SAVE
      ISULP=0
      ISMK=0
      ISMK_M=0
      ISMK_H=0
C
C urban/industrial pollution in E. US/Canada & W. Europe (ALL Year)
C

      IF ((LATM .GE. LATMIN1 .AND. LATM .LE. LATMAX1) .AND.
     *    (LONM .GE. LONMIN1 .AND. LONM .LE. LONMAX1)) ISULP=1

C
C moderate absorption in W. US/Canada including Alaska  and all Pacific islands(  MODERATE SMOKE ALL Year)
C

      IF ((LATM .GE. LATMIN2 .AND. LATM .LE. LATMAX2) .AND. 
     *    (LONM .GE. LONMIN2 .AND. LONM .LT. LONMAX2)) ISMK_M=1

C
C moderate absorption in South America (MODERATE SMOKE ALL Year)
C
 

      IF ((LATM .GE. LATMIN3 .AND. LATM .LE. LATMAX3) .AND.
     *    (LONM .GT. LONMIN3 .AND. LONM .LE. LONMAX3)) ISMK_M=1

 

C
C moderate absorption in Northern/Central Afria (ALL Year)
C

       IF ((LATM .GE. LATMIN4 .AND. LATM .LT. LATMAX4) .AND. 
     *    (LONM .GT. LONMIN4 .AND. LONM .LT. LONMAX4)) THEN

         IF (IMONTH .GE.5 .AND. IMONTH .LE.11) ISMK_H=1 
         IF (IMONTH .LE.5 .OR. IMONTH .GT.11) ISMK_M=1
      Endif

C
C heavy absorption in Southern Africa for biomass burning season (5-10) & 
C urban/industrial pollution for non-biomass burning season (<4, >10) 
C

      IF ((LATM .GE. LATMIN5 .AND. LATM .LT. LATMAX5) .AND.
     *    (LONM .GT. LONMIN5 .AND. LONM .LT. LONMAX5)) THEN

      IF (IMONTH .GT.5 .AND. IMONTH .LE.10) ISMK_H=1 
      IF (IMONTH .LE.5 .OR. IMONTH .GE.11) ISULP=1

      ENDIF
C
C MOderate Smoke Northern polar (  MODERATE SMOKE ALL Year) 
C
 
      IF ((LATM .GT. LATMIN6 .AND. LATM .LE. LATMAX6) .AND.
     *    (LONM .GE. LONMIN6 .AND. LONM .LT. LONMAX6)) ISMK_M=1
C
C Heavy Smoke China(   Heavy Smoke ALL Year) 
C
        IF ((LATM .GE. LATMIN7 .AND. LATM .LE. LATMAX7) .AND.
     *     (LONM .GE. LONMIN7 .AND. LONM .LE. LONMAX7)) ISMK_H=1
C
C urban/industrial pollution in S. Korea and Japan (ALL Year)
C
        IF ((LATM .GE. LATMIN8 .AND. LATM .LE. LATMAX8) .AND.
     *     (LONM .GT. LONMIN8 .AND. LONM .LE. LONMAX8)) ISULP=1
c
C moderate absorption in Central  Asia 
C
c
       IF ((LATM .GE. LATMIN9 .AND. LATM .LE. LATMAX9) .AND.
     *     (LONM .GT. LONMIN9 .AND. LONM .LT. LONMAX9)) ISMK_M=1
c
C moderate absorption in North China 
c
        IF ((LATM .GT. LATMIN10 .AND. LATM .LE. LATMAX10) .AND.
     *     (LONM .GE. LONMIN10 .AND. LONM .LT. LONMAX10)) ISMK_M=1
c
C moderate absorption in India
C
C
         IF ((LATM .GE. LATMIN11 .AND. LATM .LT. LATMAX11) .AND.
     *     (LONM .GE. LONMIN11 .AND. LONM .LT. LONMAX11)) ISMK_M=1
c
C moderate absorption in  SE Asia & Australia
C
        IF ((LATM .GE. LATMIN12 .AND. LATM .LT. LATMAX12) .AND.
     *     (LONM .GE. LONMIN12 .AND. LONM .LT. LONMAX12)) ISMK_M=1
c
C
C IF none of regions/months satisfied, use moderate absorption
C

      IF(ISULP.EQ.0.AND.ISMK_H.EQ.0.AND.ISMK_M.EQ.0) ISMK_M=1
      IF(ISMK_H.EQ.1.OR.ISMK_M.EQ.1) ISMK=1
 

      RETURN
      END
C**********************************************************************
      SUBROUTINE CORRECTED(ISULP,ISMK,TAUARED,TAUABLUE,PDUSTSULB,
     * PDUSTSMKB,PCONTINTALB,PDUSTSULR,PDUSTSMKR,PCONTINTALR,
     * TAUABLUEC,TAUAREDC,WEIGHT)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c !F77
C
c!DESCRIPTION:
C              To perform correction of aerosol optical thickness
C              derived from continental model based on the scattering
C              function and single scattering albedo
C
c!INPUT PARAMETERS:
C
C  ISULP           Flag of sulfate aerosol
C  ISMK            Flag of smoke aerosol
C  TAUARED         Continental aerosol optical depth at 0.67 micron
C  TAUABLUE        COntinental aerosol optical depth at 0.47 micron
C  PDUSTSULB       Path radiance for sulfate aerosol at 0.47 micron
C  PDUSTSMKB       Path radiance for smoke aerosol   at 0.47 micron
C  PCONTINTALB     Path radiance for continental aerosol at 0.47 micron
C  PDUSTDULR       Path radiance for sulfate aerosol at 0.66 micron
C  PDUSTSMKR       Path radiance for smoke aerosol at 0.66 micron
C  PCONTINTALR     Path radiance for continental aerosol at 0.66 micron
C  WEIGHT          Dust weighting factor
C
C!OUTPUT PARAMETERS:
C
C  TAUABLUEC       Corrected aerosol optical depth at 0.47 micron
C  TAUAREDC        Corrected aerosol optical depth at 0.66 micron
C
C!REVISION HISTORY:
C 02/01/98 achu
C added prolog.
C
C!REFERENCES AND CREDITS
C
C MODIS atmospehere aerosol team
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol Team at GSFC, Greenbelt, MD
C
C!DESIGN NOTES:
C
C!END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      IMPLICIT NONE
      INTEGER ISULP,ISMK
      REAL TAUARED,TAUABLUE,PDUSTSULB,PDUSTSMKB,PCONTINTALB,PDUSTSULR
      REAL PDUSTSMKR,PCONTINTALR,TAUAREDC,TAUABLUEC,WEIGHT
C
C If dust then Isul/smk=0
C
      IF (WEIGHT.LE.0) THEN
        ISULP=0
        ISMK=0
      ENDIF
C
      IF (ISULP.EQ.1.AND.ISMK.EQ.0.AND.WEIGHT.GT.0.0) THEN
         IF(PDUSTSULR.LE.0)PDUSTSULR=0.0000001
         IF(PDUSTSULB.LE.0)PDUSTSULB=0.0000001
         TAUAREDC=(TAUARED*PCONTINTALR)/PDUSTSULR
         TAUABLUEC=(TAUABLUE*PCONTINTALB)/PDUSTSULB
      ELSE
         IF(ISMK .EQ. 1 .AND.ISULP .EQ.0 .AND. WEIGHT.GT.0.0) THEN
           IF(PDUSTSMKR.LE.0)PDUSTSMKR=0.0000001
           IF(PDUSTSMKB.LE.0)PDUSTSMKB=0.0000001
           TAUAREDC=(TAUARED*PCONTINTALR)/PDUSTSMKR
           TAUABLUEC=(TAUABLUE*PCONTINTALB)/PDUSTSMKB
         ELSE
           IF(ISMK.EQ.0 .AND.ISULP.EQ.0 .AND. WEIGHT.LE.0.0) THEN
             IF(PDUSTSMKR .LE.0)PDUSTSMKR=0.0000001
             IF(PDUSTSMKB .LE.0)PDUSTSMKB=0.0000001
             TAUAREDC=(TAUARED*PCONTINTALR)/PDUSTSMKR
             TAUABLUEC=(TAUABLUE*PCONTINTALB)/PDUSTSMKB
           ELSE
             TAUAREDC=TAUARED
             TAUABLUEC=TAUABLUE
           ENDIF
         ENDIF
      ENDIF
C
      RETURN
      END


C***********************************************************************
      SUBROUTINE OUTPUT(LATM,LONM,IAER,BARLBLUE,IDATA,
     *TAUABLUE,BARLRED,BARL0P86,TAUARED,SDBLUE,SDRED,SD0P86,TAUABLUEC,
     *TAUAREDC,IBLUE,IRED,IERROR,weight,ISULP,ISMK,IPROCE,SAVERATIO,
     *BARRBLUE,BARRRED,TAUABLUENN,TAUAREDNN,SCAT_ANGLE_LAND,ANGSTROM,
     *MTHET0,MTHET,MDPHI,QA_Flag_Land,Success_Ret_Land,Fail_Ret_Land,
     *FLUXDNBLUE,FLUXDNRED,FLUXUPBLUE,FLUXUPRED,MASSCON,
     *SDSLAT,SDSLON,SDS_MTHET0,SDS_MTHET,SDS_MPHI,SDS_Tau_Land_Ocean,
     *SDS_Aerosol_Type,
     *SDS_SCAT_ANGLE_land,SDS_mass_conc_land,SDS_angs_coeff_land,
     *SDS_CLDFRC_land,SDS_dust_weighting,SDSTAU_continantal,
     *SDS_est_uncer,SDS_RefF_land,SDS_TranF_land,SDS_NUMPIXELS_land,
     *SDSTAU_corrected,
     *SDS_ref_land,SDS_ref_STD_land,SDS_QCONTROL_land,quality_land,
     * Ret_Quality_cirrus,cloud_num_land)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C                 This subroutine stores all SDSe arrays to HDF file
C
C!INPUT PARAMETERS: all varaiables to be wrriten as output
C
C   LATM          Latitude of data cell
C   LONM          Longitude of data cell
C   IAER          Aerosol type
C   BARLBLUE      Average reflectance for blue channel
C   IDATA         Data cell number from 1 to NUMDATA
C   NUMDATA       Number of retrieval cells across orbit swath
C   TAUABLUE      Blue channel aerosol optical thickness (Continental)
C   BARLRED       Average reflectance for red channel
C   TAUARED       Red channel aerosol optical thickness (Continental)
C   SDBLUE        STD of blue channel reflectances
C   SDRED         STD of red channel reflectances
C   TAUABLUEC     Blue channel aerosol optical thickness (Corrected)
C   TAUAREDC      Red channel aerosol optical thickness (Corrected)
C   IBLUE         Number of blue channel observations
C   IRED          Number of red channel observations
C   IERROR        Error flag (0-4)
C   WEIGHT        Relative contribution of smoke/sulfate particles to
C                 dust in the computation of the aerosol optical depth
C   ISULP         Sulfate (1)/no sulfate flag (0)
C   ISMK          Smoke (1)/no smoke flag (0)
C   IPROCE        Aerosol retrieval procedure ID (0-4)
C   SAVERATIO     Aerosol path radiance ration (red to blue channel
C                 for continental model)
C   PIXELS        Number of cells across swath (same as NUMDATA)
C   LINES         Number of cells along swath (same as NUMSCAN)
C   NUMXBOX       Not used
C   NUMYBOX       Not used
C
C!OUTPUT PARAMETERS: 13 variables for output to HDF FILE
C
C  SDS1           HDF array of cell latitudes
C  SDS2           HDF array of cell longitudes
C  SDS3           HDF array of spectral reflectances
C  SDS4           HDF array of aerosol optical thicknesses for
C                     continental model
C  SDS5           HDF array of the STD of spectral reflectances
C  SDS6           HDF array of aerosol optical thicknesses for
C                     corrected model
C  SDS7           HDF array of STD for corrected optical thickness
C                     blue channel for continental model)
C  SDS8           HDF array of aerosol path radiance ratio (red to
C                     blue channel for continental model)
C  SDS9           HDF array of relative aerosol optical depth
C                     (smoke/sulfate particles to dust)
C  SDS10          HDF array of the number of blue channel clear pixels
C  SDS11          HDF array of the number of red channel clear pixels
C  SDS12          HDF array of retrieval procedure ID (0-4)
C  SDS13          HDF array of aerosol type (0-3)
C  SDS14          HDF array of Error flag (0-4)
C
C!REVISION HISTORY:
C 01/28/98 fhliang
C fixed prolog.
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol Team at NASA GSFC, Greenbelt, MD
C
C!REFERENCES AND CREDITS
C     WRITTEN BY: Shana Mattoo
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
      IMPLICIT NONE
      INCLUDE 'mod04.inc'
C

c rhucek 01/15/98:  Changed type declaration of QA_Temp from INTEGER
c                   to BYTE.
      BYTE QA_Temp
      INTEGER I,QA_Flag_Land(19),Success_Ret_Land,Fail_Ret_Land,cloud_num_land
      integer quality_land, Ret_Quality_cirrus
      INTEGER IERROR,IBLUE,IRED,IAER,IPROCE,IPR,ISULP,ISMK,IDATA
      REAL  TAUABLUE,TAUARED,TAUAREDC,TAUABLUEC,TAUAREDC2,TAUABLUEC2,SDBLUE,SDRED,SD0P86
      REAL  LATM,LONM,SAVERATIO,BARLBLUE,BARLRED,BARL0P86,SDS_SCAT_ANGLE
      REAL  BARRBLUE,BARRRED,TAUABLUENN,TAUAREDNN,SDTAUCBLUE,SDTAUCRED
      REAL  MTHET0,MTHET,MDPHI,WEIGHT,SCAT_ANGLE_LAND,ANGSTROM
      REAL  TAUARAT,TAUAGREENC,TAUAGREENC2
      REAL  FLUXDNBLUE,FLUXDNRED,FLUXUPBLUE,FLUXUPRED,MASSCON
      Real New_cloud_num
C
      INTEGER*2   SDS_MTHET0(NUMCELLS),
     &            SDS_MTHET(NUMCELLS),
     &            SDS_MPHI(NUMCELLS),
     &            SDS_Tau_Land_Ocean(NUMCELLS),
     &            SDS_Aerosol_Type(NUMCELLS),
     &            SDS_SCAT_ANGLE_land(NUMCELLS),
     &            SDS_angs_coeff_land(NUMCELLS),
     &            SDS_CLDFRC_land(NUMCELLS),
     &            SDS_dust_weighting(NUMCELLS),
     &            SDSTAU_continantal(NUMCELLS,Land_Sol1),
     &            SDS_est_uncer(NUMCELLS,Land_Sol1),
     &            SDS_RefF_land(NUMCELLS,Land_Sol2),
     &            SDS_TranF_land(NUMCELLS,Land_Sol1),
     &            SDS_NUMPIXELS_land(NUMCELLS,Land_Sol1),
     &            SDSTAU_corrected(NUMCELLS,Land_Sol2),
     &            SDS_ref_land(NUMCELLS,Band_land),
     &            SDS_ref_STD_land(NUMCELLS,Band_land)
      REAL  SDSLAT(NUMCELLS),SDSLON(NUMCELLS),
     *      SDS_mass_conc_land(NUMCELLS)
      BYTE  SDS_QCONTROL_land(QA_LAND,NUMCELLS)
      SAVE
      SDS_CLDFRC_land(IDATA)=-9999
c compute fraction instead of % for outpt

       if( cloud_num_land .ge.0) New_cloud_num=real(cloud_num_land/100.)
 
C
C Change the following may affect level 3 result
C

C      IF(SAVERATIO .LE.0 .AND. WEIGHT.LE.0)IAER=0
C      IF(SAVERATIO .GT.0 .AND. WEIGHT.LE.0)IAER=1
      IF(WEIGHT.EQ.1.0) THEN
        IF(ISULP.EQ.1.AND.ISMK.EQ.0)IAER=2
        IF(ISULP.EQ.0.AND.ISMK.EQ.1)IAER=3
      ENDIF
      IF(WEIGHT.EQ.0.0)IAER=1
C mixed aerosol types
      IF(WEIGHT.GT.0.0.AND.WEIGHT.LT.1.0)IAER=0
C continental model
       IF(IPROCE.GT.4) IAER=4

C If error flag value other than zero fill with fill_values
C (See file spces of MOD04 for details)

      IF(TAUABLUEC.LT.0.0.OR.TAUAREDC.LT.0.0) IERROR=7
       if(IPROCE.eq.0)IERROR=8
      IF(IERROR.GT.0) THEN
    
      
       Fail_Ret_Land=Fail_Ret_Land+1
       QA_Flag_Land(7)=0
       QA_Flag_Land(8)=0
       QA_Flag_Land(9)=0
       QA_Flag_Land(10)=0
       QA_Flag_Land(11)=IPROCE
       QA_Flag_Land(12)=IERROR
       QA_Flag_Land(15)=0 

c 
c        if(New_cloud_num .le. 0.9 )New_cloud_num=-99
         if( New_cloud_num .ge.0)
     *  SDS_CLDFRC_land(IDATA)=(New_cloud_num*SCALE3+OFFSET3)
      CALL FILLVALUE_LAND(IDATA,SDS_Tau_Land_Ocean,SDS_Aerosol_Type,
     *SDS_SCAT_ANGLE_land,SDS_mass_conc_land,SDS_angs_coeff_land,
     *SDS_CLDFRC_land,SDS_dust_weighting,SDSTAU_continantal,
     *SDS_est_uncer,SDS_RefF_land,SDS_TranF_land,SDS_NUMPIXELS_land,
     *SDSTAU_corrected,SDS_ref_land,SDS_ref_STD_land,
     *SDS_QCONTROL_land)

      ELSE IF(IERROR.EQ.0) THEN
  
C Store SDS arrays for population to HDF file

     
       Success_Ret_Land=Success_Ret_Land+1
       QA_Flag_Land(7)=1
       QA_Flag_Land(8)=3
       IF(IPROCE.GT.4) QA_Flag_Land(8)=0
       QA_Flag_Land(9)=1
       QA_Flag_Land(10)=3
       IF(IPROCE.GT.4) QA_Flag_Land(10)=0
Cquality_land .eq.0 is if there is one single pixel of water
      if( quality_land .eq.0)then
              QA_Flag_Land(8)=0
               QA_Flag_Land(10)=0
        endif
      if(Ret_Quality_cirrus .eq.0)then
               QA_Flag_Land(8)=0
               QA_Flag_Land(10)=0
        endif
       QA_Flag_Land(11)=IPROCE
       QA_Flag_Land(12)=IERROR
       IF(IAER.LE.3) QA_Flag_Land(15)=IAER

      SDS_Aerosol_Type(IDATA)=IAER

      SDS_SCAT_ANGLE_land(IDATA)=SCAT_ANGLE_LAND*SCALE2+OFFSET2

     

      IF(IAER.EQ.2.OR.IAER.EQ.3) THEN
        TAUABLUEC2=TAUABLUEC*weight
        TAUAREDC2=TAUAREDC*weight
      ENDIF
      IF(IAER.EQ.1)THEN
        TAUABLUEC2=TAUABLUEC*(1.0-weight)
        TAUAREDC2=TAUAREDC*(1.0-weight)
      ENDIF

      IF(TAUABLUEC.GT.0.0.AND.TAUAREDC.GT.0.0) THEN
       TAUARAT=TAUABLUEC/TAUAREDC
       TAUARAT=ALOG(TAUARAT)
       TAUAGREENC=EXP(0.5370*TAUARAT)*TAUAREDC
       TAUAGREENC2=EXP(0.5370*TAUARAT)*TAUAREDC2
      ENDIF
      IF(TAUABLUEC.EQ.0.0.AND.TAUAREDC.EQ.0.0) THEN
      TAUAGREENC=0.0
      ENDIF

     
       SDSTAU_corrected(IDATA,1)=NINT(TAUABLUEC * SCALE3 + OFFSET3)
       SDSTAU_corrected(IDATA,3)=NINT(TAUAREDC * SCALE3 + OFFSET3)
        IF(TAUAGREENC.LT.0.0) THEN
                  SDSTAU_corrected(IDATA,2)=-9999
        ELSE IF(TAUAGREENC.GE.0.0) THEN 
            SDSTAU_corrected(IDATA,2)=NINT(TAUAGREENC * SCALE3 + OFFSET3)
        ENDIF

         SDS_CLDFRC_land(IDATA)= (New_cloud_num*SCALE3+OFFSET3)
c        SDS_CLDFRC_land(IDATA)=QA_Flag_Land(8)
c Shana made the change after meeting with Kaufman
         SDS_dust_weighting(IDATA)=NINT((weight) * SCALE3 + OFFSET3)
      IF(WEIGHT.LT.0) THEN
         SDS_dust_weighting(IDATA)=-9999
       ENDIF
C
      SDSTAU_continantal(IDATA,1)=NINT(TAUABLUE * SCALE3 + OFFSET3)
      SDSTAU_continantal(IDATA,2)=NINT(TAUARED * SCALE3 + OFFSET3)
      
           IF(TAUABLUE.LT.-0.05.OR.TAUARED.LT.-0.05) THEN 
        SDSTAU_continantal(IDATA,1)=-9999
        SDSTAU_continantal(IDATA,2)=-9999
           ENDIF
       SDS_NUMPIXELS_land(IDATA,1)=IBLUE
       SDS_NUMPIXELS_land(IDATA,2)=IRED
       
C
        if(IPROCE . le.4) then
       SDS_mass_conc_land(IDATA)=MASSCON*SCALE1+OFFSET1
        IF(ANGSTROM.LT.5.0.AND.ANGSTROM.GT.0.0) THEN
        SDS_angs_coeff_land(IDATA)=ANGSTROM*SCALE3+OFFSET3
       ENDIF
       IF(ANGSTROM.GE.5.0) THEN
        SDS_angs_coeff_land(IDATA)=-9999
       ENDIF
      IF(ANGSTROM.LE.0.0) THEN
        SDS_angs_coeff_land(IDATA)=-9999
      ENDIF
      IF(TAUABLUE.GT.0.0.AND.TAUARED.GT.0.0) THEN
        SDTAUCBLUE=ABS((TAUABLUENN-TAUABLUE)*(TAUABLUEC/TAUABLUE))
        SDTAUCRED= ABS((TAUAREDNN-TAUARED)*(TAUAREDC/TAUARED))
      ENDIF
      SDS_est_uncer(IDATA,1)=NINT(SDTAUCBLUE * SCALE3 + OFFSET3)
      SDS_est_uncer(IDATA,2)=NINT(SDTAUCRED  * SCALE3 + OFFSET3)
      IF(TAUAGREENC.EQ.0.0) THEN
        SDS_est_uncer(IDATA,1)=-9999
        SDS_est_uncer(IDATA,2)=-9999
      ENDIF
C    
      SDS_RefF_land(IDATA,1)=NINT(FLUXUPBLUE * SCALE3 + OFFSET3)
      SDS_RefF_land(IDATA,3)=NINT(FLUXUPRED * SCALE3 + OFFSET3)
      SDS_RefF_land(IDATA,2)=NINT(0.5*(FLUXUPRED+FLUXUPBLUE) * SCALE3 + OFFSET3)
      SDS_TranF_land(IDATA,1)=NINT(FLUXDNBLUE * SCALE3 + OFFSET3)
      SDS_TranF_land(IDATA,2)=NINT(FLUXDNRED * SCALE3 + OFFSET3)
       else
       SDS_RefF_land(IDATA,1)=-9999
       SDS_RefF_land(IDATA,3)=-9999
       SDS_RefF_land(IDATA,2)=-9999
       SDS_TranF_land(IDATA,1)=-9999
       SDS_TranF_land(IDATA,2)=-9999
        SDS_est_uncer(IDATA,1)=-9999
        SDS_est_uncer(IDATA,2)=-9999
        SDS_angs_coeff_land(IDATA)=-9999
        SDS_mass_conc_land(IDATA)=-999.0
      ENDIF
C
      SDS_ref_land(IDATA,1) = NINT(BARLBLUE * SCALE4 + OFFSET4)
      SDS_ref_land(IDATA,2) = NINT(BARLRED * SCALE4 + OFFSET4)
      SDS_ref_land(IDATA,3) = NINT(BARL0P86 * SCALE4 + OFFSET4)
      SDS_ref_land(IDATA,4) = NINT(2.0*BARRRED * SCALE4 + OFFSET4)
      SDS_ref_land(IDATA,5) = NINT(0.0*SCALE4 + OFFSET4)
c
      SDS_ref_STD_land(IDATA,1) =NINT(SDBLUE * SCALE4 + OFFSET4)
      SDS_ref_STD_land(IDATA,2) =NINT(SDRED * SCALE4 + OFFSET4)
      SDS_ref_STD_land(IDATA,3) =NINT(SD0P86 * SCALE4 + OFFSET4)
      SDS_ref_STD_land(IDATA,4) =NINT(4.0*SDBLUE * SCALE4 + OFFSET4)
      SDS_ref_STD_land(IDATA,5) =NINT(0.0*SCALE4 + OFFSET4)

      ENDIF
   
C
C Store QA flags into Quality_Assurance_Land array according to the order
C of bits in MODIS atmosphere QA plan
C
      QA_Temp=0
      CALL BYTE_SET(QA_Flag_Land(7),0,QA_Temp)
      CALL BYTE_SET(QA_Flag_Land(8),1,QA_Temp)
      CALL BYTE_SET(QA_Flag_Land(9),4,QA_Temp)
      CALL BYTE_SET(QA_Flag_Land(10),5,QA_Temp)

      SDS_QCONTROL_land(1,IDATA)=QA_Temp

      QA_Temp=0
      QA_Flag_Land(13)=0
      QA_Flag_Land(14)=0
      CALL BYTE_SET(QA_Flag_Land(11),0,QA_Temp)
      CALL BYTE_SET(QA_Flag_Land(12),3,QA_Temp)
      CALL BYTE_SET(QA_Flag_Land(13),6,QA_Temp)
      CALL BYTE_SET(QA_Flag_Land(14),7,QA_Temp)

      SDS_QCONTROL_land(2,IDATA)=QA_Temp

      QA_Temp=0
      QA_Flag_Land(17)=3
      QA_Flag_Land(18)=3

      CALL BYTE_SET(QA_Flag_Land(15),0,QA_Temp)
      CALL BYTE_SET(QA_Flag_Land(16),2,QA_Temp)
      CALL BYTE_SET(QA_Flag_Land(17),4,QA_Temp)
      CALL BYTE_SET(QA_Flag_Land(18),6,QA_Temp)

      SDS_QCONTROL_land(3,IDATA)=QA_Temp

      QA_Temp=0
      QA_Flag_Land(19)=1
      CALL BYTE_SET(QA_Flag_Land(19),0,QA_Temp)
 
      SDS_QCONTROL_land(4,IDATA)=QA_Temp

      SDS_QCONTROL_land(5,IDATA)=0

C
C Re-initialized working variables
C
 

      DO I=1,19 
        QA_Flag_Land(I)=0
      ENDDO
      TAUAGREENC=-9999 
      TAUABLUEC=-9999 
      TAUAREDC=-9999 
      TAUABLUE=-9999 
      TAUARED=-9999 
      SDBLUE=-9999 
      SDRED=-9999 
      SD0P86=-9999 
      ISULP=-9999
      ISMK=-9999
      IAER=-9999
      IBLUE=-9999
      IRED=-9999
      IPROCE=-9999
      WEIGHT=-9999
      SAVERATIO=-9999 
      IERROR=-9999
      BARLRED=-9999 
      BARLBLUE=-9999 
      BARL0P86=-9999 
      SDTAUCBLUE=-9999 
      SDTAUCRED=-9999 
C
100   FORMAT(i4,10(10f10.4,/))
     
      RETURN
      END


C***********************************************************************
      SUBROUTINE COMPUTE_SCATTANGLE_LAND(MTHET0,MTHET,MDPHI,IDATA,
     *SCAT_ANGLE_LAND)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              This subroutine computes scattering angle from MODIS
C              geometry.
C
C!INPUT PARAMETERS:
C
C       MEHET0            Solar zenith angle
C       MTHET             Satellite Viewing angle
C       MDPHI             Relative azimuth angle
C       IDATA             Index of 10x10 km box
C
C!OUTPUT PARAMETERS:
C
C       SCAT_ANGLE_LAND   Scattering angle
C
C!REVISION HISTORY:
C 02/02/98 achu
C identified input/output arguments in INPUT / OUTPUT PARAMETERS sections
C in prolog.
C
C 01/28/98 fhliang
C fixed prolog.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol Team at NASA GSFC, Greenbelt, MD
C
C!DESIGN NOTES:
C
C!END
C-----------------------------------------------------------------------
C
      IMPLICIT NONE
      INCLUDE 'mod04.inc'
C
C      DTR=ACOS(-1.)/180.; DTR=0.1745329
C      RTD=180./ACOS(-1.); RTD=57.29577
C
      INTEGER IDATA
      SAVE
      REAL DEGRAD,RADDEG,SCAT_ANGLE_LAND
      REAL MTHET0,MTHET,MDPHI
      SCAT_ANGLE_LAND=0.0

C
C Scattering angle of MODIS geometry
C

       SCAT_ANGLE_LAND= -COS(MTHET0*DTR)*COS(MTHET*DTR)
     *                  +SIN(MTHET0*DTR)*SIN(MTHET*DTR)
     *                  *COS(MDPHI*DTR)

       SCAT_ANGLE_LAND= ACOS(SCAT_ANGLE_LAND)*RTD

       RETURN
       END

C*****************************************************************

      SUBROUTINE  statistics_land(W470_SYN,W659_SYN,W213_SYN,CLDMSK_500,
     *      SDS_Mean_Reflectance_Land_All,
     *      SDS_SDev_Reflectance_Land_All,
     *      SDS_Path_Radiance_Land,
     *      SDS_Critical_Reflectance_Land,
     *      SDS_Error_Crit_Reflectance_Land,
     *      SDS_Error_Path_Radiance_Land,
     *      SDS_QFlag_Critical_Ref_Land,SDS_QFlag_Path_Radiance_Land,
     *  START_1KM,END_1KM,Idata,iscan, MTHET0,MTHET,
     *  SDS_QCONTROL_CritRef_land,SCAT_ANGLE_LAND,Rayleigh_look,W865_SYN)

C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              This subroutine computes different Statistics like path radiance,slopes and
C error for land
C
C!INPUT PARAMETERS:
C
C        W470_SYN        Reflectance for 0.470 um 
C        W659_SYN        Reflectance for 0.659 um   
C       W213_SYN         Reflectance for 2.13 um            
C       CLDMSK_500    Cloud mask   
C       MEHET0             Solar zenith angle
C       MTHET               Satellite Viewing angle
C       IDATA                Index of 10x10 km box
C
C!OUTPUT PARAMETERS:
C
C      SDS_Mean_Reflectance_Land_All 
C      SDS_SDev_Reflectance_Land_All 
C      SDS_Path_Radiance_Land 
C      SDS_Critical_Reflectance_Land 
C      SDS_Error_Crit_Reflectance_Land 
C      SDS_Error_Path_Radiance_Land 
C      SDS_QFlag_Critical_Ref_Land
C     SDS_QFlag_Path_Radiance_Land
C!REVISION HISTORY:
CDeveloped 4/2001
C
C!TEAM-UNIQUE HEADER:
C
C Developed by Shana Mattoo (Aerosol Team at NASA GSFC, Greenbelt, MD
C
C!DESIGN NOTES:
C
C!END
C-----------------------------------------------------------------------
         IMPLICIT NONE
         SAVE
       INCLUDE 'mod04.inc'
       REAL W659_SYN(4*ISWATH,4*ILINE),W865_SYN(4*ISWATH,4*ILINE),
     *     W470_SYN(2*ISWATH,2*ILINE),W550_SYN(2*ISWATH,2*ILINE),
     *     W124_SYN(2*ISWATH,2*ILINE),W164_SYN(2*ISWATH,2*ILINE),
     *     W213_SYN(2*ISWATH,2*ILINE),
     *     W443o_SYN(ISWATH,ILINE),W551o_SYN(ISWATH,ILINE),
     *     W667o_SYN(ISWATH,ILINE),W869o_SYN(ISWATH,ILINE)
      REAL arefw659,arefw865,AVE,SDEV,array(2*IGRIDX*2*IGRIDY)
      REAL ref_interm(3,2*IGRIDX*2*IGRIDY),SCAT_ANGLE_LAND
      REAL diff_full_half,final_diff_full_half(Land_Sol1)
      INTEGER CLDMSK_500(2*ISWATH,2*ILINE),Flag_full
      INTEGER Final_Flag_full(Land_Sol1)
      INTEGER START_1KM,END_1KM,JJ,II
      INTEGER  QCONTROL_land_wav1,QCONTROL_land_wav2
      INTEGER *2  SDS_Mean_Reflectance_Land_All(NUMCELLS,Land_Sol3),
     &SDS_SDev_Reflectance_Land_All(NUMCELLS,Land_Sol3),
     &SDS_Path_Radiance_Land(NUMCELLS,Land_Sol1),
     &  SDS_Critical_Reflectance_Land(NUMCELLS,Land_Sol1),
     &  SDS_Error_Crit_Reflectance_Land(NUMCELLS,Land_Sol1),
     &  SDS_Error_Path_Radiance_Land(NUMCELLS,Land_Sol1),
     &  SDS_QFlag_Critical_Ref_Land(NUMCELLS,Land_Sol1),
     &  SDS_QFlag_Path_Radiance_Land(NUMCELLS,Land_Sol1)
          BYTE  SDS_QCONTROL_CritRef_land(QA_LAND,NUMCELLS)
      INTEGER NUMDATA_659,NUMDATA_865,NUMDATA_470,ij
      integer index_error_flags(Land_Sol1,12),
     * NUMDATA_550,NUMDATA_124,NUMDATA_164,NUMDATA_213,NUMDATA_443o,
     * NUMDATA_551o,NUMDATA_667o, NUMDATA_869o
      INTEGER JMASK,IMASK,IBLUE,JBLUE,IXX,IYY,IX,IY,IIJ,IIK,No_water
      INTEGER IDUMMY,kdata,iwav,iscan,idata,numdata,inum,ik,index_wave
      Real array_47(2*ISWATH*2*ILINE),array_67(2*ISWATH*2*ILINE),SIG
      real A,B,SIGA,SIGB,CHI2,Q,array_213(2*ISWATH*2*ILINE),abdev
      real Average_allwaves(Land_Sol3),standard_allwaves(Land_Sol3)
      real log_standard,MTHET0,MTHET 
      integer MWT,INDX(2*ISWATH*2*ILINE),numaver1
      real Rayleigh_look(Land_Sol1),co_variance,Surface
      real THRSHOLD1,THRSHOLD2,SCATMODIS,Diff_path,Path_rad(Land_Sol1)
      real Error_Pathrad(Land_Sol1),critic_ref(Land_Sol1)
      real error_critic(Land_Sol1),slope(Land_Sol1)
      integer weighting_critical(Land_Sol1),weighting_path(Land_Sol1),
     *     Qa_Flag_final(Land_Sol1)
      real chisq_for_flag,diff_critic_ref
      real final_chisq_for_flag(Land_Sol1) 
      real final_diff_critic_ref(Land_Sol1)
      integer index_outlayer(2*ISWATH*2*ILINE),index_inlayer(2*ISWATH*2*ILINE)
c intialize
        No_water=0
        aREFW659=0.0
        NUMDATA_470=0
        NUMDATA_659=0
        NUMDATA_213=0
         numaver1=0
         numdata=0
c

c loop around 1 km

        DO   IYY = 1,IGRIDY
         IMASK=4*IYY-3
         IBLUE=2*IYY-1
        DO  IXX=START_1KM,END_1KM
          JMASK=4*IXX-3
          JBLUE=2*IXX-1
c loop around 500 meters
       DO IY =IBLUE,2*IYY
           IIJ=2*IY-1
         DO IX= JBLUE,2*IXX
            IIK=2*IX-1
         IF(CLDMSK_500(IX,IY) .GT. 0) THEN
c   For each clear pixel find product of
c   Reflectance for 0.659 um
             numaver1=0
             AREFW659=0.0
c loop around 250 meters
            DO JJ= IIJ,2*IY
            DO II= IIK,2*IX
       IF(W659_syn(II,JJ).GT. 0.0 .AND. W659_syn(II,JJ).LE. 1.0) THEN
       numaver1=numaver1+ CLDMSK_500(IX,IY)
       AREFW659=AREFW659+(W659_syn(II,JJ)*CLDMSK_500(IX,IY))
        ENDIF
C ENDDO FOR 250METER RESOLUTION
            ENDDO
            ENDDO
c loop around 250 meters to find water pixels....
            No_water=0
            DO JJ= IIJ,2*IY
            DO II= IIK,2*IX
       IF(W865_SYN(IX,IY)-W659_syn(II,JJ).GT.0.1) THEN
       No_water=No_water+1 
        ENDIF
C ENDDO FOR 250METER RESOLUTION
            ENDDO
            ENDDO
c All pixels for 250 resolution have to be clear
      IF( numaver1 .EQ.4 .and. No_water .eq.4) then
c
       NUMDATA_659=NUMDATA_659+1

c    Foe clear pixel find product all Reflectances for
c    channel 3-channel 7 and cloud mask
C
C     WAVE 0.470 UM
C
      IF(W470_SYN(IX,IY).GT. 0.0 .AND.W470_SYN(IX,IY).LE. 1.0) THEN
          NUMDATA_470= NUMDATA_470+1

         ENDIF



C     WAVE 2.13 UM
C

        IF(W213_syn(IX,IY).GT. 0.0 .AND. W213_syn(IX,IY).LE. 1.0) THEN
         NUMDATA_213= NUMDATA_213+1

         ENDIF
c endif or all good  pixels in 250 meter resolution
         endif
      
c  register same pixels.........
       IF( NUMDATA_470 .eq.NUMDATA_659 .and.
     *   NUMDATA_659 .eq.NUMDATA_213 .and.numaver1 .EQ.4 ) then
c eliminate water pixels
        if( NUMDATA_659 .gt. 0 .and.W213_syn(IX,IY) .gt.0.01 ) then
        numdata=numdata+1
        ref_interm(1, numdata)=(W470_syn(IX,IY)*  CLDMSK_500(IX,IY))
        ref_interm(2, numdata)=AREFW659/numaver1
        ref_interm(3, numdata)=(W213_syn(IX,IY)*CLDMSK_500(IX,IY))
               endif
c  endif for equal pixels
        ENDif
c endif for all cloudy pixels
       ENDIF
c    ENDDO for 500 * 500 meter 20*20 box
          ENDDO
          ENDDO
C      ENDDO's for 1km pixels for 10*10 box
         ENDDO
         ENDDO
c Compute average and standard deviation.....
        AVE=0.0
        SDEV=0.0

c   fine average and standard deviation for data if cloud free and good pixels
c   meet the criteria

      IF( numdata .gt. 30) then
c order the 2.13 data
       Do kdata=1,numdata
            array(kdata)=ref_interm(3,kdata)
       Enddo
         CALL INDEXX(numdata,array,INDX)
c
         Do Iwav =1 ,Land_Sol3
            Do kdata=1,numdata
            array(kdata)=ref_interm(iwav,INDX(kdata))
           enddo
         call ave_std(array,numdata,AVE,SDEV)
        Average_allwaves(iwav)=ave
        standard_allwaves(iwav)=SDEV

c Save SDS for HDF file.......

       SDS_Mean_Reflectance_Land_All(IDATA,iwav)=
     *nint(AVE * SCALE4 + OFFSET4)
       SDS_SDev_Reflectance_Land_All(IDATA,iwav)=
     * nint(SDEV * SCALE4 + OFFSET4)
c calculate log deviation for only 2.13 to be used in tests
         Do kdata=1,numdata
            array(kdata)=alog(ref_interm(3,INDX(kdata)))
           enddo
         call ave_std(array,numdata,AVE,SDEV)
         log_standard=SDEV
c Enddo for  Wavelengths
      enddo

c  call fitting routine only when numdata meets criteria...
            Do kdata=1,numdata
            array_47(kdata)=ref_interm(1,INDX(kdata))
            array_67(kdata)=ref_interm(2,INDX(kdata))
            array_213(kdata)=ref_interm(3,INDX(kdata))
           enddo 

cfit for wav 47 * 2.13 um
       do Iwav =1,2
         Path_rad(iwav)=-99
         slope(iwav)=-99
          kdata=0
         if( Iwav .eq.1)then 
         Surface=Surface_wav47
        call medfit(array_213,array_47,numdata,A,B,Siga,Sigb,abdev,
     *Average_allwaves,standard_allwaves,log_standard,iscan,idata,
     *SDS_QCONTROL_CritRef_land,SCAT_ANGLE_LAND,co_variance,
     *Flag_full,diff_full_half,surface,chisq_for_flag,diff_critic_ref,
     * index_outlayer,index_inlayer)
        
c fit for wav 67 * 2.13 um
       Else 
        Surface=Surface_wav66
       call medfit(array_213,array_67,numdata,A,B,Siga,Sigb,abdev,
     *Average_allwaves,standard_allwaves,log_standard,iscan,idata,
     *SDS_QCONTROL_CritRef_land,SCAT_ANGLE_LAND,co_variance,
     *Flag_full,diff_full_half,surface,chisq_for_flag,diff_critic_ref,
     *index_outlayer,index_inlayer) 
        endif
c
c A is path radiance B is slope sigma error in path radiance sigb is error in slope
c index wave sets the index in the fillvalue routine
c write only if slope and path radiance are positive
        index_wave=iwav
      if( A. gt.0 .and. B .gt.0) then
        Path_rad(iwav)=A
        slope(iwav)=B
        Error_Pathrad(iwav)=SIGA
        critic_ref(iwav)=A/(1.-B/Surface)
         error_critic(iwav)=sqrt((SIGA**2)+(2.*critic_ref(iwav)*Co_variance)+
     *                        (SIGB**2)*(critic_ref(iwav)**2))
         Final_Flag_full(iwav)=Flag_full
         Final_diff_full_half(iwav)= diff_full_half
         final_chisq_for_flag(iwav)=chisq_for_flag
         final_diff_critic_ref(iwav)=diff_critic_ref
      Else
           kdata=kdata+1
         CALL FILLVALUE_LAND_extra(IDATA,index_wave,
     *      SDS_Mean_Reflectance_Land_All,
     *      SDS_SDev_Reflectance_Land_All,
     *      SDS_Path_Radiance_Land,
     *      SDS_Critical_Reflectance_Land,
     *      SDS_Error_Crit_Reflectance_Land,
     *      SDS_Error_Path_Radiance_Land,
     *      SDS_QFlag_Critical_Ref_Land,
     *      SDS_QFlag_Path_Radiance_Land)
              QCONTROL_land_wav1=0
              QCONTROL_land_wav2=0
              call  Fill_QAflag_CritRef_land(QCONTROL_land_wav1,
     *QCONTROL_land_wav2, SDS_QCONTROL_CritRef_land,Idata)
       Endif
c ENDDO for wavelength
       Enddo
        CAll Flag_analysis(Rayleigh_look,SCAT_ANGLE_LAND,Path_rad,
     *   Error_Pathrad,critic_ref,error_critic,index_error_flags,
     *   Final_Flag_full,Final_diff_full_half,weighting_critical,
     *  weighting_path,Qa_Flag_final,final_chisq_for_flag,
     *final_diff_critic_ref)
c  Quality flag for final qa flag
        QCONTROL_land_wav1=Qa_Flag_final(1)
        QCONTROL_land_wav2=Qa_Flag_final(2)
        call  Fill_QAflag_CritRef_land(QCONTROL_land_wav1,
     *QCONTROL_land_wav2,
     * SDS_QCONTROL_CritRef_land,Idata)
        Do  iwav=1,2
         index_wave=iwav
       if( Path_rad(iwav). gt.0 .and. slope(iwav) .gt.0) then
c path radiance is corrected for the view angle...........
       SDS_Path_Radiance_Land(IDATA,iwav)=
     $  nint((Path_rad(Iwav)*(COS(MTHET*DTR)))*SCALE4 + OFFSET4)
c Error in path radiance is corrected for the view angle...........
      SDS_Error_Path_Radiance_Land(IDATA,iwav)=
     $ nint((Error_Pathrad(iwav)*(COS(MTHET*DTR)))*SCALE4 + OFFSET4)
c Critical Reflactance
        SDS_Critical_Reflectance_Land(IDATA,iwav)=
     *          nint(critic_ref(iwav)* SCALE4 + OFFSET4)
c Error Critical Reflactance
          SDS_Error_Crit_Reflectance_Land(IDATA,Iwav)=
     *nint(error_critic(iwav)* SCALE4 + OFFSET4)
c Quality flag for PAth radiance
      SDS_QFlag_Critical_Ref_Land(IDATA,Iwav)=
     *               weighting_critical(iwav)
c Quality flag for PAth radiance
      SDS_QFlag_Path_Radiance_Land(IDATA,Iwav)=
     *           weighting_path(iwav)
       ELSE
      CALL FILLVALUE_LAND_extra(IDATA,index_wave,
     *      SDS_Mean_Reflectance_Land_All,
     *      SDS_SDev_Reflectance_Land_All,
     *      SDS_Path_Radiance_Land,
     *      SDS_Critical_Reflectance_Land,
     *      SDS_Error_Crit_Reflectance_Land,
     *      SDS_Error_Path_Radiance_Land,
     *      SDS_QFlag_Critical_Ref_Land,
     *      SDS_QFlag_Path_Radiance_Land)
              QCONTROL_land_wav1=0
              QCONTROL_land_wav2=0
              call  Fill_QAflag_CritRef_land(QCONTROL_land_wav1,
     *QCONTROL_land_wav2, SDS_QCONTROL_CritRef_land,Idata)
       Endif
C ENDDO FOR WAVELENGTHS
       ENDDO

         ELSE
c  call fill values
             index_wave=3
            CALL FILLVALUE_LAND_extra(IDATA,index_wave,
     *      SDS_Mean_Reflectance_Land_All,
     *      SDS_SDev_Reflectance_Land_All,
     *      SDS_Path_Radiance_Land,
     *      SDS_Critical_Reflectance_Land,
     *      SDS_Error_Crit_Reflectance_Land,
     *      SDS_Error_Path_Radiance_Land,
     *      SDS_QFlag_Critical_Ref_Land,SDS_QFlag_Path_Radiance_Land)
         QCONTROL_land_wav1=0
         QCONTROL_land_wav2=0
        call  Fill_QAflag_CritRef_land(QCONTROL_land_wav1,
     *QCONTROL_land_wav2,
     * SDS_QCONTROL_CritRef_land,Idata)
c endif  for criteria for number of points
        ENDIF
100     format( 2i7,f10.3,10i15)
         RETURN
         END

c
         SUBROUTINE MEDFIT(X,Y,NDATA,A,B,Siga,Sigb,ABDEV,
     *Average_allwaves,standard_allwaves,log_standard, iscan,idata,
     *SDS_QCONTROL_CritRef_land,SCAT_ANGLE_LAND,co_variance,
     *Flag_full,diff_full_half,surface,chisq_for_flag,diff_critic_ref,
     *index_outlayer,index_inlayer)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              This subroutine was taken from numrical recepies but was extensivily
C modified. The routine is basic fitting routine ands calculates slopes etc.

C!INPUT PARAMETERS:
C
C!INPUT PARAMETERS:
C     REAL             X                   elements of array X 
C     REAL             X                   elements of array Y 
C    INTEGER      NDATA          Number of data pointa
C!OUTPUT PARAMETERS:
C
C    A                 Intercept
C    B                 Slope
C    Siga             Error in Intercept
C    Sigb             Error in Slope
C    ABDEV       Absolute Deviation
C
C!REVISION HISTORY:
CDeveloped 4/2001
C
C!TEAM-UNIQUE HEADER:
C
C Developed by Shana Mattoo (Aerosol Team at NASA GSFC, Greenbelt, MD
C
C!DESIGN NOTES:
C
C!END
C-----------------------------------------------------------------------
         IMPLICIT NONE
         SAVE
         INCLUDE 'mod04.inc'
         INTEGER NMAX,NDATA,j,ik,NDATAT,numpoints_outerlayer
         integer  numpoints_half,iscan,idata
        integer index_outlayer(ndata),index_inlayer(ndata)
       integer  numpoints_inlayer,numpoints_half_inlayer,im
       integer numpoints_outerlayer_half,Flag_full
      PARAMETER (NMAX=1000)
      EXTERNAL ROFUNC
       real Average_allwaves(3),a,b,abdev,aa,ABDEVT,arr,del,CHISQ
       real standard_allwaves(3),log_standard,Co_variance
       real st_dev,b1,F1,b2,f2,f,xt,yt,bb,ROFUNC
      COMMON /ARRAYS/ NDATAT,XT(NMAX),YT(NMAX),ARR(NMAX),AA,ABDEVT
      REAL X(NDATA),Y(NDATA) 
       real Siga,Sigb,diff_full_half
       real SX,SY,SXY,SXX,SX_half,sy_half,SXY_half,SXX_half
       real SX_outlayer,SY_outlayer,SXY_outlayer,SXX_outlayer
      real SX_outlayer_half,SY_outlayer_half
      real SXY_outlayer_half,SXX_outlayer_half
       real  SX_noutlayer,SY_noutlayer,SXY_noutlayer,SXX_noutlayer
       real  SX_half_noutlayer,SY_half_noutlayer,SXY_half_noutlayer,
     *SXX_half_noutlayer,SCAT_ANGLE_LAND
       real AA_noutlayer,BB_noutlayer,CHISQ_noutlayer
       real AA_half_noutlayer,BB_half_noutlayer,CHISQ_half_noutlayer
       real  Sigma_all_one,Sigma_all_two,Sigma_half_one,Sigma_half_two
       BYTE  SDS_QCONTROL_CritRef_land(QA_LAND,NUMCELLS)
       real Co_variance_noutlayer,Co_variance_noutlayer_half,surface
       real chisq_for_flag,diff_critic_ref
c intialize
      DO  J=1,NDATA
               index_outlayer(j)=0
               index_inlayer(j)=0
      ENDDO
      Siga=0.
      Sigb=0.
      SX=0.
      SY=0.
      SXY=0.
      SXX=0.
      SX_half=0.
      sy_half=0.
      SXY_half=0.
      SXX_half=0.
      numpoints_half=0
c  compute sums for whole array and array till Average value
      DO 11 J=1,NDATA
        XT(J)=X(J)
        YT(J)=Y(J)
        SX=SX+X(J)
        SY=SY+Y(J)
        SXY=SXY+X(J)*Y(J)
        SXX=SXX+X(J)**2
      if(X(J).ge.X(1).and.X(J).le.Average_allwaves(3))then
       numpoints_half= numpoints_half+1
      SX_half=SX
      SY_half=SY
      SXY_half=SXY
      SXX_half=SXX
      endif
11    CONTINUE
c  compute  slope,intecept and chisqare
      NDATAT=NDATA
      DEL=NDATA*SXX-SX**2
      AA=(SXX*SY-SX*SXY)/DEL
      BB=(NDATA*SXY-SX*SY)/DEL
      CHISQ=0.
      DO 12 J=1,NDATA
        CHISQ=CHISQ+(Y(J)-(AA+BB*X(J)))**2
12    CONTINUE
      SIGB=SQRT(CHISQ/DEL)
      B1=BB
      F1=ROFUNC(B1)
      B2=BB+SIGN(3.*SIGB,F1)
      F2=ROFUNC(B2)
1     IF(F1*F2.GT.0.)THEN
        BB=2.*B2-B1
        B1=B2
        F1=F2
        B2=BB
        F2=ROFUNC(B2)
        GOTO 1
      ENDIF
      SIGB=0.01*SIGB
2     IF(ABS(B2-B1).GT.SIGB)THEN
        BB=0.5*(B1+B2)
        IF(BB.EQ.B1.OR.BB.EQ.B2)GOTO 3
        F=ROFUNC(BB)
        IF(F*F1.GE.0.)THEN
          F1=F
          B1=BB
        ELSE
          F2=F
          B2=BB
        ENDIF
        GOTO 2
      ENDIF
3     A=AA
      B=BB
      ABDEV=ABDEVT/NDATA
c compute indexs for outer layers to elminate them
      ik=0
      im=0
      numpoints_outerlayer=0
      numpoints_outerlayer_half=0
      st_dev=2*SQRT(CHISQ/(NDATA-1))
      DO  J=1,NDATA
         if (Y(J)-(A+B*X(J)) .gt. st_dev)then
               ik=ik+1
               index_outlayer(ik)=j
       else
               im=im+1
               index_inlayer(im)=j
       endif
      ENDDO
       numpoints_outerlayer=ik

c check the points in outer layer till average 

       do J=1,numpoints_half
       if(index_outlayer(j) .gt.0 .and.
     *   index_outlayer(j) .le.numpoints_half) then
      numpoints_outerlayer_half=numpoints_outerlayer_half+1

       endif
      enddo
c compute sums for outlayers
c
      SX_outlayer=0.
      SY_outlayer=0.
      SXY_outlayer=0.
      SXX_outlayer=0.
      SX_outlayer_half=0.
      SY_outlayer_half=0.
      SXY_outlayer_half=0.
      SXX_outlayer_half=0.
      SX_noutlayer=0
      SY_noutlayer=0.
      SXY_noutlayer=0.
      SXX_noutlayer=0.
      SX_half_noutlayer=0
      SY_half_noutlayer=0.
      SXY_half_noutlayer=0.
      SXX_half_noutlayer=0.

         DO  J=1,numpoints_outerlayer
        SX_outlayer=SX_outlayer+X(index_outlayer(j))
        SY_outlayer=SY_outlayer+Y(index_outlayer(j))
        SXY_outlayer=SXY_outlayer+
     *X(index_outlayer(j))*Y(index_outlayer(j))
        SXX_outlayer=SXX_outlayer+X(index_outlayer(j))**2
        ENDDO

c compute sum without outer_layers
c
c     for all points
      SX_noutlayer=SX - SX_outlayer
      SY_noutlayer=SY - SY_outlayer
      SXY_noutlayer=SXY - SXY_outlayer
      SXX_noutlayer=SXX -  SXX_outlayer

c     for half of points till average
       DO  J=1,numpoints_outerlayer_half
        SX_outlayer_half=SX_outlayer_half+X(index_outlayer(j))
        SY_outlayer_half=SY_outlayer_half+Y(index_outlayer(j))
        SXY_outlayer_half=SXY_outlayer_half+
     *X(index_outlayer(j))*Y(index_outlayer(j))
        SXX_outlayer_half=SXX_outlayer_half+X(index_outlayer(j))**2
        ENDDO
      SX_half_noutlayer=SX_half - SX_outlayer_half
      SY_half_noutlayer=SY_half - SY_outlayer_half
      SXY_half_noutlayer=SXY_half - SXY_outlayer_half
      SXX_half_noutlayer=SXX_half -  SXX_outlayer_half

c  compute new A and B for half of points and all points without outer layers

c    for all points without outer layer

       numpoints_inlayer  = NDATA - numpoints_outerlayer

c  compute chisq for half and all points without outerlayer


        AA_noutlayer=0.
        BB_noutlayer=0.
        CHISQ_noutlayer=0.
        Sigma_all_one=0.
         Sigma_all_two=0.
        Co_variance_noutlayer=0.0

         DEL=numpoints_inlayer*SXX_noutlayer-SX_noutlayer**2
        if( DEL .gt.0.0) then
         AA_noutlayer=
     * (SXX_noutlayer*SY_noutlayer - SX_noutlayer * SXY_noutlayer)/DEL
         BB_noutlayer=
     *( numpoints_inlayer*SXY_noutlayer-SX_noutlayer*SY_noutlayer)/DEL
c calculate     CHISQ
          DO   J=1,numpoints_inlayer
          CHISQ_noutlayer=CHISQ_noutlayer+
     *             (Y( index_inlayer(j))-
     *      (AA_noutlayer+BB_noutlayer*X( index_inlayer(j))))**2
           ENDDO
       endif

      
      
c calculate Sigma all points without outer layers
         numpoints_half_inlayer  =numpoints_half -
     *             numpoints_outerlayer_half
c   calculate    Co_variance
            if( DEL .gt.0.0 .and.numpoints_inlayer .gt.0 ) then
          Co_variance_noutlayer=
     * -(1./DEL)*(SX_noutlayer*(CHISQ_noutlayer/numpoints_inlayer))
        Sigma_all_one=((1./DEL)*(SXX_noutlayer*CHISQ_noutlayer))
        Sigma_all_two=((numpoints_inlayer/DEL)*CHISQ_noutlayer)
        Sigma_all_one=SQRT(Sigma_all_one)
     *                          /SQRT(real(numpoints_inlayer))  
        Sigma_all_two=SQRT(Sigma_all_two)
     *                           /SQRT(real(numpoints_inlayer))
         endif
    
       AA_half_noutlayer=0.        
       BB_half_noutlayer=0.
      CHISQ_half_noutlayer=0.
       Sigma_half_one=0.
       Sigma_half_two=0.
      Co_variance_noutlayer_half=0.0
c
      DEL=numpoints_half_inlayer*SXX_half_noutlayer-SX_half_noutlayer**2
         if( DEL .gt.0.0) then
        AA_half_noutlayer=
     * (SXX_half_noutlayer*SY_half_noutlayer -
     *SX_half_noutlayer * SXY_half_noutlayer)/DEL
       BB_half_noutlayer=( numpoints_half_inlayer*
     *SXY_half_noutlayer-SX_half_noutlayer*SY_half_noutlayer)/DEL
c calculate Sigma half points without outer layers

       DO   J=1, numpoints_half_inlayer
          CHISQ_half_noutlayer=CHISQ_half_noutlayer+
     * (Y( index_inlayer(j))-
     *(AA_half_noutlayer+BB_half_noutlayer*X( index_inlayer(j))))**2
        ENDDO
        endif
c   calculate    Co_variance
          if(DEL .gt.0.0 .and.  numpoints_half_inlayer .gt.0) then
          Co_variance_noutlayer_half=
     * -(1./DEL)*(SX_half_noutlayer*
     *  (CHISQ_half_noutlayer/numpoints_half_inlayer))
         Sigma_half_one=((1./DEL)*(SXX_half_noutlayer
     $         *CHISQ_half_noutlayer))
        Sigma_half_two=((numpoints_half_inlayer/DEL)
     $   *   CHISQ_half_noutlayer) 
          Sigma_half_one=SQRT(Sigma_half_one)
     $     /SQRT(real(numpoints_half_inlayer))
          Sigma_half_two=SQRT(Sigma_half_two)
     $                             /SQRT(real(numpoints_half_inlayer))

           endif
c  consistancy checks for positivity of intercept and slopes

         if((AA_half_noutlayer .gt. 0.0 .and.AA_half_noutlayer .le. 1.0)
     * .and.(BB_half_noutlayer .gt. 0.0 .and .
     *       BB_half_noutlayer .le. 1.0) .or.
     * (AA_noutlayer .gt.0.0 .and.AA_noutlayer .le.1.0) .and.
     * (BB_noutlayer .gt. 0.0 .and . BB_noutlayer.le. 1.0)) then
c
c   call subroutine tests to test if full or half of the points are to be used 
c   for fit
       Call tests( AA_half_noutlayer,BB_half_noutlayer,AA_noutlayer,
     *BB_noutlayer,Sigma_half_one,Sigma_half_two,Sigma_all_one,
     *Sigma_all_two,Average_allwaves,standard_allwaves,log_standard,
     *iscan,idata,A,B,Siga,Sigb,
     *SCAT_ANGLE_LAND,numpoints_inlayer,numpoints_half_inlayer,
     *CHISQ_noutlayer,CHISQ_half_noutlayer,CHISQ,Flag_full,diff_full_half,
     *Co_variance_noutlayer,Co_variance_noutlayer_half,Co_variance,
     *chisq_for_flag,diff_critic_ref,surface)
       else
        A=-99.0
        B=-99.0
        siga=-99.0
        sigb=-99.0
        endif
      RETURN
      END
c
      FUNCTION ROFUNC(B)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              This function is taken from numrical recepies 
C
C!INPUT PARAMETERS:
C     REAL             B                   
C
C!OUTPUT PARAMETERS:
C
C   ROFUN
C
C!REVISION HISTORY:
CDeveloped 4/2001
C
C!TEAM-UNIQUE HEADER:
C
C 
C
C!DESIGN NOTES:
C
C!END
C 
C-----------------------------------------------------------------------
         SAVE
      INTEGER NMAX
      PARAMETER (NMAX=1000)
      COMMON /ARRAYS/ NDATA,X(NMAX),Y(NMAX),ARR(NMAX),AA,ABDEV
      N1=NDATA+1
      NML=N1/2
      NMH=N1-NML
      DO 11 J=1,NDATA
        ARR(J)=Y(J)-B*X(J)
11    CONTINUE
      CALL SORT_data(NDATA,ARR)
      AA=0.5*(ARR(NML)+ARR(NMH))
      SUM=0.
      ABDEV=0.
      DO 12 J=1,NDATA
        D=Y(J)-(B*X(J)+AA)
        ABDEV=ABDEV+ABS(D)
        SUM=SUM+X(J)*SIGN(1.0,D)
12    CONTINUE
      ROFUNC=SUM
      RETURN
      END
      SUBROUTINE SORT_data(N,RA)
      DIMENSION RA(N)
      L=N/2+1
      IR=N
10    CONTINUE
        IF(L.GT.1)THEN
          L=L-1
          RRA=RA(L)
        ELSE
          RRA=RA(IR)
          RA(IR)=RA(1)
          IR=IR-1
          IF(IR.EQ.1)THEN
            RA(1)=RRA
            RETURN
          ENDIF
        ENDIF
        I=L
        J=L+L
20      IF(J.LE.IR)THEN
          IF(J.LT.IR)THEN
            IF(RA(J).LT.RA(J+1))J=J+1
          ENDIF
          IF(RRA.LT.RA(J))THEN
            RA(I)=RA(J)
            I=J
            J=J+J
          ELSE
            J=IR+1
          ENDIF
        GO TO 20
        ENDIF
        RA(I)=RRA
      GO TO 10
       RETURN
      END
      Subroutine tests ( AA_half_noutlayer,BB_half_noutlayer,
     *AA_noutlayer,
     *BB_noutlayer,Sigma_half_one,Sigma_half_two,Sigma_all_one,
     *Sigma_all_two,Average_allwaves,standard_allwaves,log_standard,
     *iscan,idata,A,B,Siga,Sigb,
     *SCAT_ANGLE_LAND,numpoints_inlayer,numpoints_half_inlayer,
     *CHISQ_noutlayer,CHISQ_half_noutlayer,CHISQ,Flag_full,
     *diff_full_half,Co_variance_noutlayer,Co_variance_noutlayer_half,
     *Co_variance,chisq_for_flag,diff_critic_ref,surface)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C              This subroutine tests the array that should be used for fitting
C!INPUT PARAMETERS:
C     AA_half_noutlayer        Intercept till average
C    BB_half_noutlayer         Slope till average
C    AA_noutlayer                Intercept for whole array
C    BB_noutlayer                 Slope for whole array
C   Sigma_half_one              Error in Intercept till average
C   Sigma_half_two              Error in Slope till average
C    Sigma_all_one               Error in Intercept for whole array
C    Sigma_all_two               Error in Slope for whole array
C    Average_allwaves          Average values for whole array
C   standard_allwaves           Standard devaition for whole array
C   log_standard                   Standard devaition for log of whole array
C    idata                               Index for 10*10 array
C!OUTPUT PARAMETERS:
C
C                       A                   Final Intercept used after tests
c                        B                   Final Slope used after tests
C                  Siga                   Final error in Intercept used after tests
C                  Sigb                   Final error in Slope used after tests
C                  CHISQ
C
C!REVISION HISTORY:
CDeveloped 4/2001
C
C!TEAM-UNIQUE HEADER:
C
C 
C
C!DESIGN NOTES:
C
C!END
C-----------------------------------------------------------------------

        IMPLICIT NONE
         SAVE
         INCLUDE 'mod04.inc'
         Integer iscan, idata,QCONTROL_land,Flag_full
         Real   AA_half_noutlayer,BB_half_noutlayer,AA_noutlayer,
     *BB_noutlayer,Sigma_half_one,Sigma_half_two,Sigma_all_one,
     *Sigma_all_two,Average_allwaves(3),standard_allwaves(3)
        Real  log_standard,A,B,Siga,Sigb,SCATMODIS,SCAT_ANGLE_LAND
        Real THRSHOLD1,THRSHOLD2,diff_full_half
        Integer numpoints_inlayer,numpoints_half_inlayer
        Real CHISQ_noutlayer,CHISQ_half_noutlayer,CHISQ
        real Co_variance_noutlayer,Co_variance_noutlayer_half
        real Co_variance,chisq_for_flag,surface,diff_critic_ref
         A=-99.0
         B=-99.0
         Siga=-99.0
         Sigb=-99.0
         Co_variance=-99.0
         CHISQ=-99.0
         Flag_full=0
         chisq_for_flag=-99.0
        diff_critic_ref=-99.0
        diff_full_half=ABS(AA_half_noutlayer - AA_noutlayer)
        
        diff_critic_ref=
     *   ABS((AA_noutlayer/(1.-BB_noutlayer/Surface))-
     *     (AA_half_noutlayer/(1.-BB_half_noutlayer/Surface)))
              
c   test 0
       if(( 2*log_standard)/Average_allwaves(3) .ge .0.5) then
c   test 1
       if(standard_allwaves(3) .ge. 0.01) then
c   test 2
       IF(AA_half_noutlayer  . lt.  AA_noutlayer .and.
     *     numpoints_inlayer .gt. 25 ) then
c   test 4
        if(Sigma_all_one .lt. 0.01) then
          if( AA_noutlayer .gt.0 .and. BB_noutlayer .gt.0)then
               A=AA_noutlayer
               B=BB_noutlayer
               siga=Sigma_all_one
               sigb=Sigma_all_two
               CHISQ=CHISQ_noutlayer
               Flag_full=1
            chisq_for_flag=sqrt(CHISQ_noutlayer/(numpoints_inlayer-1))
              Co_variance=Co_variance_noutlayer
         endif
      endif
c else for test2
       ELSE
c test 3
c        
         if (ABS(AA_half_noutlayer - AA_noutlayer) .lt. 0.003 .and.
     *     numpoints_inlayer .gt. 25 )  then
c test 4
           if(Sigma_all_one .lt. 0.01) then
               if( AA_noutlayer .gt.0 .and. BB_noutlayer .gt.0)then
               A=AA_noutlayer
               B=BB_noutlayer
               siga=Sigma_all_one
               sigb=Sigma_all_two
               CHISQ=CHISQ_noutlayer
               Flag_full=1
               Co_variance=Co_variance_noutlayer
           chisq_for_flag=sqrt(CHISQ_noutlayer/(numpoints_inlayer-1))
            endif
         endif
      
c else for test3 to use half layer
      else
       if(Sigma_half_one .lt. 0.01) then
        if( AA_noutlayer .gt.0 .and. BB_noutlayer .gt.0 .and.
     *    numpoints_half_inlayer.gt. 25 )  then
               A= AA_half_noutlayer
               B= BB_half_noutlayer
               siga=Sigma_half_one
               sigb=Sigma_half_two
               CHISQ=CHISQ_half_noutlayer 
               Co_variance=Co_variance_noutlayer_half
               Flag_full=0
            chisq_for_flag=
     *sqrt(CHISQ_half_noutlayer/(numpoints_half_inlayer-1))
           endif
        endif
c   Endif for test3
       endif
c   endif test 2
      endif
cc   else for test 1
      endif
cc   else for test 0
      else
       if (ABS(AA_half_noutlayer - AA_noutlayer) .lt. 0.003.and.
     *     numpoints_inlayer .gt. 25 ) then
          if( AA_noutlayer .gt.0 .and. BB_noutlayer .gt.0)then
               A=AA_noutlayer
               B=BB_noutlayer
               siga=Sigma_all_one
               sigb=Sigma_all_two
               CHISQ=CHISQ_noutlayer
               Flag_full=1
               Co_variance=Co_variance_noutlayer
             chisq_for_flag=sqrt(CHISQ_noutlayer/(numpoints_inlayer-1))
             endif
         endif
       endif
   

      Return
      end
C*****************************************************************

      Subroutine Fill_QAflag_CritRef_land(QCONTROL_land_wav1,
     *QCONTROL_land_wav2, SDS_QCONTROL_CritRef_land,Idata)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION: This subroutine sets the array for quality control flags
C
C!INPUT PARAMETERS:
C              QA_Flag_CritRef_land         Quality flag
C               Idata                                     Index of 10*10 box
C
C!OUTPUT PARAMETERS:
C              SDS_QCONTROL_CritRef_land      HDF array for quality control
C
C!REVISION HISTORY:
C $Log: Process_land_V2.f,v $
c 04/2001
c 
C
C!TEAM-UNIQUE HEADER:
C
C!REFERENCES AND CREDITS Shana Mattoo
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------

       IMPLICIT NONE
       SAVE

       INCLUDE 'mod04.inc'

       Integer QA_Flag_CritRef_land(12),idata,ii 
       Integer QCONTROL_land_wav1,QCONTROL_land_wav2
       BYTE   SDS_QCONTROL_CritRef_land(QA_Land,NUMCELLS),QA_Temp


c Product Quality & Retrieval Processing QA flags over ocean
C Quality of 3 and 4 ( Average solution)are repeated tion have
c as best and average solution have same quality.
c

c        QA_Flag_CritRef_land(1)=0     NO retrieval ( Not useful)
c        QA_Flag_CritRef_land(1)=1     retrieval    (  useful)
c
c    For all non retrieval boxes QA_Flag_CritRef_land is set to zero to indicate
c    not useful and QA_Flag_CritRef_land(5) is set to values of QCONTROL_land_land to
c    indicate the cause of Retrieval. QA_Flag_CritRef_land(6) is set to 11 for
c    no retrieval
         If ( QCONTROL_land_wav1.le. 0) then
           QA_Flag_CritRef_land(1)=0
           QA_Flag_CritRef_land(2)=0
         Else
            QA_Flag_CritRef_land(1)=1
         endif
         If ( QCONTROL_land_wav2.le. 0) then
           QA_Flag_CritRef_land(3)=0
           QA_Flag_CritRef_land(4)=0
         Else
            QA_Flag_CritRef_land(3)=1 
         endif
c  All retrieval boxes
c Estimated Quality of aerosol parameters
c        QA_Flag_CritRef_land(2)=3     very Good
c        QA_Flag_CritRef_land(2)=2     Good
c        QA_Flag_CritRef_land(2)=1     Marginal
c        QA_Flag_CritRef_land(2)=0     Bad
c        QA_Flag_CritRef_land(4)=3     very Good
c        QA_Flag_CritRef_land(4)=2     Good
c        QA_Flag_CritRef_land(4)=1     Marginal
c        QA_Flag_CritRef_land(4)=0     Bad

c
cIF  QCONTROL_landis 0( see doc.)quality of retrieval  is very good set control for
cboth wave .46 and 66
c
       QA_Flag_CritRef_land(2)= QCONTROL_land_wav1
       QA_Flag_CritRef_land(4)= QCONTROL_land_wav2
          

      QA_Temp=0
      CALL BYTE_SET(QA_Flag_CritRef_land(1),0,QA_Temp)
      CALL BYTE_SET(QA_Flag_CritRef_land(2),1,QA_Temp)
      CALL BYTE_SET(QA_Flag_CritRef_land(3),4,QA_Temp)
      CALL BYTE_SET(QA_Flag_CritRef_land(4),5,QA_Temp)
C  End of 1 byte

        SDS_QCONTROL_CritRef_land(1,IDATA)=QA_Temp

       Return
       end
C**********************************************************************
        SUBROUTINE Get_Rayleigh(INT,THET0,THE,PHI,LTAU,
     *  NUMWAV,LTHET0,LTHE,LPHI,MTHET0,MTHET,MDPHI,
     *  Rayleigh_look,NEW, NEWINT,NEWINT2,
     * NEWN,NEWN2,FINT)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C
C          This subroutine derives the optical thicknesses for
C          both red (0.66 micron) and blue (0.47 micron) channels.
C
C!INPUT PARAMETERS:
C
C          NUMWAV   Number of wavelengths in lookup table.
C          LTHET0   Number of solarzenith angle in lookup table.
C          LTHE     Number of view angle in lookup table.
C          LPHI     Number of azimuths in lookup table.
C          LTAU     Number of optical depths in lookup table.
C          THET0    Solar Angle
C          THE      Satellite View angle
C          PHI      Azimuth
C          SBAR     Sbar
C          INT      Reflectance
C          NEW      Temporary arrays
C          NEWN     Temporary arrays
C          NEWINT   Temporary arrays
C          NFDOWN   Temporary arrays
C          NFUP     Temporary arrays
C          NEWINT2  Temporary arrays
C          MTHET0   MODIS input solar zenith angle
C          MTHET    MODIS input satellite view angle
C          MDPHI    MODIS input relative azimuth angle
C
C!OUTPUT PARAMETERS:
C
C    Rayleigh_look  Reflactance for Pure Rayleigh taua=0.0
C!REVISION HISTORY:
C 
C 4/23/01  
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by MODIS Aerosol/Solar Water Vapor Retrieval Team
C modified 4/23/01 GSFC, Greenbelt, MD
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
C
      IMPLICIT NONE
      INTEGER I,IBLUE,ICOM,IERROR,IJK,IPHI,IRED,ITAU,JTABLE,INDEX,ITABLE
      INTEGER ITHE,ITHET0,IWAV,LOPT,LPHI,LTAU,LTHE,LTHET0,NUMWAV
      INTEGER IND_THET0_1,IND_THET0_2,IND_THET_1,IND_THET_2,IND_PHI

      REAL Rayleigh_look(NUMWAV)
      REAL THET0(LTHET0),THE(LTHE),PHI(LPHI)
      REAL INT(LTHET0,LPHI,LTHE,LTAU,NUMWAV)
      REAL NEW(LPHI)
      REAL NEWINT(LPHI,LTHE,LTAU),NEWINT2(LTHE,LTAU)
      REAL NEWN(LPHI),NEWN2(LPHI),FINT(LTHE)
      REAL MTHET0,MTHET,MDPHI
      REAL ARG(1),YY(1),TAUA(3,2),MINT(3,2)

      SAVE

     
C Interpolation of radiance and flux incident upon solar
C zenith angle
C

      DO 9995 IWAV=1,NUMWAV

      DO 240 IPHI=1,LPHI
        NEW(IPHI)=0.0
        NEWN(IPHI)=0.0
        NEWN2(IPHI)=0.0
 240  CONTINUE

      DO 104 ITAU=1,LTAU
        DO 105 ITHE=1,LTHE
          DO 106 IPHI=1,LPHI
            DO 107 ITHET0=1,LTHET0
              NEW(ITHET0)=INT(ITHET0,IPHI,ITHE,ITAU,IWAV)
 107        CONTINUE

            IJK=1

C
C Call to subroutine INTERP_LAND to interpolate INT.
C If mthet0 is out of range of values of thet0 subroutine
C INTERP_LAND returns the value of LOPT=0
C

            ARG(1)=MTHET0
            CALL LOCATE_OPT(THET0,LTHET0,MTHET0,IND_THET0_1,JTABLE,LOPT)
            CALL LININT_OPT(MTHET0,YY(1),THET0,NEW,LTHET0,IND_THET0_1)
            NEWINT(IPHI,ITHE,ITAU)=YY(1)


 106      CONTINUE
 105    CONTINUE
 104  CONTINUE

C
C Interpolation of radiance upon measured azimuth angle.
C NEWINT is interpolated for measured mphi to derive varaiable
C NEWINT2 (formly NEWNEW).
C
      DO 241 I=1,LPHI
        NEW(I)=0.0
        NEWN(I)=0.0
 241  CONTINUE

      DO 108 ITAU=1,LTAU
        DO 109 ITHE=1,LTHE

          DO 110 IPHI=1,LPHI
            NEW(IPHI)=NEWINT(IPHI,ITHE,ITAU)
 110      CONTINUE

          IJK=3

C
C Call to subroutine INTERP_LAND to interpolate on azimuth angle.
C If mphi is out of range subroutine INTERP_LAND returns
C the value of LOPT=0
C

          ARG(1)=MDPHI
          CALL LOCATE_OPT(PHI,LPHI,MDPHI,IND_PHI,JTABLE,LOPT)
          CALL LININT_OPT(MDPHI,YY(1),PHI,NEW,LPHI,IND_PHI)
          NEWINT2(ITHE,ITAU)=YY(1)

 109    CONTINUE
 108  CONTINUE

C
C Interpolation of radiance and transmission/pi at surface
C satellite zenith angle NEWINT2 is interpolated to get new
C varaiable FINT. Transmission factor PIT is interpolated to
C derive FT.
C

      DO 242 I=1,LPHI
        NEW(I)=0.0
        NEWN(I)=0.0
 242  CONTINUE
c For rayleigh only
      DO 111 ITAU=1,1
        DO 112 ITHE=1,LTHE
          NEW(ITHE)=NEWINT2(ITHE,ITAU)
 112    CONTINUE

C
C Call to subroutine INTERP_LAND to interpolate radiance on
C satellite zenith angle. If it is out of range, subroutine
C INTERP_LAND returns the value of lopt=0
C

        ARG(1)=MTHET
         CALL LOCATE_OPT(THE,LTHE,MTHET,IND_THET_1,JTABLE,LOPT)
         CALL LININT_OPT(MTHET,YY(1),THE,NEW,LTHE,IND_THET_1)
        FINT(ITAU)=YY(1)

 111  CONTINUE
       Rayleigh_look(iwav)=FINT(1)

9995   CONTINUE

       RETURN
       END
         Subroutine Flag_analysis(Rayleigh_look,SCAT_ANGLE_LAND,Path_rad,
     *      Error_Pathrad,critic_ref,error_critic,index_error_flags,
     *     Flag_full,diff_full_half,weighting_critical,weighting_path,
     *     Qa_Flag_final,chisq_for_flag,diff_critic_ref)
C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C
C          This subroutine  Uses combination of tests to derive the weighting and
C          final Qa flag for the Quality Assurance
C
C!INPUT PARAMETERS:
C
C          Rayleigh_look    Rayleigh from Lookup table
C          SCAT_ANGLE_LAND  Scattering angle
C          Path_rad         Path Radiance derived
C          Error_Pathrad    Error in path radiance
C          critic_ref       Critical Reflactance
C          error_critic     Error in Critical Ref
C          Flag_full       Flag if Full or half data used
C          diff_full_half  Difference between half and full fit
C 
C
C!OUTPUT PARAMETERS:
C      index_error_flags     Index of different errors
C      weighting_critical    Weighting critic_ref based on the Errors
C      weighting_path        Weighting path radiance based on the Errors
C      Qa_Flag_final         Final flag for QA to determine Quality of data
C!REVISION HISTORY:
C 
C 4/23/01  
C
C Updated code to comply with most MODIS software standards.
C
C!TEAM-UNIQUE HEADER:
C
C Developed by Shana Mattoo
C modified 4/23/01 GSFC, Greenbelt, MD
C
C!DESIGN NOTES:
C
C!END
C----------------------------------------------------------------------
         IMPLICIT NONE
         SAVE
         INCLUDE 'mod04.inc'
       real Rayleigh_look(Land_Sol1)
       real THRSHOLD1,THRSHOLD2,SCATMODIS,Diff_path,Path_rad(Land_Sol1)
       real Error_Pathrad(Land_Sol1),critic_ref(Land_Sol1)
       real error_critic(Land_Sol1)
       real diff_full_half(Land_Sol1),SCAT_ANGLE_LAND
       integer iwav,ierr_index,num_ierr_index
       parameter(num_ierr_index=11)
       integer index_error_flags(Land_Sol1,num_ierr_index)
       integer Flag_full(Land_Sol1)
       Integer sum_index(Land_Sol1),weighting_critical(Land_Sol1)
       integer Qcontrol_crit_land(Land_Sol1) 
       integer weighting_path(Land_Sol1)
       integer Qcontrol_path_land(Land_Sol1)
       integer Qa_Flag_final(Land_Sol1)
       real chisq_for_flag(Land_Sol1),diff_critic_ref(Land_Sol1)
       
c  TEST for Particle Size
         SCATMODIS=SCAT_ANGLE_LAND
       IF(SCATMODIS .LT.60.0)SCATMODIS=60.0
       IF(SCATMODIS .GT.170.0)SCATMODIS=170.0
       IF(SCATMODIS.GE.60.0.AND.SCATMODIS.LT.150.0) THEN
        THRSHOLD1=1.1+0.0011*(SCATMODIS-60.0)
        THRSHOLD2=0.9+0.0011*(SCATMODIS-60.0)
      ELSE IF(SCATMODIS.GE.150.0.AND.SCATMODIS.LE.180.0) THEN
        THRSHOLD1=1.2-0.0033*(SCATMODIS-150.0)
        THRSHOLD2=1.0-0.0033*(SCATMODIS-150.0)
      ENDIF
           Diff_path=(Path_rad(2)-Rayleigh_look(2))/
     *     (Path_rad(1)-Rayleigh_look(1))

         Do iwav=1,Land_Sol1
         Do ierr_index=1,num_ierr_index
         index_error_flags(iwav,ierr_index)=0
         enddo
         enddo
         Do iwav =1,2
c   ERRRORS>>>>>>>>>>
         If(critic_ref(iwav) .gt.0.0 .and.critic_ref(iwav) .le.1.)
     *   index_error_flags(iwav,1)= (2**(0))
         If(Error_Pathrad(iwav) .lt.0.005)
     * index_error_flags(iwav,2)=(2**(1))
c one of the wavelength has no values
         If(Diff_path.lt.0) index_error_flags(iwav,3)=
     *                           (2**(2))
c Large Mode
        if(Diff_path.gt.THRSHOLD1) index_error_flags(iwav,4)= 
     *                                 (2**(3))
c Mixed Mode
         if(Diff_path.lt.THRSHOLD1 .and.Diff_path.GT.THRSHOLD2)
     *       index_error_flags(iwav,5)= (2**(4))
c Small Mode
         if(Diff_path.lt.THRSHOLD2) index_error_flags(iwav,6)= 
     *                                  (2**(5))
c
        If( error_critic(iwav) .lt.0.02)
     * index_error_flags(iwav,7)=(2**(6))

      If(  error_critic(iwav) .lt.0.01)
     * index_error_flags(iwav,8)=(2**(7))

        If(Error_Pathrad(iwav) .lt.0.0005)
     * index_error_flags(iwav,9)=(2**(8))
  

       If(diff_full_half(iwav). lt.003)index_error_flags(iwav,10)= 
     *                                  (2**(9))

      If(Flag_full(iwav) .eq.1)index_error_flags(iwav,11)= 
     *                                  (2**(10))
c Enddo for wavelength
       enddo
c For critical reflactance flags sum first 8 tests  
        Do iwav=1,2
         weighting_critical(iwav)=0
         sum_index(iwav)=0
        Do ierr_index=1,num_ierr_index-3
          sum_index(iwav)=sum_index(iwav)+
     *   index_error_flags(iwav,ierr_index)
        Enddo
         IF(sum_index(iwav) .EQ. 227) THEN
          weighting_critical(iwav)=12
          Qcontrol_crit_land(iwav)=3
         Elseif(sum_index(iwav) .EQ.225) THEN
          weighting_critical(iwav)=10
          Qcontrol_crit_land(iwav)=3
          Elseif(sum_index(iwav) .EQ.99) THEN
          weighting_critical(iwav)=8
          Qcontrol_crit_land(iwav)=2
          Elseif(sum_index(iwav) .EQ.97) THEN
          weighting_critical(iwav)=6
          Qcontrol_crit_land(iwav)=2
          Elseif(sum_index(iwav) .EQ.81) THEN
          weighting_critical(iwav)=4
          Qcontrol_crit_land(iwav)=1
          Elseif(sum_index(iwav) .EQ.211) THEN
          weighting_critical(iwav)=4
          Qcontrol_crit_land(iwav)=1
          elseif(sum_index(iwav) .EQ.83) THEN
          weighting_critical(iwav)=4
          Qcontrol_crit_land(iwav)=1
          elseif(sum_index(iwav) .EQ.209) THEN
          weighting_critical(iwav)=4
          Qcontrol_crit_land(iwav)=1
          elseif(sum_index(iwav) .EQ.73) THEN
          weighting_critical(iwav)=3
          Qcontrol_crit_land(iwav)=0
          elseif(sum_index(iwav) .EQ.203) THEN
          weighting_critical(iwav)=3
          Qcontrol_crit_land(iwav)=0
          elseif(sum_index(iwav) .EQ.75) THEN
          weighting_critical(iwav)=3
          Qcontrol_crit_land(iwav)=0
          elseif(sum_index(iwav) .EQ.201) THEN
          weighting_critical(iwav)=3
          Qcontrol_crit_land(iwav)=0
          elseif(sum_index(iwav) .EQ.69) THEN
          weighting_critical(iwav)=2
          Qcontrol_crit_land(iwav)=0
          elseif(sum_index(iwav) .EQ.199) THEN
          weighting_critical(iwav)=2
          Qcontrol_crit_land(iwav)=0
          elseif(sum_index(iwav) .EQ.71) THEN
          weighting_critical(iwav)=2
          Qcontrol_crit_land(iwav)=0
          elseif(sum_index(iwav) .EQ.197) THEN
          weighting_critical(iwav)=2
          Qcontrol_crit_land(iwav)=0
          elseif(mod(sum_index(iwav),2).EQ.0) THEN
          weighting_critical(iwav)=1
          Qcontrol_crit_land(iwav)=0
          elseif(mod(sum_index(iwav),2).NE.0) THEN
             IF(sum_index(iwav) .le.63) then
              weighting_critical(iwav)=1
              Qcontrol_crit_land(iwav)=0
             Endif
          Endif
           weighting_critical(iwav)= weighting_critical(iwav)*10
          if( diff_critic_ref(iwav) .le.0.01)weighting_critical(iwav)=
     *     weighting_critical(iwav)+3
          if( chisq_for_flag(iwav) .le.0.005)weighting_critical(iwav)=
     *     weighting_critical(iwav)+5
c  change to -ve for test11 ,-weighting means half fit used else full
          if(index_error_flags(iwav,11) .eq.0)
     *     weighting_critical(iwav)=-weighting_critical(iwav)
c enddo for wave
        enddo
c For Path radiance 
        Do iwav=1,2
         weighting_path(iwav)=0
         sum_index(iwav)=0
        Do ierr_index=2,6
          sum_index(iwav)=sum_index(iwav)+
     *   index_error_flags(iwav,ierr_index)
        Enddo
          Do ierr_index=9,10
          sum_index(iwav)=sum_index(iwav)+
     *   index_error_flags(iwav,ierr_index)
          enddo
          
         IF(sum_index(iwav) .EQ. 802) THEN
          weighting_path(iwav)=12
         Qcontrol_path_land(iwav)=3
         Elseif(sum_index(iwav) .EQ.546) THEN
          weighting_path(iwav)=10
         Qcontrol_path_land(iwav)=3
          Elseif(sum_index(iwav) .EQ.290) THEN
          weighting_path(iwav)=8
         Qcontrol_path_land(iwav)=3
          Elseif(sum_index(iwav) .EQ.34) THEN
          weighting_path(iwav)=7
         Qcontrol_path_land(iwav)=2
          Elseif(sum_index(iwav) .EQ.544) THEN
          weighting_path(iwav)=6
         Qcontrol_path_land(iwav)=2
          Elseif(sum_index(iwav) .EQ.32) THEN
          weighting_path(iwav)=5
         Qcontrol_path_land(iwav)=1
          elseif(sum_index(iwav) .EQ.18) THEN
          weighting_path(iwav)=4
         Qcontrol_path_land(iwav)=1
          elseif(sum_index(iwav) .EQ.530) THEN
          weighting_path(iwav)=4
         Qcontrol_path_land(iwav)=1
          elseif(sum_index(iwav) .EQ.786) THEN
          weighting_path(iwav)=4
         Qcontrol_path_land(iwav)=1
          elseif(sum_index(iwav) .EQ.274) THEN
          weighting_path(iwav)=4
         Qcontrol_path_land(iwav)=1
          elseif(sum_index(iwav) .EQ.10) THEN
          weighting_path(iwav)=2
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.522) THEN
          weighting_path(iwav)=2
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.778) THEN
          weighting_path(iwav)=2
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.266) THEN
          weighting_path(iwav)=2
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.6) THEN
          weighting_path(iwav)=1
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.518) THEN
          weighting_path(iwav)=1
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.774) THEN
          weighting_path(iwav)=1
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.262) THEN
          weighting_path(iwav)=1
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.0) THEN
          weighting_path(iwav)=0
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.16) THEN
          weighting_path(iwav)=0
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.8) THEN
          weighting_path(iwav)=0
         Qcontrol_path_land(iwav)=0
          elseif(sum_index(iwav) .EQ.4) THEN
          weighting_path(iwav)=0
         Qcontrol_path_land(iwav)=0
          Endif
            weighting_path(iwav)= weighting_path(iwav)*10
         if( diff_critic_ref(iwav) .le.0.01)weighting_path(iwav)=
     *    weighting_path(iwav)+3
          if( chisq_for_flag(iwav) .le.0.005)weighting_path(iwav)=
     *     weighting_path(iwav)+5
c  change to -ve for test11 ,-weighting means half fit used else full
          if(index_error_flags(iwav,11) .eq.0)
     *     weighting_path(iwav)=-weighting_path(iwav)
c enddo for wave
        enddo
        do iwav=1,2
        Qa_Flag_final(iwav)=(Qcontrol_path_land(iwav)+
     *                       Qcontrol_crit_land(iwav))/2
        enddo

       Return
       end












